// CalcInfo.cpp: implementation of the CCalcInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "CalcInfo.h"

#include <qdom.h>

#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CCalcInfo::CCalcInfo(QObject* parent, CasioModel nCalcType)
: QObject(parent)
{
	m_nCalcModel = nCalcType;
	m_hFile      = INVALID_HANDLE_VALUE;
}

//-----------------------------------------------------------------------------------
CCalcInfo::~CCalcInfo()
{
	CloseCalcFile();
}

//-----------------------------------------------------------------------------------
void CCalcInfo::CloseCalcFile()
{
	if(m_hFile != INVALID_HANDLE_VALUE) {
		QDomDocument doc("CalcInfo");		
		QDomElement e = doc.createElement("Informations");
		doc.appendChild(e);		
		for(int i  = 0; i < 7; i++) {
			e.setAttribute(QString("D%1").arg(i), QString::number(m_nCounters[i]));
		}
		e.setAttribute("CasioModel", QString::number(m_nCalcModel));

		QString s = doc.toString();
		SeekDesc(m_hFile, 0);
		WriteDesc(m_hFile, s, s.length());
		SetFileSize(m_hFile, s.length());
	}
	CloseDesc(m_hFile);
	m_sCalcFile  = QString::null;
	m_nCalcModel = CM_INVALID;
	memset(m_nCounters, 0, sizeof(m_nCounters));
}

//-----------------------------------------------------------------------------------
CasioModel CCalcInfo::GetCalcModel() const
{
	return m_nCalcModel;
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::IsAFX2LIKE() const
{
	return (m_nCalcModel & CT_G100_LIKE) && (m_nCalcModel != CM_INVALID);
}

//-----------------------------------------------------------------------------------
QString CCalcInfo::GetCalcName() const
{
	return GetCalcName(m_nCalcModel);
}

//-----------------------------------------------------------------------------------
QString CCalcInfo::GetCalcName(CasioModel nCalcModel)
{
	switch(nCalcModel)
	{
	case CM_G100:
		return tr("Graph100");
	case CM_G100_PLUS:
		return tr("Graph100+");
	case CM_OTHER:
		return tr("Autre Casio");
	case CM_GRAPH25:
		return tr("Graph 25");
	case CM_GRAPH30:
		return tr("Graph 30");
	case CM_GRAPH60:
		return tr("Graph 60");
	case CM_GRAPH65:
		return tr("Graph 65");
	case CM_GRAPH80:
		return tr("Graph 80");
	case CM_CLASSPAD:
		return tr("ClassPad");
	default:
		return tr("Invalide");
	}
}

//-----------------------------------------------------------------------------------
QString CCalcInfo::GetCalcType(CasioType nCalcType)
{
	switch(nCalcType)
	{
	case CT_G100_LIKE:
		return tr("Graph100(+)");
	case CT_GRAPH_LIKE:
		return tr("Graph25-85");
	case CT_CLASSPAD:
		return tr("ClassPad");
	default:
		return tr("Invalide");
	}
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::SetCalcModel(CasioModel nCalcModel)
{
	if(m_hFile == INVALID_HANDLE_VALUE) {
		return false;
	}
	if(m_nCalcModel != nCalcModel) {
		m_nCalcModel = nCalcModel;
		if(!IsAFX2LIKE()) {
			memset(m_nCounters, 0, sizeof(m_nCounters));
		}
		emit InformationChanged();
	}
	return true;
}

//-----------------------------------------------------------------------------------
const QString& CCalcInfo::GetCalcDir() const
{
	return m_sCalcDir;
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::SetCalcDir(const QString& sDir)
{
	return OpenCalcFile(sDir, m_sCalcFile);
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::SetCalcFile(const QString& sCalc)
{
	return OpenCalcFile(m_sCalcDir, sCalc);
}

//-----------------------------------------------------------------------------------
const QString& CCalcInfo::GetCalcFile() const
{
	return m_sCalcFile;
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::OpenCalcFile(const QString& sCalcDir, const QString& sCalcFile)
{
	if(sCalcDir.isEmpty()) {
		return false;
	}
	if(m_sCalcDir == sCalcDir && m_sCalcFile == sCalcFile) {
		return true;
	}
	if(sCalcFile.isEmpty()) {
		CloseCalcFile();
		m_sCalcDir = sCalcDir;
		emit InformationChanged();
		return true;
	}
	MkDir(sCalcDir);//S'assure que le dernier dossier est bien prsent
	QString sPath = sCalcDir + '/' + sCalcFile;
	HND h = OpenDesc(sPath, READWRITE, OPEN_ALWAYS);
	if(h == INVALID_HANDLE_VALUE) {
		return false;
	}
	int nCounters[7];
	CasioModel nCalcModel = CM_G100;
	int nSize = FileSize(h);
	bool bOk  = (nSize == 0);
	memset(nCounters, 0, sizeof(m_nCounters));
	if(nSize > 0) {
		char* pBuffer = new char[nSize + 1];
		if(pBuffer != NULL) {
			ReadDesc(h, pBuffer, FileSize(h));
			pBuffer[nSize] = '\0';
			QDomDocument doc;
			if(doc.setContent(QString(pBuffer)))	{
				QDomElement e = doc.documentElement();
				if(!e.isNull())	{
					nCalcModel = (CasioModel)e.attribute("CasioModel", QString::number(CM_G100)).toInt();
					if(nCalcModel == CM_G100 || nCalcModel == CM_G100_PLUS) {
						for(int i  = 0; i < 7; i++)	{
							QString s = e.attribute(QString("D%1").arg(i));
							nCounters[i] = s.toInt();
						}
					}
				}
				bOk = (nCalcModel != CM_INVALID);
			}
			delete[] pBuffer;
		}
	}
	if(bOk) {
		CloseCalcFile();
		m_hFile      = h;
		m_sCalcDir   = sCalcDir;
		m_sCalcFile  = sCalcFile;
		m_nCalcModel = nCalcModel;
		memcpy(m_nCounters, nCounters, sizeof(m_nCounters));
		emit InformationChanged();
	}
	else {
		CloseDesc(h);
	}
	return bOk;
}

//-----------------------------------------------------------------------------------
int CCalcInfo::GetCounter(int nDrive) const
{
	if(nDrive >= 0 && nDrive < 7) {
		return m_nCounters[nDrive];
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::SetCounter(int nDrive, int nValue)
{
	if(nDrive >= 0 && nDrive < 7 && nValue > 0 && m_hFile != INVALID_HANDLE_VALUE
		&& IsAFX2LIKE()) {
		m_nCounters[nDrive] = nValue;
		emit InformationChanged();
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
bool CCalcInfo::IncCounter(int nDrive)
{
	return SetCounter(nDrive, GetCounter(nDrive) + 1);
}
