// CheckFileWindow.cpp: implementation of the CheckFileWindow class.
//
//////////////////////////////////////////////////////////////////////

#include "CheckFileWindow.h"

#include <qfileinfo.h>

#include <APIFunction.h>

#include "MessageBox.h"

//-----------------------------------------------------------------------------------
CheckFileWindow::CheckFileWindow(CONSTRUCTOR_PARAM, bool bForceReload, bool bAutoClose)
:CToolWindow(CALL_PARAM)
{
	m_nTimerId     = -1;
	m_bForceReload = bForceReload;
	m_bAutoClose   = bAutoClose;
	m_bFileDestroy = FALSE;
	GetMainWidget()->installEventFilter(this);
	StartChecking();
} 

//-----------------------------------------------------------------------------------
int	CheckFileWindow::LoadFrom(QDomElement& element)
{
	if(CToolWindow::LoadFrom(element) == 1) {
		QDomElement e = element.namedItem("CheckFileWindow").toElement();
		if(!e.isNull()) {
			QString sFileName = ConvertToFullPath(e.attribute("m_sFileName"));
			if(SetFileName(sFileName, FALSE, FALSE)) {
				return 1;
			}
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int	CheckFileWindow::SaveTo(QDomElement& element)
{
	if(CToolWindow::SaveTo(element) == 1)
	{
		QDomElement e = element.ownerDocument().createElement("CheckFileWindow");
		element.appendChild(e);
		e.setAttribute("m_sFileName", ConvertToRelativePath(m_sFileName));
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CheckFileWindow::SetFileName(const QString& sFileName, bool bEmitStateChanged, bool bShowMsgErr)
{
	if(sFileName == m_sFileName) {
		return TRUE;
	}
	if(!sFileName.isEmpty()) {
		QFileInfo qFileInfo(sFileName);
		if(!qFileInfo.exists())
		{
			if(bShowMsgErr){
				QString s = QObject::tr("Le fichier '%1' n'existe pas!")
					.arg(sFileName);
				CMessageBox::MsgCritical(GetMainWidget(), s);
			}
			return FALSE;
		}
	}
	m_sFileName    = sFileName;
	m_bFileDestroy = FALSE;
	ResetTime();
	if(bEmitStateChanged) {
		emit StateChanged();
	}
	DoShowCaption();
	return TRUE;
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::ResetTime()
{
	if(!m_sFileName.isEmpty()) {
		QFileInfo qFileInfo(m_sFileName);
		if(qFileInfo.exists()) {
			m_qTime = qFileInfo.lastModified().time();
		}
	}
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::DoShowCaption()
{
	if(!m_sFileName.isEmpty()) {
		const char* pFileName;
		GetNameOfFile(m_sFileName, pFileName);
		setCaption(caption() + QString(" - ") + pFileName);
	}
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::timerEvent(QTimerEvent* e)
{
	if(e->timerId() == m_nTimerId) {
		CheckFile();
	}
	else {
		CToolWindow::timerEvent(e);
	}
}

//-----------------------------------------------------------------------------------
bool CheckFileWindow::eventFilter(QObject* o, QEvent* e)
{
	if(o == GetMainWidget() && e->type() == QEvent::WindowActivate || e->type() == QEvent::Show) {
		CheckFile();
	}
	return CToolWindow::eventFilter(o, e);
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::CheckFile()
{
	static bool bLocked = FALSE;
	if(!m_sFileName.isEmpty() && m_nTimerId != -1 && !bLocked)
	{
		bLocked = TRUE;
		QFileInfo qFileInfo(m_sFileName);
		if(qFileInfo.exists())
		{
			QTime qTime = qFileInfo.lastModified().time();
			if(qTime != m_qTime)
			{
				m_qTime = qTime;
				if(m_bForceReload || CMessageBox::MsgQuestion(GetMainWidget(), QObject::tr("Le fichier '%1' a t modifi par un autre programme.\nVoulez-vous le recharger ?").arg(m_sFileName))) {
					if(DoReloadFile(true) < 1) {
						OnInvalidFile();
					}
				}
			}
			m_bFileDestroy = FALSE;
		}
		else
		{
			if(!m_bFileDestroy) {
				OnFileRemoved();
				m_bFileDestroy = TRUE;
			}
		}
		bLocked = FALSE;
	}
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::OnFileRemoved()
{
	QString s = QObject::tr("Le fichier '%1' n'existe plus!")
		.arg(m_sFileName);
	if(!m_bAutoClose || m_bForceReload) {
		CMessageBox::MsgCritical(GetMainWidget(), s);
		if(m_bAutoClose) {
			m_sFileName = QString::null;
			close();
		}
	}
	else{
		if(CMessageBox::MsgQuestion(GetMainWidget(), s + '\n' + QObject::tr("Voulez-vous fermer l'diteur ?"))) {
			m_sFileName = QString::null;
			close();
		}							
	}
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::OnInvalidFile()
{
	if(m_bAutoClose) {
		close();
	}
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::StopChecking()
{
	if(m_nTimerId != -1) {
		killTimer(m_nTimerId);
		m_nTimerId = -1;
	}
}

//-----------------------------------------------------------------------------------
void CheckFileWindow::StartChecking()
{
	if(m_nTimerId == -1) {
		ResetTime();
		m_nTimerId = startTimer(3000);
	}
}
