// SettingTabWidget.h: interface for the CSettingTabWidget class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _SETTINGTABWIDGET_H_
#define _SETTINGTABWIDGET_H_

#include <qwidget.h>
#include <qvaluelist.h>

#include <F100Qt.h>

class IMainWindow;

/*!
	\class CSettingTabWidget SettingTabWidget.h
	\brief Fentre qui contient des options
	\author Quaranta David
	\version 1.00
	
	\ingroup F100Qt
	Fentre pouvant tre ajoute dans un CSettingsManager pour permettre  l'utilisateur de pouvoir
	rgler des options.\n
	Cette fentre peut contenir des fentres filles qui seront visibles depuis la liste de CSettingsManager.\n
	Le titre d'une fentre, caption(), est utilis par CSettingsManager pour tre mis dans la liste.
*/
class DLLEXPORT CSettingTabWidget : public QWidget
{
	Q_OBJECT
public:
	/** Liste des enfants */
	typedef QValueList<CSettingTabWidget*> SettingTabList;
public:
	/** Constructeur
	 * @param[in] pMainWindow Fentre principale
	 * @param[in] sKey Cl pour indiquer le chemin de dpart de la sauvegarde, relative  celle de la fentre principale
	 * @param[in] bRemoveRecursive Vrai la restauration des donnes supprime les cls rcursivement pour dtruire les sous-cls
	 * @param[in] bShowInList Vrai si l'lment doit tre vu dans la liste lors de son ajout dans le CSettingsManager
	 */
	CSettingTabWidget(IMainWindow* pMainWindow, const QString& sKey, bool bRemoveRecursive = FALSE, bool bShowInList = TRUE);

	virtual ~CSettingTabWidget();

	/** @return Retourne la fentre principale de l'application */
	IMainWindow*    GetMainWindow() const;

	/** @return Le chemin de sauvegarde */
	const QString&  GetKey() const;
	
	/** Sauve les prfrences
	 * @return -1 en cas d'chec\n -2 Demande de redmarrage\n 0 non implment \n > 0 pour Ok
	 * @see WriteSettings(const QString&, const int&)
	 * @see WriteSettings(const QString&, const QString&)
	 */
	virtual int     DoSave() = 0;

	/** Rafraichir les options 
	 * @see ReadSettings(const QString&, int&, const int&)
	 * @see ReadSettings(const QString&, QString&, const QString&)
	 */
	virtual void    DoRefresh() = 0;

	/** Remet les options par dfaut
	 * @return > 0 pour Ok\n -2 pour redmarrer\n -1 erreur
	 */
	virtual int     RestoreDefaults();

	/** Initialisation des mots succeptibles d'tre traduit */
	virtual void    DoInitWords() = 0;

	/** @return La description de la fentre des options */
	virtual QString GetDescription() const = 0;

	/** @return Vrai si ses enfants doivent tre tris par ordre alphabtique.\n
	 * Par dfaut retourne VRAI.
	 */
	virtual bool    SortChildrenInList() const;

	/** @return La liste des fentres filles de cette fentre */
	const SettingTabList* GetChildren() const;
	
	/** @return La fentre parent de cette fentre ou NULL si elle n'a pas de parent */
	CSettingTabWidget* GetParent() const;

	/** @return Vrai si l'lment doit tre vu dans la liste lors de son ajout dans le CSettingsManager */
	bool            IsShownInList() const;
	
	/** Permet d'ajouter une fentre d'options
	 * @param[in] pChild Fentre  ajouter
	 */
	void AddChild(CSettingTabWidget* pChild);
	
	/** Permet d'enlever une fentre d'options
	 * @param[in] pChild Fentre  enlever
	 */
	void RemoveChild(CSettingTabWidget* pChild);

protected:

	/** Permet de sauver des donnes
	 * @param[in] sPath Chemin du paramtre. Ne doit pas commencer par '/'.
	 * @param[in] nNumber Nombre  sauver
	 * @remarks Les donnes seront stockes comme ceci : 'Racine'/Cl/pWindowType/\a sPath.
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	virtual bool WriteSettings(const QString& sPath, const int& nNumber);

	/** Permet de sauver des donnes
	 * @param[in] sPath Chemin du paramtre. Ne doit pas commencer par '/'.
	 * @param[in] sText Donnes
	 * @remarks Les donnes seront stockes comme ceci : 'Racine'/Cl/pWindowType/\a sPath.
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	virtual bool WriteSettings(const QString& sPath, const QString& sText);

	/** Permet de rcuprer des donnes sauver depuis WriteSettings(const QString&, const int&)
	 * @param[in] sPath Chemin du paramtre. Ne doit pas commencer par '/'.
	 * @param[out] nNumber Nombre lu
	 * @param[in] nDefault Nombre par dfaut si le paramtre n'est pas trouv
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	virtual bool ReadSettings(const QString& sPath, int& nNumber, const int& nDefault = 0) const;

	/** Permet de rcuprer des donnes sauver depuis WriteSettings(const QString&, const QString&)
	 * @param[in] sPath Chemin du paramtre. Ne doit pas commencer par '/'.
	 * @param[out] sText Texte lu
	 * @param[in] sDefault Texte par dfaut si le paramtre n'est pas trouv
	 * @return Retourne vrai en cas de succs, sinon faux
	 */
	virtual bool ReadSettings(const QString& sPath, QString& sText, const QString& sDefault = QString::null) const;

	/** Permet de lire les entres d'un rpertoire
	 * @param[in] sPath Chemin relatif
	 * @return La liste des sous-rpertoires du dossier
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour charger des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual QStringList  SettingsList(const QString& sPath) const;

	/** Permet de lire les rpertoires d'un rpertoire
	 * @param[in] sPath Chemin relatif
	 * @return La liste des sous-rpertoires du dossier
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour charger des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual QStringList  SubSettingsList(const QString& sPath) const;

private:
	/** Permet de dfinir le parent de cette fentre
	 * @param[in] pParent Parent
	 */
	void SetParent(CSettingTabWidget* pParent);

signals:
	/** Signal mit lorsque une fentre d'options est ajoute
	 * @param[in] pChild Fentre ajoute
	 */
	void ChildAdded(CSettingTabWidget* pChild);

	/** Signal mit lorsque une fentre d'options est retire
	 * @param[in] pChild Fentre retire
	 */
	void ChildRemoved(CSettingTabWidget* pChild);

private:
	CSettingTabWidget* m_pParent;
	SettingTabList*    m_pChildren;
	IMainWindow*       m_pMainWindow;
	QString            m_sKey;
	bool               m_bShowInList;
	bool               m_bRemoveRecursive;
};

#endif
