
#include "TransfertWindow.h"

#include <qapplication.h>
#include <qlayout.h>

#include <DriveAccess.h>
#include <CasioSystem.h>

#include "CalcInfo.h"
#include "language.h"
#include "MessageBox.h"

//-----------------------------------------------------------------------------------
CTransfertWindow::CTransfertWindow(CCalcInfo* pCalcInfo, QWidget* parent)
: QDialog(parent, 0, TRUE, WStyle_Customize | WStyle_DialogBorder | WStyle_Title),
  m_cmdCancel(this),
  m_lblEstimated(this, 0, TRUE),
  m_lblRemaining(this, 0, TRUE),
  m_lblElapsed(this, 0, TRUE),
  m_lblSize(this, 0, TRUE),
  m_lblInfo(this, 0, TRUE)
{
	resize(300, 140);

	m_pCalcInfo = pCalcInfo;

	QBoxLayout* pLayout = new QBoxLayout(this, QBoxLayout::TopToBottom);

	pLayout->addWidget(&m_lblInfo);
	pLayout->addWidget(&m_lblSize);
	pLayout->addWidget(&m_lblElapsed);
	pLayout->addWidget(&m_lblRemaining);
	pLayout->addWidget(&m_lblEstimated);
	pLayout->addWidget(&m_cmdCancel, 0, AlignCenter);

	m_lblEstimated.setAlignment(QLabel::AlignCenter);
	m_lblRemaining.setAlignment(QLabel::AlignCenter);
	m_lblElapsed.setAlignment(QLabel::AlignCenter);
	m_lblSize.setAlignment(QLabel::AlignCenter);
	m_lblInfo.setAlignment(QLabel::AlignCenter);

	m_cmdCancel.setText(CLanguage::GetWordTranslation(CLanguage::WO_CANCEL));
	connect(&m_cmdCancel, SIGNAL(clicked()), SLOT(CancelClick()));

	m_pProtocolOp = NULL;
	m_pProtocol   = NULL;
}

//-----------------------------------------------------------------------------------
CTransfertWindow::~CTransfertWindow()
{
	DeleteProtocol();
}

//-----------------------------------------------------------------------------------
CasioType CTransfertWindow::GetCasioType() const
{
	if(m_pProtocol != NULL) {
		return m_pProtocol->GetCasioType();
	}
	return CT_INVALID;
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::DeleteProtocol()
{
	if(m_pProtocol && m_pProtocolOp) {
		m_pProtocolOp(OP_DELETEINSTANCE, m_pProtocol);
		m_pProtocol = NULL;
	}
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::SetProtocol(Protocol_Op& pOp, IComDevice* pComDevice, int nSpeed, int nTotalSize)
{
	CloseComm(1);
	DeleteProtocol();
	m_pProtocolOp = pOp;
	if(m_pProtocolOp != NULL) {
		m_pProtocol = (IProtocol*)m_pProtocolOp(OP_CREATEINSTANCE, NULL);
	}
	if(m_pProtocol != NULL) {
		setCaption(tr("Initialisation..."));
		SetData(nTotalSize);
		m_cmdCancel.setEnabled(FALSE);
		show();
		qApp->processEvents();
		if(m_pProtocol->InitCom(pComDevice, nSpeed)) {
			return 1;
		}
	}
	ShowError();
	return -1;
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::SetTransfertMode(IProtocol::TRANSFERT nTransfert)
{
	if(m_pProtocol != NULL) {
		return m_pProtocol->SetTransfertMode(nTransfert);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::ShowError()
{
	if(m_pProtocol)
	{
		QString s = tr("Une erreur inconnue s'est produite!");
		QString sCheckComm = tr("Vrifiez que la calculette soit bien branche et en mode rception.");
		switch(m_pProtocol->GetLastError())
		{
		case IProtocol::OPENPORT:
			s = tr("Impossible d'ouvrir le port!");
			break;
		case IProtocol::GETPORT:
			s = tr("Impossible de lire l'tat du port!");
			break;
		case IProtocol::SETPORT:
			s = tr("Impossible de configurer le port!");
			break;
		case IProtocol::TIMEOUT:
			s = tr("Erreur de transmission! Aucune rponse de la calculatrice.") + '\n' + 
				sCheckComm + '\n' +
				tr("La transmission va tre arrte.");
			break;
		case IProtocol::NOINIT:
			s = tr("Il s'est produit une erreur lors de l'initialisation.") + '\n' +
				sCheckComm;
			break;
		case IProtocol::NOSEND:
			s = tr("Le message n'a pas t entirement envoy.");
			break;
		case IProtocol::NOSUPPORTED:
			s = tr("Cette fonction n'est pas supporte par ce protocole.");
			break;
		case IProtocol::INVALID_VALUE:
			s = tr("La valeur reue est incorrecte!") + '\n' + sCheckComm;
			break;
		case IProtocol::BAD_CALL:
			s = CLanguage::GetMsgInternalError(tr("Mauvais appel de fonction.\nMerci de le signaler  l'auteur du protocole."));
			break;
		case IProtocol::CANT_CHANGE_TRANSFERT_MODE:
			s = tr("Impossible de changer de mode transfert car ceci n'est pas gr par ce protocole.\nConseil : Faites tous ces transferts en plusieurs tapes.");
			break;
		case IProtocol::OK:
			return;
		}
		CMessageBox::MsgCritical((QWidget*)parent(), s);
	}
	else {
		CMessageBox::MsgCritical((QWidget*)parent(), tr("Impossible de crer une instance du protocole."));
	}
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::CloseComm(int nStatus)
{
	if(m_pProtocol != NULL) {
		m_pProtocol->CloseCom(nStatus);
	}
	close();
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::ShowTime(bool bShowBytes)
{
	if(bShowBytes) {
		m_lblSize.setText(QString::number(nByteRead) + m_sMaxBytes);
	}

	int nElapsed = m_qStartTime.elapsed();
	if(nElapsed > m_nLastElapsed + 900) {
		QString t1 = tr("Temps coul :" ) + " ";
		QString t2 = tr("Temps restant :") + " ";
		QString t3 = tr("Temps estim :" ) + " ";

		ShowMinSec(&m_lblElapsed, t1, nElapsed);
		if(nByteMax != -1 && nByteRead != 0) {
			int nRemaining = (int)((nElapsed-m_nMSecIgnored) / (float)nByteRead * (nByteMax - nByteRead));
			ShowMinSec(&m_lblRemaining, t2, nRemaining);
			ShowMinSec(&m_lblEstimated, t3, nElapsed + nRemaining);
		}
		m_nLastElapsed = nElapsed;
	}
	qApp->processEvents();
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::ShowMinSec(QLabel* pLabel, const QString& sText, int nMilliSec)
{
	QString	s;
	nMilliSec /= 1000;
	s.sprintf("%i min %.2i sec", nMilliSec/60, nMilliSec%60);
	pLabel->setText(sText + s);
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::Receiving(int numDrive, unsigned char* buffer)
{
	TimeCounter(TA_EXCLUDE_TIME);
	m_cmdCancel.setEnabled(TRUE);
	m_bCanceled = false;
	if(SetTransfertMode(IProtocol::TR_FLASH) < 1) {
		ShowError();
		return -1;
	}
	setCaption(tr("Rception en cours..."));

	QString s = CDriveAccess::GetDriveName(numDrive);
	if(!numDrive) {
		s += " (" + CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM) + QString(")");
	}

	s = tr("Rception du lecteur") + " " + s;
	m_lblInfo.setText(s);
	ShowTime(false);

	//Demande de rception du lecteur
	int nReceive = m_pProtocol->ImportDrive(numDrive);
	int nByteReadDrive = 0;
	if(nReceive < 0) {
		ShowError();
		return -1;
	}

	TimeCounter(TA_END_OF_EXCLUDE_TIME);

	//Dpart du transfert
	while(nByteReadDrive < nReceive) {
		int n = m_pProtocol->ReceiveData(buffer + nByteReadDrive, nReceive - nByteReadDrive);
		if(n < 1) {
			ShowError();
			return -1;
		}
		nByteReadDrive	+= n;
		nByteRead		+= n;
		ShowTime();
		if(m_bCanceled) {
			return -1;
		}
	}
	memset(buffer + nByteReadDrive, 0xFF, 131072 - nByteReadDrive);
	return nReceive;
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::Sending(int numDrive, unsigned char* buffer)
{
	TimeCounter(TA_EXCLUDE_TIME);
	m_cmdCancel.setEnabled(FALSE);

	bool bOk = true;
	if(numDrive == 0) {
		CCasioSystem system;
		system.OpenDriveFile(buffer, DRIVE_SIZE);
		if(!system.IsValid()) {
			if(!CMessageBox::MsgQuestion(this, tr("Le systme envoy n'est pas valide.") + '\n' + tr("Souhaitez-vous quand mme l'envoyer ?"))) {
				bOk = false;
			}
		}
		else{
			if(m_pCalcInfo != NULL && m_pCalcInfo->IsAFX2LIKE() && m_pCalcInfo->GetCalcModel() != system.GetSystemType() &&
				!CMessageBox::MsgQuestion(this, tr("Le systme envoy n'est pas compatible avec le modle de votre calculatrice.") + '\n' + tr("Souhaitez-vous quand mme l'envoyer ?"))) {
				bOk = false;
			}
		}
	}
	if(!bOk) {
		TimeCounter(TA_END_OF_EXCLUDE_TIME);
		return -1;
	}

	if(SetTransfertMode(IProtocol::TR_FLASH) < 1) {
		ShowError();
		return -1;
	}
	setCaption(tr("Envoi en cours..."));


	QString s = CDriveAccess::GetDriveName(numDrive);
	if(!numDrive) {
		s += " (" + CLanguage::GetWordTranslation(CLanguage::WO_SYSTEM) + QString(")");
	}

	s = tr("Envoi le lecteur") + " " + s;
	m_lblInfo.setText(s);
	m_lblSize.setText(tr("Formatage en cours..."));
	ShowTime(false);

	//Demande d'envoi
	if(m_pProtocol->ExportDrive(numDrive, buffer) < 1)
	{
		ShowError();
		return -1;
	}

	TimeCounter(TA_END_OF_EXCLUDE_TIME);

	int nByteReadDrive = 0;
	int nSend = m_pProtocol->GetUsefulSize(buffer);
	while(nByteReadDrive < nSend)
	{
		int r = m_pProtocol->SendData(buffer + nByteReadDrive, nSend - nByteReadDrive);
		if(r <  1)
		{
			ShowError();
			return -1;
		}
		nByteReadDrive	+= r;
		nByteRead		+= r;
		ShowTime();
	}
	if(m_pCalcInfo != NULL) {
		m_pCalcInfo->IncCounter(numDrive);
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::SendingRAM(unsigned short nZone, const char* name, const void* buffer, unsigned int nSize, const char* password)
{
	TimeCounter(TA_EXCLUDE_TIME);
	m_cmdCancel.setEnabled(TRUE);
	m_bCanceled = false;

	if(SetTransfertMode(IProtocol::TR_SENDRAM) < 1)
	{
		ShowError();
		return -1;
	}

	setCaption(tr("Envoi en cours..."));

	m_lblInfo.setText(tr("Envoi '%1'").arg(name));
	m_lblSize.setText(tr("Demande d'envoi..."));
	ShowTime(false);

	//Demande d'envoi
	if(m_pProtocol->SendRAMFile(nZone, name, nSize) == -1)
	{
		ShowError();
		return -1;
	}
	m_lblSize.setText(tr("Attente de confirmation..."));
	
	int nResult;
	while(!m_bCanceled && (nResult=m_pProtocol->WaitRAMFileConfirmation(nZone, password)) == 0)
	{
		ShowTime(false);
	}
	TimeCounter(TA_END_OF_EXCLUDE_TIME);
	if(m_bCanceled)
	{
		return -1;
	}
	switch(nResult)
	{
	case -1:
		//Refus par le destinataire
		nByteRead += nSize;
		return 1;
	case -2:
		ShowError();
		return -1;
	}
	unsigned int nCpt = 0;
	nResult = 1;
	while(nResult > 0 && nCpt < nSize)
	{
		nResult = m_pProtocol->SendData((const char*)buffer + nCpt, nSize - nCpt);
		if(nResult > 0)
		{
			nCpt      += nResult;
			nByteRead += nResult;
		}
		ShowTime();
	}
	if(nResult < 1)
	{
		ShowError();
		return -1;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::WaitingRAM(unsigned short& nZone, char* name, char* password)
{
	TimeCounter(TA_EXCLUDE_TIME);
	m_cmdCancel.setEnabled(TRUE);
	m_bCanceled = false;

	if(SetTransfertMode(IProtocol::TR_RECEIVERAM) < 1) {
		ShowError();
		return -1;
	}	
	setCaption(tr("Rception en cours..."));
	m_lblInfo.setText(tr("Attente de fichiers..."));
	
	int nResult;
	while(!m_bCanceled && (nResult=m_pProtocol->ReceiveRAMFile(nZone, name, password)) == -3) {
		ShowTime();
	}
	TimeCounter(TA_END_OF_EXCLUDE_TIME);
	if(m_bCanceled) {
		return -1;
	}
	if(nResult == -1) {
		ShowError();
		return -1;
	}
	m_lblInfo.setText(tr("Rception de '%1'").arg(name));
	return nResult;
}

//-----------------------------------------------------------------------------------
int CTransfertWindow::ReceivingRAM(void* buffer, unsigned short nZone, int nSize, char* password)
{
	int nRet = m_pProtocol->SendRAMFileConfirmation(buffer != NULL, nZone, password);
	if(buffer != NULL) {
		int nBytesReceived = 0;
		while(nBytesReceived < nSize && nRet > 0) {
			nRet = m_pProtocol->ReceiveData((char*)buffer + nBytesReceived, nSize - nBytesReceived);
			if(nRet > 0) {
				nBytesReceived += nRet;
				nByteRead      += nRet;
			}
			ShowTime();
		}			
	}
	if(nRet < 1) {
		ShowError();
		return -1;
	}
	return nSize;
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::SetData(const int& nTotalBytes, bool bClear)
{
	nByteMax = nTotalBytes;
	if(bClear) {
		m_qStartTime.restart();
		nByteRead	   = 0;
		m_nMSecIgnored = 0;
		m_nLastElapsed = -99999;
	}
	if(nByteMax > 0) {
		m_sMaxBytes = " / " + QString::number(nByteMax);
	}
	else {
		m_sMaxBytes = QString::null;
	}
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::CancelClick()
{
	m_bCanceled = true;
}

//-----------------------------------------------------------------------------------
void CTransfertWindow::TimeCounter(TimeAction nAction)
{
	switch(nAction)
	{
	case TA_EXCLUDE_TIME:
		m_qExcludeTime.restart();
		break;
	case TA_END_OF_EXCLUDE_TIME:
		m_nMSecIgnored += m_qExcludeTime.elapsed();
	}
}

//-----------------------------------------------------------------------------------
unsigned int CTransfertWindow::GetUsefulSize(const void* pBuffer)
{
	if(m_pProtocol) {
		return m_pProtocol->GetUsefulSize(pBuffer);
	}
	return 0;
}

//-----------------------------------------------------------------------------------
unsigned int CTransfertWindow::GetSizeReceive(bool bSystem, bool bDriveL, bool bDriveM, bool bDriveN,
											bool bDriveO, bool bDriveP, bool bDriveQ)
{
	if(m_pProtocol) {
		return m_pProtocol->GetSizeOfDrives(bSystem, bDriveL, bDriveM, bDriveN, bDriveO, bDriveP, bDriveQ);
	}
	return 0;
}
