
#ifndef TRANSFERTWINDOW_H
#define TRANSFERTWINDOW_H


#include <qdatetime.h>
#include <qdialog.h>
#include <qpushbutton.h>

#include <F100Qt.h>
#include <IProtocol.h>

#include "Label.h"

class CCalcInfo;


/*!
	\class CTransfertWindow TransfertWindow.h
	\brief Fentre de transfert de donnes entre un PC et une calculatrice
	\author Quaranta David
	\version 1.00
	
	\ingroup F100Qt
	Classe permettant d'afficher une fentre de transfert et de
	transfrer des donnes  une calculatrice en utilisant un protocole quelconque
*/
class DLLEXPORT CTransfertWindow : public QDialog
{
    Q_OBJECT
public:
	/** Constructeur
	 * @param[in] pCalcInfo Information sur la calculatrice
	 * @param[in] parent Fentre parente
	 */
	CTransfertWindow(CCalcInfo* pCalcInfo, QWidget* parent = 0);
	
	/** Destructeur.\n
	 * Appelle DeleteProtocol()
	 */
	virtual ~CTransfertWindow();

	/** @return Retourne le type de calculatrice support par le protocole charg */
	CasioType GetCasioType() const;

	/** Permet de dfinir le protocole  utiliser.\n
	 * Si un autre protocole tait dj choisi, l'ancien protocole
	 * est dtruit, mais avant la communication sera ferme proprement
	 * @param[in] pOp Point d'entre du protocole
	 * @param[in] pComDevice Priphrique  utiliser
	 * @param[in] nSpeed Index de la vitesse de communication
	 * @param[in] nTotalSize Taille totale en octets du transfert (-1 si inconnue)
	 * @return 1 pour OK \n -1 erreur (Appelle ShowError() dans ce cas)
	 */
	int          SetProtocol(Protocol_Op& pOp, IComDevice* pComDevice, int nSpeed, int nTotalSize);
	
	/** Spcifie la taille totale que l'on veut transmettre.
	 * @param[in] nTotalBytes Taille du transfert en octets (< 0 pour inconnue)
	 * @param[in] bClear Si  vrai, efface les anciennes donnes (temps coul,...)
	 */
	void         SetData(const int& nTotalBytes, bool bClear = true);
	
	/** Demande  revoir le lecteur de la flash \a numDrive
	 * @param[in] numDrive Numro du leteur  recevoir [0, 6]
	 * @param[out] buffer Zone mmoire pour recevoir les donnes du lecteur (Taille minimum de DRIVE_SIZE octets)
	 * @return 1 pour OK\n -1 pour chec
	 */
	int			 Receiving(int numDrive, unsigned char* buffer);

	/** Envoi des donnes dans le lecteur de la flash numDrive
	 * @param[in] numDrive Numro du leteur  envoyer [0, 6]
	 * @param[in] buffer Zone mmoire des donnes  envoyer dans le lecteur
	 * @return 1 pour OK\n -1 pour chec
	 */
	int			 Sending(int numDrive, unsigned char* buffer);
	
	/** Envoi un fichier en RAM
	 * @param[in] nZone Zone du fichier concern
	 * @param[in] name Nom du fichier
	 * @param[in] buffer Donnes  envoyer
	 * @param[in] nSize Taille en octect du fichier (donnes utiles)
	 * @param[in] password Mot de passe du fichier (optionnel et valable que pour les fichiers BASIC (nZone == 1))
	 * @return 1 pour OK\n -1 pour chec
	 */
	int          SendingRAM(unsigned short nZone, const char* name, const void* buffer, unsigned int nSize, const char* password = NULL);
	
	/** Attend la rception d'un fichier de la RAM 
	 * @param[out] nZone Zone du fichier concern
	 * @param[out] name Nom du fichier
	 * @param[out] password Mot de passe du fichier reu (optionnel et valable que pour les fichiers BASIC (nZone == 1))
	 * @return -1 (erreur ou annul) \n
	 * > 0 Ok, taille en octets du fichier et rempli les champs \a nZone, \a name et \a password \n
	 * -2 Communication termine
	 * @remarks Le mot de passe peut tre reu soit ici, soit dans ReceivingRAM(void*, unsigned short, int, char*) suivant les protocoles.
	 * @post ReceivingRAM(void*, unsigned short, int, char*) doit tre appele pour recevoir ou non le fichier
	 */
	int          WaitingRAM(unsigned short& nZone, char* name, char* password = NULL);

	/** Recoit le contenu d'un fichier de la RAM.
	 * @param[in] buffer Si gal  NULL alors signifie  l'expditeur que le fichier est refus,
	 * sinon \a buffer doit avoir une zone mmoire suffisante pour recevoir les donnes du fichier
	 * @param[in] nZone Zone du fichier  recevoir
	 * @param[in] nSize Taille en octets du fichier  recevoir
	 * @param[out] password Mot de passe du fichier reu (optionnel et valable que pour les fichiers BASIC (nZone == 1))
	 * @return La taille du fichier reu ou -1 si chec
	 * @pre Doit tre appele aprs WaitingRAM(unsigned short&, char*, char*)
	 * @remarks Le mot de passe peut tre reu soit ici, soit dans WaitingRAM(unsigned short&, char*, char*) suivant les protocoles.
	 */
	int          ReceivingRAM(void* buffer, unsigned short nZone, int nSize, char* password = NULL);
	
	/** Ferme la communication du protocole
	 * @param[in] nStatus Valeur d'arrt de la communication. Si suprieur  0, signale que la communication peut tre termine correctement,
	 * sinon la communication sera ferme sans prvenir l'autre interlocuteur.
	 * @see IProtocol::CloseCom(int)
	 */
	void		 CloseComm(int nStatus);

	/** Demande au protocole courant la taille utile d'une zone mmoire (valable uniquement pour la Flash)
	 * @param[in] pBuffer Zone mmoire  analyser
	 * @return Retourne la taille utile en octets ou 0 si erreur
	 * @see IProtocol::GetUsefulSize(const void*)
	 */
	unsigned int GetUsefulSize(const void* pBuffer);

	/** Demande au protocole la taille de lecteur
	 * @param[in] bSystem A vrai si on veut compter la taille du systme
	 * @param[in] bDriveL A vrai si on veut compter la taille du lecteur L
	 * @param[in] bDriveM A vrai si on veut compter la taille du lecteur M
	 * @param[in] bDriveN A vrai si on veut compter la taille du lecteur N
	 * @param[in] bDriveO A vrai si on veut compter la taille du lecteur O
	 * @param[in] bDriveP A vrai si on veut compter la taille du lecteur P
	 * @param[in] bDriveQ A vrai si on veut compter la taille du lecteur Q
	 * @return Retourne la taille totale en octets ou 0 si erreur
	 * @see IProtocol::GetSizeOfDrives(bool, bool, bool, bool, bool, bool, bool)
	 */
	unsigned int GetSizeReceive(bool bSystem, bool bDriveL, bool bDriveM, bool bDriveN,
											bool bDriveO, bool bDriveP, bool bDriveQ);

	/** @return Le protocole utilis */
	IProtocol*   GetProtocol() const {return m_pProtocol;}

protected:
	/** Permet d'exclure un lapse de temps du calcul du temps restant */
	enum TimeAction {
		TA_EXCLUDE_TIME = 0,   /*!< Dbut d'exclusion du calcul */
		TA_END_OF_EXCLUDE_TIME /*!< Fin de l'exclusion */
	};

protected:
	/** Permet d'afficher les temps (coul, restant et estim)
	 * @param[in] bShowBytes A vrai si on veut remettre  jour le nombre d'octets transfrs
	 */
	void		 ShowTime(bool bShowBytes = true);

	/** Permet de dfinir l'action sur le calcul du temps restant
	 * @param[in] nAction Type d'action
	 */
	void		 TimeCounter(TimeAction nAction);

	/** Permet de dfinir le mode de transfert
	 * @param[in] nTransfert Nouveau mode de transfert
	 * @see IProtocol::SetTransfertMode(IProtocol::TRANSFERT)
	 */
	int          SetTransfertMode(IProtocol::TRANSFERT nTransfert);

	/** Affiche le message d'erreur dans une boite de dialogue */
	void		 ShowError();

	/** Dtruit le protocol courant */
	void         DeleteProtocol();

protected:
	/** Permet d'afficher un temps en minutes et secondes
	 * @param[in] pLabel Objet recevant le texte
	 * @param[in] sText Texte devant l'affichage du temps
	 * @param[in] nMilliSec Temps en milli-secondes
	 */
	static void	 ShowMinSec(QLabel* pLabel, const QString& sText, int nMilliSec);

private slots:
	/** Bouton \a Annuler press */
	void		 CancelClick();

private:
    QPushButton    m_cmdCancel;
    CLabel		   m_lblEstimated;
    CLabel		   m_lblRemaining;
    CLabel		   m_lblElapsed;
    CLabel		   m_lblSize;
    CLabel		   m_lblInfo;
	QTime		   m_qStartTime;
	QTime          m_qExcludeTime;
	int			   m_nMSecIgnored;
	int			   m_nLastElapsed;
	QString		   m_sMaxBytes;
	IProtocol*     m_pProtocol;
	unsigned int   nByteRead;
	int			   nByteMax;
	bool		   m_bCanceled;
	Protocol_Op    m_pProtocolOp;
	CCalcInfo*     m_pCalcInfo;
};

#endif
