
#ifndef APIFUNCTION_H
#define APIFUNCTION_H

/** \file APIFunction.h
 * \brief API supportant plusieurs OS.
 * \ingroup F100TK
 * Toutes ces fonctions sont implmentes avec des mthodes de l'API
 * de Windows et UNIX donc il faut reimplmenter ces fontions pour un autre OS.
 * Ce sont des fonctions utiles, elles sont utilises un peu partout.
 */

#ifdef WIN32
#	include <windows.h>
#	define HND HANDLE
#	define READWRITE GENERIC_READ | GENERIC_WRITE
#	define DLL_EXTENSION "*.dll"
#else
#	include <sys/types.h>
#	include <sys/stat.h>
#	include <fcntl.h>
#	include <string.h>
#	define HND int
#	define GENERIC_READ      O_RDONLY
#	define READWRITE         O_RDWR
#	define GENERIC_WRITE     O_WRONLY
#	define FILE_SHARE_READ   0
#	define FILE_SHARE_WRITE  0
#	define CREATE_ALWAYS     O_CREAT | O_TRUNC
#	define OPEN_ALWAYS       O_CREAT
#	define OPEN_EXISTING     0
#	define INVALID_HANDLE_VALUE -1
#	ifdef MAC
#		define DLL_EXTENSION "*.dylib"
#	else
#		define DLL_EXTENSION "*.so"
#	endif
#endif

#include <F100_Op.h>

/** Permet d'ouvrir une page avec un navigateur
 * @param[in] sPath Chemin du fichier ou URL  ouvrir
 * @param[in] bHttpProtocol Si  vrai alors rajoute la chaine "http:://"
 * @return 1 en cas de succs \n
 * 0 Pas assez de mmoire \n
 * -1 Fichier ou rpertoire non trouv. \n
 * -2 Erreur non inconnue
 * @remark Sous Windows, le document est ouvert dans notre navigateur par dfaut
 * @remark Sous Linux, le document sera ouvert avec Mozilla
 */
DLLEXPORT int          LaunchNavigator(const char* sPath, bool bHttpProtocol = true);

/** Ouvre la boite de mail pour crire un message
 * @param[in] name Sujet du message
 * @remark Sous Windows, c'est la messagerie par dfaut qui sera ouverte
 * @remark Sous Linux, c'est la messagerie de Mozilla qui s'ouvrira
 */
DLLEXPORT void         OpenEMail(const char* name);

/** Permet d'excuter un fichier
 * @param[in] name Chemin complet  excuter
 * @param[in] param Paramtres
 * @param[in] bWait Si  vrai alors attend que le nouveau processus s'arrte.
 * Dans ce cas le fichier doit tre un fichier binaire excutable
 * @return 1 en cas de succs \n
 * 0 Pas assez de mmoire \n
 * -1 Fichier ou rpertoire non trouv. \n
 * -2 Erreur non inconnue
 */
DLLEXPORT int          ExecuteFile(const char* name, const char* param = 0, bool bWait = false);

/** Met le processus en attente
 * @param[in] dwMilliseconds Temps d'attente en milli-secondes
 */
DLLEXPORT void         Wait(int dwMilliseconds);

/** Permet d'ouvrir d'avoir un HANDLE sur un objet (Fichier, Pipe, Socket, etc...)
 * @param[in] name Chemin
 * @param[in] fa Mode d'ouverture (GENERIC_READ, GENERIC_WRITE, READWRITE)
 * @param[in] fc Option d'ouverture (OPEN_EXISTING, OPEN_ALWAYS, CREATE_ALWAYS)
 * @param[in] shared Option de partage d'accs (FILE_SHARE_READ ou FILE_SHARE_WRITE)
 * @return Retourne un HANDLE, ou INVALID_HANDLE_VALUE si chou
 */
DLLEXPORT HND          OpenDesc(const char* name, int fa, int fc, int shared=0);

/** Ferme l'accs  un objet
 * @param[in] h HANDLE de l'objet
 * @remarks Vrifie la validit de \a h (ie. h != INVALID_HANDLE_VALUE)
 */
DLLEXPORT void         CloseDesc(HND& h);

/** Permet de rcuprer des donnes d'un objet
 * @param[in] h HANDLE de l'objet
 * @param[out] buf Zone mmoire qui va rcuprer les donnes
 * @param[in] nBytes Nombre d'octets  lire
 * @return Retourne vrai en cas de succs, sinon faux
 */
DLLEXPORT bool         ReadDesc(HND h, void* buf, unsigned int nBytes);

/** Permet d'crire des donnes sur un objet
 * @param[in] h HANDLE de l'objet
 * @param[in] buf Zone mmoire  crire
 * @param[in] nBytes Nombre d'octets  crire
 * @return Retourne vrai en cas de succs, sinon faux
 */
DLLEXPORT bool         WriteDesc(HND h, const void* buf, unsigned int nBytes);

/** Permet de dplacer le pointeur courant d'un objet. (Ne fonctionne pas pour tous les types objets)
 * @param[in] h HANDLE de l'objet
 * @param[in] nPos Nouvelle position du pointeur courant (en octets)
 */
DLLEXPORT void         SeekDesc(HND h, unsigned int nPos);

/** Permet de rcuprer le pointeur courant d'un objet. (Ne fonctionne pas pour tous les types objets)
 * @param[in] h HANDLE de l'objet
 * @return La position du pointeur en octets
 */
DLLEXPORT unsigned int PointerPosition(HND h);


/** Permet d'ouvrir un fichier en lecture
 * @param[in] sFile Chemin du fichier
 * @return Retourne un HANDLE, ou INVALID_HANDLE_VALUE si chou
 * @see OpenDesc(const char*, int, int, int)
 */
DLLEXPORT HND          OpenDescModeRead(const char* sFile);

/** Permet d'ouvrir un fichier en criture et lecture
 * @param[in] sFile Chemin du fichier
 * @return Retourne un HANDLE, ou INVALID_HANDLE_VALUE si chou
 * @see OpenDesc(const char*, int, int, int)
 */
DLLEXPORT HND          OpenDescModeRdWr(const char* sFile);

/** Permet de crer un dossier
 * @param[in] name Chemin du dossier  crer
 * @return 1 si le dossier est cr \n
 * 0 si le dossier existe dj \n
 * -1 en cas d'erreur
 */
DLLEXPORT int          MkDir(const char* name);

/** Permet de renommer ou dplacer un dossier ou un fichier
 * @param[in] oldName Ancien chemin
 * @param[in] newName Nouveau chemin
 * @return Retourne vrai en cas de succs, sinon faux
 */
DLLEXPORT bool         MoveOrRenameFile(const char* oldName, const char* newName);

/** Permet de supprimer un dossier ou un fichier
 * @param[in] name Chemin
 * @return Retourne vrai en cas de succs, sinon faux
 */
DLLEXPORT bool         KillFile(const char* name);

/** Permet de connaitre la taille d'un fichier
 * @param[in] name Chemin
 * @param[out] pError A vrai en sortie, si une erreur s'est produite, sinon faux. [Optionnel]
 * @return Retourne la taille du fichier en octets
 */
DLLEXPORT unsigned int FileSize(const char* name, bool* pError);

/** Permet de connaitre la taille d'un fichier
 * @param[in] h HANDLE du fichier
 * @return Retourne la taille du fichier en octets
 */
DLLEXPORT unsigned int FileSize(HND h);

/** Permet de dfinir la taill d'un fichier
 * @param[in] h HANDLE du fichier
 * @param[in] nPos Taille du fichier en octets
 * @return 1 pour OK \n
 * -1 si chou
 */
DLLEXPORT int          SetFileSize(HND h, unsigned int nPos);

/** Permet d'avoir juste le nom d'un dossier ou fichier en fonction du chemin complet
 * @param[in] sFullPath Chemin complet
 * @param[out] sName Pointeur sur la zone du nom
 */
DLLEXPORT void         GetNameOfFile(const char* sFullPath, const char*& sName);

#endif
