
#include "BASICDrive.h"

#include "DriveAccess.h"
#include "CasioSystem.h"


#include <F100_Debug.h>

#define HEADER_SIZE  sizeof(header)
#define NUMBER_INDEX 11

unsigned char header[] = { 0x00, 0xFF, 0x50, 0x72, 0x6F, 0x67, 0x72, 0x61, 0x6D, 0x73, 0x30, 0x31, 0x00
								,0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
								,0xFF, 0xFF, 0xC7, 0xDE, 0xFF, 0x00};

//-----------------------------------------------------------------------------------
CBASICDrive::CBASICDrive(bool bOneFile, bool bVariableSize)
:CDriveFile(bOneFile, bVariableSize, HEADER_SIZE)
{
	m_bIsValid = false;
	m_mFiles   = new FilesMap();
	m_nSize    = 0;
}

//-----------------------------------------------------------------------------------
CBASICDrive::~CBASICDrive()
{
	CloseDrive();
	delete m_mFiles;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::CloseDrive()
{
	int nRet = CDriveFile::CloseDrive();
	m_bIsValid = false;
	FilesMap::iterator it  = m_mFiles->begin();
	FilesMap::iterator end = m_mFiles->end();
	for(; it != end; ++it) {
		_WARNING1("CBASICDrive::CloseDrive() found file : %s",((*it).second)->GetName().c_str());
		delete ((*it).second);
	}
	m_mFiles->clear();
	return nRet;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::GetSizeInBytes() const
{
	return m_nSize;
}

//-----------------------------------------------------------------------------------
bool CBASICDrive::IsValid()
{
	if(!m_bIsValid) {
		m_bIsValid = true;
		m_nSize = DRIVE_SIZE;
		if(HeadersAreValid()) {
			CBASICFileInfo* pFile = GetFirstFile();
			m_nSize = 0;
			bool bOk = true;
			while(bOk && pFile != NULL) {
				unsigned short nSize = 0;
				m_nSize += pFile->GetSize();
				bOk = SetPosition(pFile->GetAddress() + pFile->GetSize() - 2) > 0
					&& Read(&nSize, sizeof(nSize)) > 0
					&& nSize == pFile->GetSize();
				pFile = GetNextFile(pFile, true);
			}
			if(bOk) {
				return true;
			}
		}
		m_nSize    = -1;
		m_bIsValid = false;
	}
	return m_bIsValid;
}

//-----------------------------------------------------------------------------------
void CBASICDrive::FreeFileInfo(CBASICFileInfo* pInfo)
{
	if(pInfo != NULL) {
		if(pInfo->DecCountRef() == 0) {
			m_mFiles->erase(pInfo->GetAddress());
			delete pInfo;
		}
	}
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::GetFirstFile()
{
	return GetFileInfo(0);
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::GetNextFile(CBASICFileInfo* pFileInfo, bool bFreeLastFileInfo)
{
	if(IsValid() && pFileInfo != NULL) {
		CBASICFileInfo* pNextFileInfo = GetFileInfo(pFileInfo->GetAddress() + pFileInfo->GetSize());
		if(bFreeLastFileInfo) {
			FreeFileInfo(pFileInfo);
		}
		return pNextFileInfo;
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::FindFile(unsigned int nPos)
{
	if(IsValid()) {
		CBASICFileInfo* pFile = GetFileInfo(nPos, false);
		if(pFile != NULL) {
			return pFile;
		}
		pFile = GetFirstFile();
		while(pFile != NULL) {
			if(pFile->GetAddress() == nPos) {
				return pFile;
			}
			pFile = GetNextFile(pFile, true);
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::FindFile(const string& sName)
{
	if(IsValid()) {
		CBASICFileInfo* pFile = GetFirstFile();
		while(pFile != NULL) {
			if(pFile->GetName() == sName) {
				return pFile;
			}
			pFile = GetNextFile(pFile, true);
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::GetFileInfo(unsigned int nPos, bool bCreateIt)
{
	CBASICFileInfo* pInfo = Find((int)nPos);
	if(pInfo != NULL) {
		pInfo->IncCountRef();
		return pInfo;
	}
	if(bCreateIt) {
		pInfo = new CBASICFileInfo(this);
		if(pInfo != NULL) {
			pInfo->m_nAddress = nPos;
			if(RefreshFileInfo(pInfo) == 1) {
				(*m_mFiles)[nPos] = pInfo;
			}
			else{
				delete pInfo;
				pInfo = NULL;
			}
		}
	}
	return pInfo;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::RefreshFileInfo(CBASICFileInfo* pFile)
{
	if(pFile != NULL && pFile->m_pDrive == this && IsValid() && SetPosition(pFile->GetAddress()) > 0) {
		HeaderBasicFile hbf;
		if(Read(&hbf, sizeof(hbf)) > 0 && (hbf.state == 0x00 || hbf.state == 0x01)) {
			char buf[9];
			buf[8] = 0;
			memcpy(buf, hbf.name, sizeof(hbf.name));
			pFile->m_sName = buf;				
			memcpy(buf, hbf.password, sizeof(hbf.password));
			pFile->m_sPassword  = buf;
			pFile->m_nSize      = hbf.size;
			pFile->m_bIsVisible = hbf.state == 1;
			pFile->m_bIsBase    = hbf.base  == 1;
			return 1;
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CBASICDrive::Find(const CBASICFileInfo* pFileInfo) const
{
	if(pFileInfo != NULL) {
		return m_mFiles->find(pFileInfo->GetAddress()) != m_mFiles->end();
	}
	return false;
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::Find(int nPos) const
{
	FilesMap::iterator it = m_mFiles->find(nPos);
	if(it != m_mFiles->end()) {
		return (*it).second;
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
bool CBASICDrive::IsOptimizable()
{
	if(IsValid()) {
		CBASICFileInfo* pFile = GetFirstFile();
		while(pFile != NULL) {
			if(!pFile->IsVisible()) {
				FreeFileInfo(pFile);
				return true;
			}
			pFile = GetNextFile(pFile, true);
		}
	}
	return false;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::Optimization()
{
	if(IsValid()) {
		CBASICFileInfo* pFile = GetFirstFile();
		while(pFile != NULL) {
			if(!pFile->IsVisible()) {
				CBASICFileInfo* pNextFile = GetNextFile(pFile, false);
				if(RemoveFile(pFile) < 1) {
					FreeFileInfo(pFile);
					FreeFileInfo(pNextFile);
					return -1;
				}
				pFile = pNextFile;
			}
			else{
				pFile = GetNextFile(pFile, true);
			}
		}
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::InsertData(unsigned int nPos, const void* pData, unsigned int nShift)
{
	int nRet = CDriveFile::InsertData(nPos, pData, nShift);
	if(nRet < 1) {
		return nRet;
	}
	m_nSize += nShift;
	UpdateFiles(nPos, (int)nShift);
	return 1;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::RemoveData(unsigned int nPos, unsigned int nSize)
{
	int nRet = CDriveFile::RemoveData(nPos, nSize);
	if(nRet < 1) {
		return nRet;
	}
	m_nSize -= nSize;
	UpdateFiles(nPos, (int)-nSize);
	return 1;
}

//-----------------------------------------------------------------------------------
void CBASICDrive::UpdateFiles(unsigned int nPos, int nShift)
{
	FilesMap::iterator it  = m_mFiles->begin();
	FilesMap::iterator end = m_mFiles->end();
	CBASICFileInfo* pInfo;
	for(; it != end; ++it) {
		pInfo = ((*it).second);
		if(pInfo->GetAddress() > nPos) {
			pInfo->m_nAddress += nShift;
		}
	}
	it = m_mFiles->begin();
	while(it != end) {
		if(Move(it)) {
			it  = m_mFiles->begin();
			end = m_mFiles->end();
		}
		else{
			it++;
		}
	}
}

//-----------------------------------------------------------------------------------
bool CBASICDrive::Move(FilesMap::iterator it)
{
	CBASICFileInfo* pInfo = ((*it).second);
	if(pInfo->GetAddress() != (*it).first) {
		m_mFiles->erase(it);
		FilesMap::iterator find = m_mFiles->find(pInfo->GetAddress());
		if(find != m_mFiles->end()) {
			_ASSERTE(Move(find));
		}
		(*m_mFiles)[pInfo->GetAddress()] = pInfo;
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::RemoveFile(CBASICFileInfo* pFile)
{
	if(pFile != NULL && Find(pFile)) {
		m_mFiles->erase(pFile->GetAddress());
		int nRet = RemoveData(pFile->GetAddress(), pFile->GetSize());
		if(nRet > 0) {
			delete pFile;
		}
		else{
			(*m_mFiles)[pFile->GetAddress()] = pFile;
		}
		return nRet;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
CBASICFileInfo* CBASICDrive::AddFile(const string& sName, const void* pData, unsigned short nSize)
{
	if(IsValid()) {
		int nAddress = GetSizeInBytes();
		HeaderBasicFile hbf;
		CCasioSystem::CpyChar(hbf.name, sName.c_str(), sizeof(hbf.name));
		memset(hbf.password, 0x00, sizeof(hbf.password));
		hbf.size  = 24;
		hbf.state = 1;
		hbf.base  = 0;
		char buf[24];
		memcpy(buf, &hbf, sizeof(hbf));
		(*(unsigned int*)(buf+20)) = 0x00180000;
		if(SetPosition(nAddress) > 0 && Write(&buf, sizeof(buf)) > 0) {
			m_nSize += sizeof(buf);
			CBASICFileInfo* pFile = GetFileInfo(nAddress);
			if(pFile != NULL && pData != NULL) {
				pFile->SetData(pData, nSize);
			}
			return pFile;
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
int CBASICDrive::ResizeFile(CBASICFileInfo* pFile, unsigned short nSize)
{
	int nRet;
	if(pFile != NULL && nSize >= 24 && Find(pFile)) {
		if(nSize == pFile->GetSize()) {
			return 1;
		}
		if(nSize < pFile->GetSize()) {
			nRet = RemoveData(pFile->GetAddress() + nSize, pFile->GetSize() - nSize);
		}
		else{
			nRet = InsertValue(pFile->GetAddress() + pFile->GetSize(), 0x00, nSize - pFile->GetSize());
		}
		if(nRet > 0 && SetPosition(pFile->GetAddress()) > 0
			&& Write(&nSize, sizeof(nSize)) > 0
			&& SetPosition(pFile->GetAddress() + nSize - sizeof(nSize)) > 0
			&& Write(&nSize, sizeof(nSize)) > 0) {
			pFile->m_nSize = nSize;
			return 1;
		}
		return -1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void* CBASICDrive::GetHeader(int nNbHeader)
{
	if(nNbHeader >= 0 && nNbHeader < 10) {
		header[0xB] = nNbHeader + '1';
		return header;
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
//----------------------------------CBASICFileInfo-----------------------------------
//-----------------------------------------------------------------------------------
CBASICFileInfo::CBASICFileInfo(CBASICDrive* pDrive)
: CFileInfo()
{
	m_pDrive = pDrive;
}

//-----------------------------------------------------------------------------------
CBASICFileInfo::~CBASICFileInfo()
{
}

//-----------------------------------------------------------------------------------
const string& CBASICFileInfo::GetPassword() const
{
	return m_sPassword;
}

//-----------------------------------------------------------------------------------
unsigned short CBASICFileInfo::GetSize() const
{
	return m_nSize;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::GetDataSize() const
{
	return m_nSize - 24;
}

//-----------------------------------------------------------------------------------
bool CBASICFileInfo::IsVisible() const
{
	return m_bIsVisible;
}

//-----------------------------------------------------------------------------------
bool CBASICFileInfo::IsBaseMode() const
{
	return m_bIsBase;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::GetData(void* pData)
{
	if(m_pDrive->SetPosition(GetAddress() + 21) > 0) {
		return m_pDrive->Read(pData, GetDataSize());
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::SetData(const void* pData, unsigned short nSize)
{
	if(nSize <= MAX_DATA_SIZE && m_pDrive->ResizeFile(this, nSize + 24) > 0 && m_pDrive->SetPosition(GetAddress() + 21) > 0) {
		return m_pDrive->Write(pData, GetDataSize());
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::Rename(const string& name)
{
	if(m_pDrive->SetPosition(GetAddress() + 3) > 0) {
		char buf[9];
		CCasioSystem::CpyChar(buf, name.c_str(), sizeof(buf));
		if(m_pDrive->Write(buf, 8) > 0
			&& m_pDrive->RefreshFileInfo(this) > 0) {
			return 1;
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::SetPassword(const string& password)
{
	if(m_pDrive->SetPosition(GetAddress() + 11) > 0) {
		char buf[9];
		CCasioSystem::CpyChar(buf, password.c_str(), sizeof(buf));
		if(m_pDrive->Write(buf, 8) > 0
			&& m_pDrive->RefreshFileInfo(this) > 0) {
			return 1;
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::SetVisibility(bool bVisible)
{
	if(m_pDrive->SetPosition(GetAddress() + 2) > 0) {
		char state = (bVisible == true);
		if(m_pDrive->Write(&state, sizeof(state)) > 0
			&& m_pDrive->RefreshFileInfo(this) > 0) {
			return 1;
		}
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CBASICFileInfo::SetBaseMode(bool bActivated)
{
	if(m_pDrive->SetPosition(GetAddress() + 19) > 0) {
		char state = (bActivated == true);
		if(m_pDrive->Write(&state, sizeof(state)) > 0
			&& m_pDrive->RefreshFileInfo(this) > 0) {
			return 1;
		}
	}
	return -1;
}
