
#ifndef BASICDRIVE_H
#define BASICDRIVE_H

#ifdef _MSC_VER
#	pragma warning(disable:4786)
#endif

#include "DriveFile.h"
#include "FileInfo.h"

#include <map>
using namespace std;

class CBASICFileInfo;

/*!
	\class CBASICDrive BASICDrive.h
	\brief Permet de lire un lecteur contenant des fichiers BASIC
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
*/
class DLLEXPORT CBASICDrive : public CDriveFile
{
	friend class CBASICFileInfo;
public:
	/** Constructeur
	 * @param[in] bOneFile A vrai si on veut qu'un seul fichier soit cr.
	 * @param[in] bVariableSize A vrai si on veut que les fichiers crs soient au maximum de DRIVE_SIZE octets (si bOneFile == FALSE alors le fichier n'aura pas forcement une taille multiple de DRIVE_SIZE)
	 * @remarks Paramtre sans effet si l'on travaille dans une zone mmoire
	 */
	CBASICDrive(bool bOneFile = false, bool bVariableSize = true);

	/** Destructeur */
	virtual ~CBASICDrive();

	virtual int     CloseDrive();
	int             GetSizeInBytes() const;
	bool            IsValid();

	/** Permet de signaler au lecteur que vous avez plus besoin des informations sur un fichier
	 * @param[in] pInfo Informations sur le fichier qui ne sont plus utiles.
	 * @remarks \a pInfo est dtruit si et seulement si il n'y a plus de rfrence sur lui.
	 */
	void            FreeFileInfo(CBASICFileInfo* pInfo);

	/** @return Le premier fichier du lecteur, NULL si aucun */
	CBASICFileInfo* GetFirstFile();

	/** Permet de rcuprer le fichier suivant
	 * @param[in] pFileInfo Fichier actuel
	 * @param[in] bFreeLastFileInfo A vrai si l'on veut que le fichier \a pFileInfo soit libr
	 * @return Retourne le fichier aprs \a pFileInfo ou NULL si aucun
	 */
	CBASICFileInfo* GetNextFile(CBASICFileInfo* pFileInfo, bool bFreeLastFileInfo);

	/** @return Retourne vrai si le lecteur peut tre optimiz. (ie. Des fichiers partiellement supprims)
	 * @remarks Recherche le premier fichier qui est partiellement supprim, donc cette mthode peut tre lente.
	 * @see Optimization()
	 */
	bool            IsOptimizable();

	/** Permet d'optimiser le lecteur. L'optimisation supprime dfinitivement qui sont partiellement supprims. (ie. Les fichiers
	 * qui ne sont pas visibles sur la Graph100 mais qui sont quand mme prsent dans le lecteur)
	 * @return 1 OK\n 0 Non implment\n < 0 Erreur
	 */
	int             Optimization();
	
	/** Permet de rcuprer des informations sur un fichier est connaissant juste sa position dans le lecteur (Adresse)
	 * @param[in] nPos Adresse du fichier en octets
	 * @return Retourne le fichier si trouv, sinon NULL
	 */
	CBASICFileInfo* FindFile(unsigned int nPos);

	/** Permet de rcuprer des informations sur un fichier est connaissant juste son nom
	 * @param[in] sName Nom du fichier  rechercher
	 * @return Retourne le fichier si trouv, sinon NULL
	 */
	CBASICFileInfo* FindFile(const string& sName);

	/** Permet de supprimer un fichier dans le lecteur
	 * @param[in] pFile Fichier  supprimer
	 * @return 1 OK\n -1 Erreur
	 * @warning La zone pointe par \a pFile est dtruite.
	 */
	int             RemoveFile(CBASICFileInfo* pFile);

	/** Permet d'ajouter un fichier dans le lecteur
	 * @param[in] sName Nom du fichier
	 * @param[in] pData Donnes du fichier, ou NULL si aucune
	 * @param[in] nSize Taille des donnes du fichier (ie. Taille de \a pData)
	 * @return Retourne le fichier cr ou NULL si erreur
	 */
	CBASICFileInfo* AddFile(const string& sName, const void* pData = NULL, unsigned short nSize = 0);

	/** Permet de redimmensionner un fichier dans le lecteur
	 * @param[in] pFile Fichier  redimensionner
	 * @param[in] nSize Nouvelle taille du fichier
	 * @return 1 OK\n < 0 Erreur
	 */
	int             ResizeFile(CBASICFileInfo* pFile, unsigned short nSize);

protected:
	int             InsertData(unsigned int nPos, const void* pData, unsigned int nShift);
	int             RemoveData(unsigned int nPos, unsigned int nSize);
	void*           GetHeader(int nNbHeader);

private:
	bool            Find(const CBASICFileInfo* pFileInfo) const;
	CBASICFileInfo* Find(int nPos) const;
	void            UpdateFiles(unsigned int nPos, int nShift);

	/** Permet de rcuprer des informations sur un fichier en connaissant juste sa position dans le lecteur (Adresse)
	 * @param[in] nPos Adresse du fichier en octets
	 * @param[in] bCreateIt A vrai pour que les informations sur le fichier soient crs si elles ne sont pas trouvs dans la liste
	 * des fichiers ouverts, sinon rien n'est cr et la mthode retournera NULL
	 * @return Retourne le fichier si trouv, sinon NULL
	 * @warning L'adresse est considre comme correcte.
	 */
	CBASICFileInfo* GetFileInfo(unsigned int nPos, bool bCreateIt = true);

	int             RefreshFileInfo(CBASICFileInfo* pFile);

private:
	typedef map<int, CBASICFileInfo*> FilesMap;
	struct HeaderBasicFile {
		unsigned short size;
		unsigned char  state;
		char           name[8];
		char           password[8];
		char           base;
	};
private:
	bool  Move(FilesMap::iterator it);

private:
	bool      m_bIsValid;
	FilesMap* m_mFiles;
	int       m_nSize;

};

/*!
	\class CBASICFileInfo BASICDrive.h
	\brief Permet d'avoir des informations sur un fichier BASIC d'un lecteur
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
*/
class DLLEXPORT CBASICFileInfo : public CFileInfo
{
	friend class CBASICDrive;
public:
	enum{
		MIN_FILE_SIZE = 24,
		MAX_FILE_SIZE = 65535,
		MAX_DATA_SIZE = MAX_FILE_SIZE - MIN_FILE_SIZE,
	};
protected:
	/** Constructeur
	 * @param[in] pDrive Lecteur contenant ce fichier
	 */
	CBASICFileInfo(CBASICDrive* pDrive);
public:
	/** Destructeur */
	virtual ~CBASICFileInfo();

	/** @return Le not de passe du fichier */
	const string&  GetPassword() const;

	/** @return La taille du fichier en octets*/
	unsigned short GetSize() const;

	/** @return La taille des donnes du fichier en octets */
	int            GetDataSize() const;

	/** @return Retourne vrai si le fichier est visible dans le menu Memory de Casio (ie. Pas dans la corbeille), sinon faux */
	bool           IsVisible() const;

	/** @return Retourne vrai si le fichier est mode 'Base' */
	bool           IsBaseMode() const;

	/** Permet de rcuprer les donnes du fichier
	 * @param[out] pData Zone mmoire pour recevoir les donnes du fichier
	 * @remarks La zone mmoire \a pData doit au moins faire GetDataSize() const octets
	 * @return 1 OK\n 0 Non Implment\n < 1 Erreur
	 */
	int            GetData(void* pData);

	/** Permet de dfinir les donnes du fichier
	 * @param[in] pData Donnes  crire dans le fichier
	 * @param[in] nSize Taille en octets des donnes (nSize <= MAX_DATA_SIZE)
	 * @remarks La zone mmoire \a pData doit au moins faire GetDataSize() const octets
	 * @return 1 OK\n 0 Non Implment\n < 1 Erreur
	 */
	int            SetData(const void* pData, unsigned short nSize);

	/** Permet de renommer le fichier
	 * @param[in] name Nom du fichier
	 * @return 1 OK\n 0 Non Implment\n < 1 Erreur
	 */
	int            Rename(const string& name);

	/** Permet de changer le mot de passe du fichier
	 * @param[in] password Mot de passe
	 * @return 1 OK\n 0 Non Implment\n < 1 Erreur
	 * @remarks Un mot de passe vide "" dsactive le mot de passe
	 */
	int            SetPassword(const string& password);

	/** Permet de sortir/mettre le fichier dans la corbeille
	 * @param[in] bVisible Visibilit
	 * @return 1 OK\n 0 Non Implment\n < 1 Erreur
	 */
	int            SetVisibility(bool bVisible);

	/** Permet de choisir le mode du fichier
	 * @param[in] bActivated A vrai pour indiquer si le fichier est mode 'Base', faux pour le mode 'Normal'
	 * @return 1 OK\n 0 Non Implment\n < 1 Erreur
	 */
	int            SetBaseMode(bool bActivated);

private:
	CBASICDrive*   m_pDrive;
	string         m_sPassword;
	unsigned short m_nSize;
	bool           m_bIsVisible;
	bool           m_bIsBase;
};

#endif
