
#ifndef CASIOSYSTEM_H
#define CASIOSYSTEM_H

#include "DriveFile.h"

#include <casio.h>

using namespace Casio;


/*!
	\class CCasioSystem CasioSystem.h
	\brief Permet de lire un systme Casio
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
	Permet de grer le menu d'un systme Casio. Ce menu est le menu principal de votre Casio
	que vous pouvez voir au dmarrage de votre Casio.
*/
class DLLEXPORT CCasioSystem : public CDriveFile
{
public:
	enum{
		MAXMENU           =    7, /*!< Nombre maximum de menus dans un systme */
		MAXICON           =   35, /*!< Nombre maximum d'icnes dans un systme */
		ICON_PER_MENU     =   12, /*!< Nombre d'icnes par menu */
		SIZEBMP           = 1024, /*!< Taille d'un menu en octets */
		ICON_BYTE_PER_ROW =    4, /*!< Nombre d'octets par ligne sur une icne */
		MENU_BYTE_PER_ROW =   16, /*!< Nombre d'octets par ligne sur un menu */
		ICON_HEIGHT       =   19, /*!< Hauteur d'une icne en pixels */
		ICON_WIDTH        =   30, /*!< Largeur d'une icne en pixels */
		MENU_HEIGHT       =   64, /*!< Hauteur d'un menu en pixels */
		MENU_WIDTH        =  128, /*!< Largeur d'un menu en pixels */
		MENU_ROW          =    4, /*!< Nombre d'icne sur une ligne dans un menu */
		MENU_COL          =    3, /*!< Nombre d'icne sur une colonne dans un menu */
		TITLE_HEIGHT      =    7, /*!< Hauteur du titre du menu en pixels */
	};
	/** \brief Structure permettant d'avoir des informations sur une icne */
	struct TInfo
	{
		char number;      /*!< Numro de l'icne. */
		char bmp;         /*!< Affich ou non dans 'System.exe'. */
		char version[5];  /*!< Version. */
		char name[13];    /*!< Nom du fichier  ouvrir. */
		char lecteur[13]; /*!< Chemin incluant le lecteur. */
		char image[13];   /*!< Nom de l'image. */
		char arg[20];     /*!< Paramtres. */
	};
	/** Liste les tats possibles d'une icne. Valeurs utilises par TInfo.bmp pour savoir si l'icne est vue dans 'System.exe' */
	enum IconVisibility{
		IV_HIDE = 1, /*!< Masque */
		IV_SHOW = 2, /*!< Vue */
	};

	/** Constructeur */
	CCasioSystem();

	/** Destructeur */
	virtual ~CCasioSystem();

	int  OpenDriveFile(const string& sPath, int nShift, bool bReadWriteMode = true);
	int  OpenDriveFile(void* pData, unsigned int nMaxSize);
	int  CloseDrive();
	bool IsValid();
	int  GetSizeInBytes() const;

	/** Retourne le type de calculatrice du menu */
	CasioModel GetSystemType() const;

	/** Permet de rcuprer les menus du systme
	 * @param[out] pMenus Zone mmoire d'au moins CCasioSystem::SIZEBMP * CCasioSystem::MAXMENU octets
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   GetMenus(void* pMenus);

	/** Permet de rcuprer un menu du systme
	 * @param[out] pMenu Zone mmoire d'au moins CCasioSystem::SIZEBMP octets
	 * @param[in] nNumMenu Numro du menu dsir.
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   GetMenu(void* pMenu, int nNumMenu);

	/** Permet de rcuprer le titre d'un menu du systme
	 * @param[out] pTitle Titre d'au moins CCasioSystem::MENU_BYTE_PER_ROW * CCasioSystem::TITLE_HEIGHT octets
	 * @param[in] nNumMenu Numro du menu dsir.
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   GetTitle(void* pTitle, int nNumMenu);

	/** Permet de rcuprer les informations du systme
	 * @param[out] pInfo Tableau d'informations de CCasioSystem::MAXICON lments
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   GetInfo(TInfo* pInfo);

	/** Permet de dfinir les menus du systme
	 * @param[in] pMenus Menu d'au moins CCasioSystem::SIZEBMP * CCasioSystem::MAXMENU octets
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   SetMenus(const void* pMenus);

	/** Permet de dfinir un menu du systme
	 * @param[in] pMenu Menu d'au moins CCasioSystem::SIZEBMP octets
	 * @param[in] nNumMenu Numro du menu dsir.
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   SetMenu(const void* pMenu, int nNumMenu);

	/** Permet de dfinir le titre d'un menu du systme
	 * @param[in] pTitle Titre d'au moins CCasioSystem::MENU_BYTE_PER_ROW * CCasioSystem::TITLE_HEIGHT octets
	 * @param[in] nNumMenu Numro du menu dsir.
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   SetTitle(const void* pTitle, int nNumMenu);

	/** Permet de rcuprer les informations du systme
	 * @param[in] pInfo Tableau d'informations de CCasioSystem::MAXICON lments
	 * @return Retourne vrai si l'opration a russi
	 */
	bool   SetInfo(const TInfo* pInfo);
	
	/** Permet de savoir combien d'icnes dispose le systme
	 * @return Retourne le nombre d'icne ou -1 si erreur
	 * @see GetNbIcon(const TInfo*)
	 */
	int    GetNbIcon();

	/** Permet de savoir combien de menus dispose le systme
	 * @return Retourne le nombre de menu ou -1 si erreur
	 * @see GetNbMenu(int)
	 */
	int    GetNbMenu();

	/** Ajoute une icne  la fin d'un systme
	 * @param[in] pIconInfo Informations sur l'icne  ajouter ( NULL si on veut ajouter une icne vide)
	 * @param[in] pIconImage Image de l'icne. ( NULL pour spcifier pas d'image)
	 * @param[in] bReportTitle A vrai si on veut que le titre du dernier menu soit report sur un possible nouveau menu engendr par la cration de l'icne
	 * @param[in] bCpyLastIcon A vrai si on veut que les icnes du dernier menu soit reportes sur un possible nouveau menu engendr par la cration de l'icne
	 * @param[in] bShowKey A vrai si on veut afficher le numro de la touche de raccourci
	 * @return Retourne vrai si l'icne est rajoute
	 * @see AddIcon(const TInfo*, const void*, TInfo*, void*, bool, bool, bool)
	 */
	bool   AddIcon(const TInfo* pIconInfo, const void* pIconImage, bool bReportTitle, bool bCpyLastIcon, bool bShowKey);

	/** Ajoute une icne dans un systme
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si l'icne est supprime
	 * @see RemoveIcon(TInfo*, void*, int)
	 */
	bool   RemoveIcon(int nNumIcon);

	/** Dplace une icne  gauche en la permutant avec l'icne la prcdant
	 * @param[in] nNumIcon Numro de l'icne
	 * @see ShiftIconLeft(TInfo*, void*, int)
	 */
	bool   ShiftIconLeft(int nNumIcon);

	/** Dplace une icne  droite en la permutant avec l'icne la devanant
	 * @param[in] nNumIcon Numro de l'icne
	 * @see ShiftIconRight(TInfo*, void*, int)
	 */
	bool   ShiftIconRight(int nNumIcon);

	/** Permet de rcuprer l'image d'une icne
	 * @param[out] pIconImage Donnes de l'image au format d'une icne 
	 * @param[in] nNumIcon Numro de l'icne dans le systme
	 * @param[in] nInMenu Numro indiquant dans quel menu est dfini l'icne. Numro de 0  2, cela dpend de la position
	 * de l'icne. En effet, une icne peut apparaitre au maximum sur trois menus.
	 * @return Retourne 1 pour OK \n -1 erreur paramtres \n -2 nInMenu invalide \n -3 erreur d'accs systme
	 * @see GetIconImage(const TInfo*, const void*, void*, int, int)
	 */
	int    GetIconImage(void* pIconImage, int nNumIcon, int nInMenu);

	/** Permet de dfinir l'image d'une icne
	 * @param[in] pIconImage Donnes de l'image au format d'une icne 
	 * @param[in] nNumIcon Numro de l'icne dans le systme
	 * @param[in] nInMenu Numro indiquant dans quel menu est dfini l'icne. Numro de 0  2, cela dpend de la position
	 * de l'icne. En effet, une icne peut apparaitre au maximum sur trois menus. (-1 pour indiquer tous les menus)
	 * @return Retourne 1 pour OK \n -1 erreur paramtres \n -2 nInMenu invalide \n -3 erreur d'accs systme
	 * @see SetIconImage(const TInfo*, void*, const void*, int, int)
	 */
	int    SetIconImage(const void* pIconImage, int nNumIcon, int nInMenu);

	/** Permet de connaitre l'tat de visibilit de l'icne dans 'System.exe'
	 * @param[in] nNumIcon Numro de l'icne
	 * @remarks Si on demande  voir une icne toutes les autres icnes add-ins prcdant cette icne seront aussi affectes
	 * @return Retourne l'tat ou -1 si erreur
	 * @see IconVisibility
	 * @see GetIconVisibility(const TInfo*, int)
	 */
	int    GetIconVisibility(int nNumIcon);

	/** Permet d'activer ou non l'affichage du chemin point par une icne dans 'System.exe'
	 * @param[in] nNumIcon Numro de l'icne
	 * @param[in] nState A vrai si vu dans 'System.exe'
	 * @remarks Si on demande  voir une icne toutes les autres icnes add-ins prcdant cette icne seront aussi affectes
	 * @return Retourne vrai si russi
	 * @see IconVisibility
	 * @see SetIconVisibility(TInfo*, int, int)
	 */
	bool   SetIconVisibility(int nNumIcon, int nState);

	/** Permet de connaitre le chemin que lancera un raccourci
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne le chemin sous forme de string
	 * @see GetIconPath(const TInfo*, int)
	 */
	string GetIconPath(int nNumIcon);

	/** Permet de dfinir le chemin que lancera un raccourci
	 * @param[in] sPath Chemin  lancer
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si le chemin est bien appliqu
	 * @see SetIconPath(TInfo*, const string&, int)
	 */
	bool   SetIconPath(const string& sPath, int nNumIcon);

	/** Permet de connaitre les arguments passs d'un raccourci
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne les arguments sous forme de string
	 * @see GetIconArguments(const TInfo*, int)
	 */
	string GetIconArguments(int nNumIcon);

	/** Permet de dfinir les paramtres passs d'un raccourci
	 * @param[in] sArg Paramtres
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 * @see SetIconArguments(TInfo*, const string&, int)
	 */
	bool   SetIconArguments(const string& sArg, int nNumIcon);

	/** Permet de connaitre le nom de l'image d'un raccourci
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne le nom de l'image
	 * @see GetIconImageName(const TInfo*, int)
	 */
	string GetIconImageName(int nNumIcon);
	
	/** Permet de dfinir le nom de l'image d'un raccourci
	 * @param[in] sImageName Nom de l'image
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 * @see SetIconImageName(TInfo*, const string&, int)
	 */
	bool   SetIconImageName(const string& sImageName, int nNumIcon);

	/** Permet de connaitre la version du fichier point par le raccourci
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne la version
	 * @see GetIconVersion(const TInfo*, int)
	 */
	string GetIconVersion(int nNumIcon);

	/** Permet de dfinir la version du fichier point par le raccourci
	 * @param[in] sVersion Version du fichier
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 * @see SetIconVersion(TInfo*, const string&, int)
	 */
	bool   SetIconVersion(const string& sVersion, int nNumIcon);

	/** Permet de dessiner une touche de raccourci dans un menu du systme au format Casio
	 * @param[in] nNumIcon Numro de l'icne de [0, CCasioSystem::MAXICON[
	 */
	bool   PutKey(int nNumIcon);

public:
	/** Permet de savoir quel est le premier menu qui peut afficher cette icne
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne l'index du premier menu
	 */
	static int  GetFirstMenuIndex(int nNumIcon);

	/** Permet de savoir quel est le dernier menu qui peut afficher cette icne
	 * @param[in] nNumIcon Numro de l'icne
	 * @param[in] nNbMenu Nombre de menus
	 * @return Retourne l'index du premier menu
	 */
	static int  GetLastMenuIndex(int nNumIcon, int nNbMenu);

	/** Permet d'avoir la position du lecteur A inclut dans le systme
	 * @return Retourne la position du lecteur en octets
	 */
	static int  GetDrivePosition();
	
	/** Permet de savoir si une zone mmoire est valide
	 * @param[in] buffer Donnes du systme
	 * @return Retourne le modle du systme
	 * @see CasioModel
	 */
	static int IsValid(void* buffer);

	/** Permet de savoir si un fichier est valide
	 * @param[in] sPath Fichier  tester
	 * @param[in] nShift Dcalage en octets
	 * @return Retourne le modle du systme
	 * @see CasioModel
	 */
	static int IsValid(const string& sPath, int nShift = 0);

	/** Permet de connaitre le nombre d'icnes d'un tableau d'informations
	 * @param[in] pInfo Tableau d'informations de CCasioSystem::MAXICON lments
	 * @return Retourne le nombre d'icnes ou -1 si erreur
	 */
	static int  GetNbIcon(const TInfo* pInfo);

	/** Permet de connaitre le nombre de menu en fonction d'un nombre d'icne
	 * @param[in] nNbIcon Nombre d'icnes
	 * @return Retourne le nombre de menus ou -1 si erreur
	 */
	static int  GetNbMenu(int nNbIcon);

	/** Efface les informations d'une icne
	 * @param[out] info Zone  effacer
	 */
	static void ClearZoneInfo(TInfo& info);

	/** Cre les informations d'une nouvelle une icne
	 * @param[out] info Zone  remplir
	 * @param[in] nPos Position de l'icne
	 */
	static void NewZoneInfo(TInfo& info, int nPos);

	/** Permet de rcuprer le titre d'un menu du systme
	 * @param[in] pMenu Menu d'au moins CCasioSystem::SIZEBMP octets
	 * @return Retourne la zone qui pointe sur le titre du menu
	 */
	static const void* GetTitle(const void* pMenu);

	/** Permet de dfinir le titre d'un menu du systme
	 * @param[out] pMenu Menu d'au moins CCasioSystem::SIZEBMP octets
	 * @param[in] pTitle Titre d'au moins CCasioSystem::MENU_BYTE_PER_ROW * CCasioSystem::TITLE_HEIGHT octets
	 */
	static void SetTitle(void* pMenu, const void* pTitle);

	/** Ajoute une icne  la fin d'un systme
	 * @param[in] pIconInfo Informations sur l'icne  ajouter ( NULL si on veut ajouter une icne vide)
	 * @param[in] pIconImage Image de l'icne. ( NULL pour spcifier pas d'image)
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in,out] pMenus Pointeur sur les menus
	 * @param[in] bReportTitle A vrai si on veut que le titre du dernier menu soit report sur un possible nouveau menu engendr par la cration de l'icne
	 * @param[in] bCpyLastIcon A vrai si on veut que les icnes du dernier menu soit report sur un possible nouveau menu engendr par la cration de l'icne
	 * @param[in] bShowKey A vrai si on veut afficher le numro de la touche de raccourci
	 * @return Retourne la zone ajoute ou NULL si erreur
	 * @remarks Si le numro de l'icne indiqu dans \a pIconInfo n'est pas correct celui-ci est corrig dans le tableau \a pInfo
	 */
	static TInfo* AddIcon(const TInfo* pIconInfo, const void* pIconImage, TInfo* pInfo, void* pMenus, bool bReportTitle, bool bCpyLastIcon, bool bShowKey);

	/** Supprime une icne dans un systme
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in,out] pMenus Pointeur sur les menus
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si l'icne est supprime
	 */
	static bool RemoveIcon(TInfo* pInfo, void* pMenus, int nNumIcon);

	/** Dplace une icne  gauche en la permutant avec l'icne la prcdant
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in,out] pMenus Pointeur sur les menus
	 * @param[in] nNumIcon Numro de l'icne
	 */
	static bool ShiftIconLeft(TInfo* pInfo, void* pMenus, int nNumIcon);

	/** Dplace une icne  droite en la permutant avec l'icne la devanant
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in,out] pMenus Pointeur sur les menus
	 * @param[in] nNumIcon Numro de l'icne
	 */
	static bool ShiftIconRight(TInfo* pInfo, void* pMenus, int nNumIcon);

	/** Permet de connaitre l'tat de visibilit de l'icne dans 'System.exe'
	 * @param[in] pInfo Zone d'informations
	 * @param[in] nNumIcon Numro de l'icne
	 * @remarks Si on demande  voir une icne toutes les autres icnes add-ins prcdant cette icne seront aussi affectes
	 * @see IconVisibility
	 */
	static int  GetIconVisibility(const TInfo* pInfo, int nNumIcon);

	/** Permet d'activer ou non l'affichage du chemin point par une icne dans 'System.exe'
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in] nNumIcon Numro de l'icne
	 * @param[in] nState A vrai si vu dans 'System.exe'
	 * @remarks Si on demande  voir une icne toutes les autres icnes add-ins prcdant cette icne seront aussi affectes
	 * @return Retourne l'tat ou -1 si erreur
	 * @see IconVisibility
	 */
	static bool SetIconVisibility(TInfo* pInfo, int nNumIcon, int nState);

	/** Permet de rcuprer l'image d'une icne
	 * @param[in] pInfo Zone d'informations
	 * @param[in] pMenus Pointeur sur les menus
	 * @param[out] pIconImage Donnes de l'image au format d'une icne 
	 * @param[in] nNumIcon Numro de l'icne dans le systme
	 * @param[in] nInMenu Numro indiquant dans quel menu est dfini l'icne. Numro de 0  2, cela dpend de la position
	 * de l'icne. En effet, une icne peut apparaitre au maximum sur trois menus.
	 * @return Retourne 1 pour OK \n -1 erreur paramtres \n -2 nInMenu invalide
	 */
	static int  GetIconImage(const TInfo* pInfo, const void* pMenus, void* pIconImage, int nNumIcon, int nInMenu);

	/** Permet de dfinir l'image d'une icne
	 * @param[in] pInfo Zone d'informations
	 * @param[in,out] pMenus Pointeur sur les menus
	 * @param[in] pIconImage Donnes de l'image au format d'une icne 
	 * @param[in] nNumIcon Numro de l'icne dans le systme
	 * @param[in] nInMenu Numro indiquant dans quel menu est dfini l'icne. Numro de 0  2, cela dpend de la position
	 * de l'icne. En effet, une icne peut apparaitre au maximum sur trois menus. (-1 pour indiquer tous les menus)
	 * @return Retourne 1 pour OK \n -1 erreur paramtres \n -2 nInMenu invalide
	 */
	static int  SetIconImage(const TInfo* pInfo, void* pMenus, const void* pIconImage, int nNumIcon, int nInMenu);

	/** Permet de connaitre le chemin que lancera un raccourci
	 * @param[in] pInfo Zone d'informations
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si le chemin est bien appliqu
	 */
	static string GetIconPath(const TInfo* pInfo, int nNumIcon);

	/** Permet de dfinir le chemin que lancera un raccourci
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in] sPath Chemin  lancer
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 */
	static bool   SetIconPath(TInfo* pInfo, const string& sPath, int nNumIcon);

	/** Permet de connaitre les arguments passs d'un raccourci
	 * @param[in] pInfo Zone d'informations
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne les arguments sous forme de string
	 */
	static string GetIconArguments(const TInfo* pInfo, int nNumIcon);

	/** Permet de dfinir les paramtres passs d'un raccourci
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in] sArg Paramtres
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 */
	static bool   SetIconArguments(TInfo* pInfo, const string& sArg, int nNumIcon);	

	/** Permet de connaitre le nom de l'image d'un raccourci
	 * @param[in] pInfo Zone d'informations
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne le nom de l'image
	 */
	static string GetIconImageName(const TInfo* pInfo, int nNumIcon);
	
	/** Permet de dfinir le nom de l'image d'un raccourci
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in] sImageName Nom de l'image
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 */
	static bool   SetIconImageName(TInfo* pInfo, const string& sImageName, int nNumIcon);

	/** Permet de connaitre la version du fichier point par le raccourci
	 * @param[in] pInfo Zone d'informations
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne la version
	 */
	static string GetIconVersion(const TInfo* pInfo, int nNumIcon);

	/** Permet de dfinir la version du fichier point par le raccourci
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in] sVersion Version du fichier
	 * @param[in] nNumIcon Numro de l'icne
	 * @return Retourne vrai si russi
	 */
	static bool   SetIconVersion(TInfo* pInfo, const string& sVersion, int nNumIcon);

	/** Permet de connaitre l'tat d'un pixel
	 * @param[in] pBuf
	 * @param[in] x Position x
	 * @param[in] y Position y
	 * @param[in] nBytePerRow Nombre d'octets par ligne sur l'image
	 * @return Retourne 1 si le pixel est allum, sinon 0
	 */
	static int    GetPixel(const void* pBuf, int x, int y, int nBytePerRow);

	/** Permet de dfinir l'tat d'un pixel
	 * @param[out] pBuf
	 * @param[in] x Position x
	 * @param[in] y Position y
	 * @param[in] nBytePerRow Nombre d'octets par ligne sur l'image
	 * @param[in] nVal Etat du pixel
	 */
	static void   PutPixel(void* pBuf, int x, int y, int nBytePerRow, int nVal);

	/** Permet de dessiner une ligne verticale
	 * @param[out] pBuf Zone mmoire o dessiner
	 * @param[in] y1 Position infrieure verticale
	 * @param[in] y2 Position suprieure verticale
	 * @param[in] x Position horizontale
	 * @param[in] nBytePerRow Nombre d'octets par ligne
	 * @param[in] nVal Couleur de la ligne
	 * @warning y1 doit tre infrieure  y2
	 */
	static void   vLine(void* pBuf, int y1, int y2, int x, int nBytePerRow, int nVal);

	/** Permet de dessiner une ligne horizontale
	 * @param[out] pBuf Zone mmoire o dessiner
	 * @param[in] x1 Position infrieure horizontale
	 * @param[in] x2 Position suprieure horizontale
	 * @param[in] y Position verticale
	 * @param[in] nBytePerRow Nombre d'octets par ligne
	 * @param[in] nVal Couleur de la ligne
	 * @warning x1 doit tre infrieure  x2
	 */
	static void   hLine(void* pBuf, int x1, int x2, int y, int nBytePerRow, int nVal);

	/** Permet de dessiner une touche de raccourci
	 * @param[out] pBuf Zone mmoire o dessiner
	 * @param[in] nPosition Numro de l'icne de [0, CCasioSystem::MAXICON[
	 * @param[in] nDx Position gauche
	 * @param[in] nDy Position haute
	 * @param[in] nBytePerRow Nombre d'octets par ligne
	 */
	static bool   PutKey(void* pBuf, int nPosition, int nDx, int nDy, int nBytePerRow);

	/** Permet de dessiner une touche de raccourci sur une icne  l'emplacement par dfaut du style Casio
	 * @param[out] pIconImage Zone mmoire o dessiner
	 * @param[in] nPosition Numro de l'icne de [0, CCasioSystem::MAXICON[
	 */
	static bool   PutKey(void* pIconImage, int nPosition);

	/** Copie une chaine de caractre pour la sauvegarde de donnes des informations sur les icnes
	 * @param[out] pBuf Zone de destications
	 * @param[in] pSource Chaine  copier
	 * @param[in] nMaxSize Taille maximale de la zone mmoire destinatrice
	 */
	static void   CpyChar(char* pBuf, const char* pSource, int nMaxSize);

	/** Permet de basculer l'image d'un menu du format d'un PC  celui de la Casio
	 * @param[in,out] pPCBuf Menu du PC
	 * @param[in,out] pCasioBuf Menu de la Casio
	 * @param[in] bCasioToPC A vrai si l'on veut envoyer les donnes de la Casio vers le PC, sinon  faux
	 */
	static void   InvertBuffer(void* pPCBuf, void* pCasioBuf, bool bCasioToPC);

protected:
	/** Copie les huit dernires icnes du menu source sur le menu destinataire en tant que 8 premires
	 * @param[out] pDest Menu destinataire (CCasioSystem::SIZEBMP octects minimum)
	 * @param[in] pSource Menu source (CCasioSystem::SIZEBMP octects minimum)
	 */
	static void   Cpy8LastIcon(void* pDest, const void* pSource);

	/** Copie une icne d'un menu ou autre, dans une autre zone spcifie
	 * @param[out] pDest Zone mmoire destinataire
	 * @param[in] nDestPos Position de l'icne dans le menu destinataire
	 * @param[in] pSrc Zone mmoire source
	 * @param[in] nSrcPos Position de l'icne dans le menu source
	 * @param[in] bInv A vrai si l'image source est inverse
	 * @param[in] nDestBytePerRow Nombre d'octets par ligne sur l'image de destination
	 * @param[in] nSrcBytePerRow Nombre d'octets par ligne sur l'image source
	 * @remarks Si nDestPos == -1 et/ou nSourcePos == -1 l'image est lue ou copie ds le premier octet de la zone mmoire
	 */
	static void   CpyIcon(void* pDest, int nDestPos, const void* pSrc, int nSrcPos, int bInv = 0, int nDestBytePerRow = MENU_BYTE_PER_ROW, int nSrcBytePerRow = MENU_BYTE_PER_ROW);

	/** Dplace une icne  gauche en la permutant avec l'icne la prcdant
	 * @param[in,out] pInfo Zone d'informations
	 * @param[in,out] pMenus Pointeur sur les menus
	 * @param[in] nNumIcon Numro d'icne
	 */
	static bool   MoveIconLeft(TInfo* pInfo, void* pMenus, int nNumIcon);

	/** Permet d'effacer une icne d'un menu
	 * @param[out] pDest Menu destinataire
	 * @param[in] nDestPos Position de l'icne dans le menu
	 */
	static void   ClearIcon(void* pDest, int nDestPos);
	
	/** Permute deux icnes
	 * @param[in,out] pBuf Menu des icnes
	 * @param[in] n1 Premire icne
	 * @param[in] n2 Deuxime icne
	 */
	static void   PermuteIcon(void* pBuf, int n1, int n2);

private:
	bool       m_bIsValid; ///< A vrai si le systme est valide
	CasioModel m_nG100Type;///< Type du menu
};

#endif
