
#ifndef DRIVEACCESS_H
#define DRIVEACCESS_H


#include <string>
using namespace std;

#include <F100_Op.h>

#include "APIFunction.h"

/** Taille maximale d'un lecteur */
#define DRIVE_SIZE 131072

/*!
	\class CDriveAccess DriveAccess.h
	\brief Fourni une collection de mthodes pour accder  des fichiers contenant des donnes d'un lecteur
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
*/
class DLLEXPORT CDriveAccess  
{
public:
	/** Permet de connaitre le nom d'un lecteur
	 * @param[in] nDrive Numro du lecteur de 0  6
	 * @return Retourne le nom du lecteur \a nDrive, ou NULL si non valide
	 */
	static char* GetDriveName(int nDrive);

	/** Permet de savoir si une zon mmoire est vide
	 * @param[in] buffer Zone mmoire  scanner
	 * @param[in] nSize Taille de la zone  scanner
	 * @return Retourne vrai si la zone mmoire est vide
	 */
	static bool  IsEmpty(const void* buffer, int nSize);

	/** Permet de savoir si un fichier est vide
	 * @param[in] h Zone mmoire  scanner
	 * @param[in] nShift Dcalage en octets
	 * @param[in] nSize Taille en octets  lire. Si infrieur  0 alors tout le fichier est lu
	 * @return Retourne vrai si le fichier (ou la partie demande) est vide
	 */
	static bool  IsEmptyH(HND h, int nShift, int nSize = -1);

	/** Permet de connaitre le nombre de fichier ncessaire pour une taille donne
	 * @param[in] nSize Taille du lecteur en octets
	 * @param[in] bOneFile A vrai si le fichier peut dpasser 128Ko
	 * @return Retourne le nombre de fichiers ncessaire pour la taille \a nSize
	 * @remarks Si bOneFile est  vrai alors le nombre retourn sera toujours gal  1
	 */
	static int   GetNbFile(int nSize, bool bOneFile = false);

	/** Permet de changer le nom d'un fichier en fonction de son numro
	 * @param[in,out] sFileName Nom du fichier
	 * @param[in] nNumDrive Numro du fichier
	 * @return Retourne le nom du fichier modifi
	 */
	static string&  ChangeFileName(string& sFileName, int nNumDrive);

	/** Permet de lire ou d'crire sur un fichier
	 * @param[in,out] h Handle du fichier
	 * @param[in,out] name Nom du fichier
	 * @param[in,out] buffer Donnes  crire ou lire
	 * @param[in] nBytes Nombre d'octets  crire ou lire
	 * @param[in] numDrive Numro du lecteur
	 * @param[in] WriteMode A vrai si on crit, sinon  faux
	 * @param[in] bOneFile A vrai si on veut qu'un seul fichier pouvant tre suprieur  DRIVE_SIZE
	 * @param[in] nHeaderSize Taille de l'en-tte pour chaque lecteur. (DRIVE_SIZE octets)
	 * @return Retourne vrai si russi
	 * @remarks Change automatiquement de fichier si dpassement
	 * @remarks Si lors de l'appel de cette mthode le pointeur courant du fichier est infrieur  l'en-tte, la lecture
	 * ou l'criture se fera avant l'en-tte.(Ceci permet de lire ou crire des donnes dans l'en-tte)
	 */
	static bool  AccessInDrive(HND& h, string& name, void* buffer, int nBytes, int& numDrive, bool WriteMode, bool bOneFile = false, unsigned int nHeaderSize = 0);

	/** Retourne la taille utile du lecteur. (Retourne la position du dernier caractre diffrent de 0xFF)
	 * @param[in] nDrive de 0  6
	 * @param[in] buffer Donnes de DRIVE_SIZE minimum
	 * @return Retourne la taille utile du lecteur
	 * @remarks Retourne la position du dernier caractre diffrent de 0xFF
	 */
	static int   GetSizeUseful(int nDrive, const void* buffer);

	/** Permet de savoir si le nom d'un fichier n'est pas un fichier compos
	 * @param[in] sFileName Nom du fichier
	 * @return Retourne vrai si le fichier spcifi n'est pas dcoup en plusieurs fichiers
	 */
	static bool  IsOnlyOneFile(const string& sFileName);

	/** Permet de connaitre le numro de lecteur associ  un nom de fichier
	 * @param[in] sFileName Nom du fichier
	 * @return Retourne 0 si non dcoup, et une valeur > 0 pour indiquer le numro de la partition
	 */
	static int   GetNumDrive(const string& sFileName);

private:
	/** Permet de connaitre la position de _L dans le nom du fichier
	 * @param[in] sFileName Nom du fichier
	 * @return -1 Non trouv\n >= 0 Position de _L
	 */
	static int   GetPartionPos(const string& sFileName);
};

#endif
