
#include "DriveFile.h"

#include "DriveAccess.h"
#include "utils.h"

#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CDriveFile::CDriveFile(bool bOneFile, bool bVariableSize, unsigned int nHeaderSize)
{
	m_nNumDrive      = 1;
	m_nShift         = 0;
	m_bOneFile       = bOneFile;
	m_hFile          = INVALID_HANDLE_VALUE;
	m_pBuffer	     = NULL;
	m_nMaxSize       = 0;
	m_bVariableSize  = bVariableSize;
	m_nHeaderSize    = nHeaderSize;
	m_bReadWriteMode = false;
}

//-----------------------------------------------------------------------------------
CDriveFile::~CDriveFile()
{
}

//-----------------------------------------------------------------------------------
bool CDriveFile::IsEmpty(int nMaxSize)
{
	bool bIsValid = SetPosition(0) > 0;
	if(bIsValid)
	{
		if(m_pBuffer)
		{
			return CDriveAccess::IsEmpty(m_pBuffer, nMaxSize);
		}
		if(m_hFile != INVALID_HANDLE_VALUE)
		{
			return CDriveAccess::IsEmptyH(m_hFile, m_nShift, nMaxSize);
		}
	}
	return bIsValid;
}

//-----------------------------------------------------------------------------------
int CDriveFile::OpenDriveFile(const string& sPath, int nShift, bool bReadWriteMode)
{
	if(!sPath.empty())
	{
		string sTmpPath(sPath);
		if(!m_bOneFile && !CDriveAccess::IsOnlyOneFile(sTmpPath)) {
			CDriveAccess::ChangeFileName(sTmpPath, 1);
		}
		HND h;
		if(bReadWriteMode) {
			h = OpenDesc(sTmpPath.c_str(), READWRITE, OPEN_ALWAYS);
		}
		else {
			h = OpenDescModeRead(sTmpPath.c_str());
		}
		if(h == INVALID_HANDLE_VALUE) {
			return -2;
		}
		CloseDrive();
		m_hFile          = h;
		m_bReadWriteMode = bReadWriteMode;
		m_pBuffer        = NULL;
		m_nShift         = nShift;
		m_sFileName      = sTmpPath;
		return SetPosition(0);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CDriveFile::OpenDriveFile(void* pData, unsigned int nMaxSize)
{
	if(pData)
	{
		CloseDrive();
		m_sFileName.erase();
		m_nShift         = 0;
		m_nMaxSize       = nMaxSize;
		m_bReadWriteMode = true;
		m_pBuffer        = pData;
		return SetPosition(0);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CDriveFile::CloseDrive()
{
	int nRet = 1;
	if(m_bReadWriteMode) {
		nRet = WriteHeaders();
	}
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		if(IsValid() && m_bReadWriteMode)
		{
			//Redimmensionne le dernier fichier du lecteur
			int nDriveSize = GetTotalSizeInBytes();
			if(nDriveSize >= 0 && SetPosition(nDriveSize) > 0) {
				unsigned int nPos = nDriveSize - (CDriveAccess::GetNbFile(nDriveSize) - 1) * DRIVE_SIZE;
				if(!m_bVariableSize)
				{
					unsigned int nWrite = DRIVE_SIZE - nPos;
					char* pBuffer = new char[nWrite];
					if(pBuffer == NULL)	{
						//Mmoire insuffisante!
						nRet = -2;
					}
					else {
						memset(pBuffer, 0xFF, nWrite);
						nRet = Write(pBuffer, nWrite);
						delete[] pBuffer;
					}
				}
				else {
					if(m_bOneFile) {
						SetFileSize(m_hFile, nDriveSize + m_nShift);
					}
					else {
						SetFileSize(m_hFile, nPos + m_nShift);
					}
				}
			}
			else {
				nRet = -1;
			}
		}
		CloseDesc(m_hFile);
		if(!CDriveAccess::IsOnlyOneFile(m_sFileName) && GetNumDrive() == 1) {
			string sNew = m_sFileName;
			CDriveAccess::ChangeFileName(m_sFileName, 1);
			CDriveAccess::ChangeFileName(sNew,        0);
			MoveOrRenameFile(m_sFileName.c_str(), sNew.c_str());
		}
	}
	m_pBuffer = NULL;
#ifdef _DEBUG
	if(nRet < 1) {
		_WARNING1("CDriveFile::CloseDrive() returns %i", nRet);
	}
#endif
	return nRet;
}

//-----------------------------------------------------------------------------------
int CDriveFile::GetNumDrive() const
{
	int nSize = GetSizeInBytes();
	if(nSize >= 0) {
		return CDriveAccess::GetNbFile(nSize);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CDriveFile::GetTotalSizeInBytes() const
{
	int nSize = GetSizeInBytes();
	if(nSize >= 0) {
		return ConvertPositionToRealPosition(nSize) - m_nShift;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CDriveFile::SetPosition(unsigned int nPos)
{
	return SetRealPosition(ConvertPositionToRealPosition(nPos));
}

//-----------------------------------------------------------------------------------
int CDriveFile::SetRealPosition(unsigned int nPos)
{
	if(m_pBuffer != NULL) {
		if(nPos >= m_nMaxSize) {
			return -1;
		}
		m_nSeekPos = nPos;
		return 1;
	}
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		int n = 1;
		if(!m_bOneFile) {
			n = (nPos / DRIVE_SIZE) + 1;
		}
		if(n != m_nNumDrive)
		{
			CloseDesc(m_hFile);
			CDriveAccess::ChangeFileName(m_sFileName, n);
			m_nNumDrive = n;
			if(m_bReadWriteMode) {
				m_hFile = OpenDesc(m_sFileName.c_str(), READWRITE, OPEN_ALWAYS);
			}
			else {
				m_hFile = OpenDescModeRead(m_sFileName.c_str());
			}
			if(m_hFile == INVALID_HANDLE_VALUE)	{
				return -1;
			}
		}
		SeekDesc(m_hFile, nPos - DRIVE_SIZE * (m_nNumDrive - 1));
		return 1;
	}
	return -2;
}

//-----------------------------------------------------------------------------------
int CDriveFile::Write(const void* pData, unsigned int nSize)
{
	if(!IsValid() && m_bReadWriteMode) {
		return -1;
	}
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		if(CDriveAccess::AccessInDrive(m_hFile, m_sFileName, (void*)pData, nSize, m_nNumDrive, true, m_bOneFile, m_nHeaderSize)) {	
			return 1;
		}
		return -1;
	}
	if(m_pBuffer != NULL)
	{
		if(m_nSeekPos + nSize >= m_nMaxSize) {
			//Buffer overflow
			return -1;
		}
		if(m_nHeaderSize == 0) {
			memcpy(((char*)m_pBuffer) + m_nSeekPos, pData, nSize);
			m_nSeekPos += nSize;
		}
		else{
			int nPos = m_nSeekPos % DRIVE_SIZE;
			int max  = m_nSeekPos + nSize;
			while(nSize > 0) {
				int nMax = DRIVE_SIZE - m_nSeekPos % DRIVE_SIZE;
				if(nSize > nMax) {
					memcpy(((char*)m_pBuffer) + m_nSeekPos, pData, nMax);
					m_nSeekPos += nMax + m_nHeaderSize;
					nSize      -= nMax;
				}
				else{
					memcpy(((char*)m_pBuffer) + m_nSeekPos, pData, nSize);
					m_nSeekPos += nSize;
					nSize       = 0;
				}

			}
		}
		return 1;
	}
	return -2;//Rien d'ouvert
}

//-----------------------------------------------------------------------------------
int CDriveFile::Read(void* pData, unsigned int nSize)
{
	if(!IsValid())
	{
		return -1;
	}
	if(m_hFile != INVALID_HANDLE_VALUE)
	{
		if(CDriveAccess::AccessInDrive(m_hFile, m_sFileName, (void*)pData, nSize, m_nNumDrive, false, m_bOneFile, m_nHeaderSize))
		{
			return 1;
		}
		return -1;
	}
	if(m_pBuffer != NULL)
	{
		if(m_nSeekPos + nSize >= m_nMaxSize) {
			//Buffer overflow
			return -1;
		}
		if(m_nHeaderSize == 0) {
			memcpy(pData, ((char*)m_pBuffer) + m_nSeekPos, nSize);
			m_nSeekPos += nSize;
		}
		else{
			int nPos = m_nSeekPos % DRIVE_SIZE;
			int max  = m_nSeekPos + nSize;
			while(nSize > 0) {
				int nMax = DRIVE_SIZE - m_nSeekPos % DRIVE_SIZE;
				if(nSize > nMax) {
					memcpy(pData, ((char*)m_pBuffer) + m_nSeekPos, nMax);
					m_nSeekPos += nMax + m_nHeaderSize;
					nSize      -= nMax;
				}
				else{
					memcpy(pData, ((char*)m_pBuffer) + m_nSeekPos, nSize);
					m_nSeekPos += nSize;
					nSize       = 0;
				}
			}
		}
		return 1;
	}
	return -2;//Rien d'ouvert
}

//-----------------------------------------------------------------------------------
unsigned int CDriveFile::ConvertPositionToRealPosition(unsigned int nPos) const
{
	int n1 = CDriveAccess::GetNbFile(nPos);
	nPos += n1 * m_nHeaderSize;
	int n2 =  CDriveAccess::GetNbFile(nPos);
	return nPos + (n2 - n1) * m_nHeaderSize + m_nShift;
}

//-----------------------------------------------------------------------------------
int CDriveFile::WriteHeader(int nNbHeader, const void* pBuffer)
{
	if(m_nHeaderSize == 0) {
		return -3;
	}
	if(pBuffer == NULL || nNbHeader < 0 || nNbHeader >= GetNumDrive()) {
		return -4;
	}
	SetRealPosition(DRIVE_SIZE * nNbHeader + m_nShift);
	return Write(pBuffer, m_nHeaderSize);
}

//-----------------------------------------------------------------------------------
int CDriveFile::ReadHeader(int nNbHeader, void* pBuffer)
{
	if(m_nHeaderSize == 0) {
		return -3;
	}
	if(pBuffer == NULL || nNbHeader < 0 || nNbHeader >= GetNumDrive()) {
		return -4;
	}
	SetRealPosition(DRIVE_SIZE * nNbHeader + m_nShift);
	return Read(pBuffer, m_nHeaderSize);
}

//-----------------------------------------------------------------------------------
void* CDriveFile::GetHeader(int nNbHeader)
{
	return NULL;
}

//-----------------------------------------------------------------------------------
int CDriveFile::WriteHeaders()
{
	int nRet = 1;
	if(m_nHeaderSize > 0) {
		int nNumDrive = GetNumDrive();
		if(nNumDrive < 1) {
			return -1;
		}
		for(int i = 0; i < nNumDrive && nRet > 0; i++) {
			nRet = WriteHeader(i, GetHeader(i));
		}
	}
	return nRet;
}

//-----------------------------------------------------------------------------------
bool CDriveFile::HeadersAreValid()
{
	bool bValid = true;
	if(m_nHeaderSize > 0) {
		int nNumDrive = GetNumDrive();
		if(nNumDrive < 1) {
			return false;
		}
		char* pBuffer = new char[m_nHeaderSize];
		bValid = (pBuffer != NULL);
		for(int i = 0; i < nNumDrive && bValid; i++) {
			bValid = (ReadHeader(i, pBuffer) > 0
				&& memcmp(pBuffer, GetHeader(i), m_nHeaderSize) == 0);
		}
		delete[] pBuffer;
	}
	return bValid;
}

//-----------------------------------------------------------------------------------
int CDriveFile::InsertData(unsigned int nPos, const void* pData, unsigned int nShift)
{
	char* pBuffer = new char[DRIVE_SIZE];
	int nDriveSize = GetSizeInBytes();
	bool bOk = nDriveSize > 0;
	unsigned int nPosRead = nDriveSize;
	unsigned int nRead    = DRIVE_SIZE;

	while(bOk && nPosRead > nPos)
	{
		if((nPosRead - nPos) < nRead) {
			nRead = nPosRead - nPos;
		}
		nPosRead -= nRead;
		bOk = SetPosition(nPosRead) > 0
			&& Read(pBuffer, nRead) > 0
			&& SetPosition(nPosRead + nShift) > 0
			&& Write(pBuffer, nRead) > 0;
	}
	delete[] pBuffer;
	if(bOk && SetPosition(nPos) > 0 && Write(pData, nShift) > 0) {
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CDriveFile::InsertValue(unsigned int nPos, int nValue, unsigned int nShift)
{
	char* pData = new char[nShift];
	memset(pData, nValue, nShift);
	int nRet = InsertData(nPos, pData, nShift);
	delete[] pData;
	return nRet;
}

//-----------------------------------------------------------------------------------
int CDriveFile::RemoveData(unsigned int nPos, unsigned int nSize)
{
	if(GetSizeInBytes() < nSize) {
		return -2;
	}
	unsigned int nPosMax = GetSizeInBytes() - nSize;
	char* pBuffer        = new char[DRIVE_SIZE];
	unsigned int nRead   = DRIVE_SIZE;
	bool bOk = true;

	while(bOk && nPos < nPosMax)
	{
		if((nPosMax - nPos) < nRead) {
			nRead = nPosMax - nPos;
		}
		bOk = SetPosition(nPos + nSize) > 0
			&& Read(pBuffer, nRead) > 0
			&& SetPosition(nPos) > 0
			&& Write(pBuffer, nRead) > 0;
		nPos += nRead;
	}
	nRead = F100MIN(nSize, DRIVE_SIZE);
	memset(pBuffer, 0xFF, nRead);
	bOk = SetPosition(nPosMax) > 0
		&& Write(pBuffer, nRead) > 0;
	delete[] pBuffer;
	if(bOk) {
		return 1;
	}
	return -1;
}
