
#ifndef DRIVEFILE_H
#define DRIVEFILE_H

#include <string>
using namespace std;

#include "APIFunction.h"

/*!
	\class CDriveFile DriveFile.h
	\brief Permet de lire un lecteur (en fichier ou zone mmoire)
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
	Conventions: \n
	Toutes les mthodes qui retournent un entier pour indiquer si oui ou non a c'est bien pass retournent : \n
	> 0 Ok \n
	= 0 pas Implment (rare) \n
	< 0 Erreur \n
	Classe abstraite permettant d'ouvrir et d'crire des donnes sur un lecteur \n
*/
class DLLEXPORT CDriveFile  
{
public:
	/** Constructeur
	 * @param[in] bOneFile A vrai si on veut qu'un seul fichier soit cr.
	 * @param[in] bVariableSize A vrai si on veut que les fichiers crs soient au maximum de DRIVE_SIZE octets (si bOneFile == FALSE alors le fichier n'aura pas forcement une taille multiple de DRIVE_SIZE)
	 * @param[in] nHeaderSize Taille de l'en-tte pour chaque lecteur
	 * @remarks Paramtre sans effet si l'on travaille dans une zone mmoire
	 */
	CDriveFile(bool bOneFile = false, bool bVariableSize = true, unsigned int nHeaderSize = 0);

	/** Destructeur */
	virtual ~CDriveFile();

	/** Permet de savoir si le lecteur est vide (0xFF)
	 * @param[in] nMaxSize -1 Valable uniquement pour l'accs  un fichier (indique de scanner tout le fichier) \n
	 *          >= 0 Nombre d'octets  scanner
	 * @return Retourne vrai si le lecteur a correctement t lu et s'il est vide
	 */
	virtual bool IsEmpty(int nMaxSize = -1);

	/** Ouvre un lecteur depuis un fichier
	 * @param[in] sPath Chemin complet du fichier
	 * @param[in] nShift Dcalage en octet
	 * @param[in] bReadWriteMode Si  faux, alors l'accs au fichier n'est fait qu'en mode lecture
	 * @return Retourne 1 pour OK et -1 pour erreur 
	 */
	virtual int  OpenDriveFile(const string& sPath, int nShift, bool bReadWriteMode = true);

	/** Ouvre un lecteur depuis un buffer
	 * @param[in,out] pData Zone mmoire  lire ou crire
	 * @param[in] nMaxSize Taille maximale de la zone mmoire
	 * @remarks La zone mmoire doit tre suffisament grande pour pouvoir crire des donnes et/ou doit contenir toutes les informations pour lire le lecteur
	 * @warning Ne fait pas de copie des donnes, les recherches
	 * de donnes se feront  partir de cette endroit. (Ne pas supprimer avant de fermer le lecteur)
	 * @remarks La fermeture du lecteur ne dtruit pas les donnes.
	 */
	virtual int  OpenDriveFile(void* pData, unsigned int nMaxSize);
	
	/** Ferme le lecteur.\n
	 * - Redimensionne le dernier fichier si ncessaire, suivant les options.\n
	 * - Renomme le lecteur primaire si ncessaire. (ie. Nom du fichier dcoup si le lecteur fait moins de DRIVE_SIZE octets)
	 * - Ecrit les en-tte si ncessaire
	 * @see WriteHeaders()
	 */
	virtual int  CloseDrive();

	/** Permet de savoir si le lecteur ouvert est valide
	 * @return Retourne vrai pour indiquer que le fichier est valide, sinon faux
	 */
	virtual bool IsValid() = 0;
	
	/** @return Le nombre de lecteur physique Graph100 qu'occupe ce type lecteur ou -1 si erreur */
	virtual int  GetNumDrive() const;
	
	/** Permet de connaitre la taille d'un lecteur
	 * @return Retourne la taille du lecteur ou une valeur ngative pour signaler une erreur (Lecteur non valide, etc.)
	 */
	virtual int  GetSizeInBytes() const = 0;

	/** Permet de connaitre la taille totale prise par le lecteur
	 * @return Retourne la taille totale du lecteur ou une valeur ngative pour signaler une erreur (Lecteur non valide, etc.)
	 */
	virtual int  GetTotalSizeInBytes() const;

protected:
	/** Permet de dfinir la position relative du pointeur courant sur les donnes
	 * @param[in] nPos Position du pointeur en octet
	 * @remarks Si un dcalage est dfini, celui-ci est automatiquement ajout  \a nPos
	 * @return 1 OK\n -1 Erreur d'accs \n -2 Rien d'ouvert
	 * @see ConvertPositionToRealPosition(unsigned int) const
	 * @see SetRealPosition(unsigned int)
	 */
	virtual int  SetPosition(unsigned int nPos);

	/** Permet de dfinir la position relle du pointeur courant sur les donnes
	 * @param[in] nPos Position du pointeur en octet
	 * @return 1 OK\n -1 Erreur d'accs \n -2 Rien d'ouvert
	 * @see SetPosition(unsigned int)
	 */
	virtual int  SetRealPosition(unsigned int nPos);

	/** Permet d'crire des donnes sur le lecteur
	 * @param[in] pData Donnes  crire
	 * @param[in] nSize Nombre d'octets  crire
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide \n -2 Rien d'ouvert
	 */
	virtual int  Write(const void* pData, unsigned int nSize);

	/** Permet de lire des donnes sur le lecteur
	 * @param[out] pData Donnes  lire
	 * @param[in] nSize Nombre d'octets  lire
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide \n -2 Rien d'ouvert
	 */
	virtual int  Read(void* pData, unsigned int nSize);

	/** Permet d'crire une en-tte
	 * @param[in] nNbHeader Numro de l'en-tte
	 * @param[in] pBuffer Donnes  crire
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide \n -2 Rien d'ouvert \n -3 Pas d'en-tte \n -4 Paramtre(s) non valide(s)
	 */
	virtual int  WriteHeader(int nNbHeader, const void* pBuffer);

	/** Permet de lire une en-tte
	 * @param[in] nNbHeader Numro de l'en-tte
	 * @param[out] pBuffer Zone mmoire pour recevoir les donnes de l'en-tte
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide \n -2 Rien d'ouvert \n -3 Pas d'en-tte \n -4 Paramtre(s) non valide(s)
	 */
	virtual int  ReadHeader(int nNbHeader, void* pBuffer);

	/** Fonction permettant d'avoir l'en-tte thorique
	 * @param[in] nNbHeader Numro de l'en-tte.
	 * @return @return Retourne l'en-tte ou NULL si pas d'en-tte
	 * @remarks L'implmentation par dfaut retourne NULL
	 */
	virtual void* GetHeader(int nNbHeader);

	/** Fonction permettant de mettre  jour toutes les en-ttes du lecteur
	 * @return 1 OK\n -1 Erreur -1 Erreur d'accs ou lecteur non valide \n -2 Rien d'ouvert \n -4 Paramtre non valide (En-tte vide retourne par GetHeader(int))
	 * @remarks Retourne 1 si le fichier n'a pas d'en-tte
	 * @see WriteHeader(int, const void*)
	 * @see GetHeader(int)
	 */
	virtual int   WriteHeaders();

	/** Fonction permettant de savoir si les en-ttes du lecteur sont valides
	 * @return Vrai si les en-ttes sont valides, faux si non valides ou problme de lecture
	 * @see ReadHeader(int, void*)
	 * @see GetHeader(int)
	 */
	virtual bool  HeadersAreValid();

	/** Permet de convertir une position relative en une position relle dans le lecteur.
	 * @param[in] nPos Position en octets
	 * @return La position relle en octets dans le lecteur
	 */
	virtual unsigned int ConvertPositionToRealPosition(unsigned int nPos) const;
	
	/** Permet d'insrer des donnes  un endroit prcis
	 * @param[in] nPos Position en octets o insrer les donnes
	 * @param[in] pData Donnes  insrer
	 * @param[in] nShift Dcalage en octets, taille des donnes  insrer
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide
	 */
	virtual int          InsertData(unsigned int nPos, const void* pData, unsigned int nShift);

	/** Permet d'insrer des donnes  un endroit prcis
	 * @param[in] nPos Position en octets o insrer les donnes
	 * @param[in] nValue Valeur  insrer
	 * @param[in] nShift Dcalage en octets, taille des donnes  insrer
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide
	 * @remarks Appelle InsertData(unsigned int, const void*, unsigned int)
	 */
	virtual int          InsertValue(unsigned int nPos, int nValue, unsigned int nShift);

	/** Permet de supprimer des donnes  un endroit prcis
	 * @param[in] nPos Dpart des donnes  supprimer
	 * @param[in] nSize Nombre d'octets  supprimer
	 * @return 1 OK\n -1 Erreur d'accs ou lecteur non valide\n -2 Valeurs incorrectes
	 */
	virtual int          RemoveData(unsigned int nPos, unsigned int nSize);

protected:
	string       m_sFileName;      ///< Nom du fichier en cours d'accs
	int          m_nNumDrive;      ///< Numro du fichier en cours d'accs
	bool         m_bOneFile;       ///< A faux si sur plusieurs fichiers de DRIVE_SIZE
	HND          m_hFile;	       ///< Handle du fichier ouvert (soit c'est m_pBuffer, soit m_hFile)
	void*        m_pBuffer;	       ///< Buffer du fichier ouvert (soit c'est m_pBuffer, soit m_hFile)
	int          m_nSeekPos;       ///< Position du pointeur dans \a m_pBuffer.
	unsigned int m_nMaxSize;       ///< Taille maximale de la zone mmoire \a m_pBuffer
	bool         m_bReadWriteMode; ///< Lecture/Ecriture active
	int          m_nShift;         ///< Dcalage du fichier
	bool         m_bVariableSize;  ///< A vrai si les fichiers crs seront de taille <= DRIVE_SIZE
	unsigned int m_nHeaderSize;    ///< Taille de l'en-tte pour chaque lecteur
};

#endif 
