// Protocol.h: interface for the CProtocol class.
// Classe abstraite qui hrite de l'interface IProtocol
// Permet de crer plus facilement un protocole car elle implmente
// les mthodes d'envoi et rception de donnes en utilisant
// l'interface IComDevice
//////////////////////////////////////////////////////////////////////

#ifndef PROTOCOL_H
#define PROTOCOL_H

#include <IProtocol.h>
#include <IComDevice.h>
#include <F100_Op.h>

/*!
	\class CProtocol Protocol.h
	\brief Permet de crer plus facilement un protocole
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK

	Classe abstraite qui hrite de l'interface IProtocol, permettant de crer plus facilement un protocole car elle implmente
	les mthodes d'envoi et rception de donnes en utilisant l'interface IComDevice
*/

class DLLEXPORT CProtocol : public IProtocol
{
public:
	CProtocol();
	virtual ~CProtocol();

	int          InitCom(IComDevice* pComDevice, int nSpeed);
	virtual void CloseCom(int nStatus);
	virtual int  SetTransfertMode(TRANSFERT nTransfert);
	TRANSFERT    GetTransfertMode() const;
	ERRORS       GetLastError() const;
	int*         GetAvailableSpeeds(int& nNumber) const;

protected:
	/** Permet de dfinir une erreur
	 * @param[in] nError Erreur dtecte
	 */
	void         SetError(ERRORS nError);

	/** Permet de dfinir les vitesses possibles
	 * @param[in] pSpeeds Tableau de vitesses possibles
	 * @param[in] nNbSpeed Nombre de vitesses possibles
	 */
	void         SetAvailableSpeeds(int* pSpeeds, int nNbSpeed);
	
	/** Permet de recevoir des donnes depuis le priphrique de transfert
	 * @param[out] buffer Zone mmoire pour recevoir les donnes
	 * @param[in] nNbBytes Nombre d'octets  recevoir
	 * @param[in] nTimeOut Temps en milli-seconde indiquant le temps maximum d'attente entre deux octets envoys. Si la valeur
	 * est infrieur  0, alors le temps n'est pas modifi (l'ancienne valeur est prise en compte)
	 * @return Retourne non-zro pour OK
	 * @see Send(const void*, unsigned int, int)
	 */
	int          Receive(void* buffer, unsigned int nNbBytes, int nTimeOut = -1);
	
	/** Permet d'envoyer des donnes depuis le priphrique de transfert
	 * @param[in] buffer Donnes  envoyer
	 * @param[in] nNbBytes Nombre d'octets  envoyer
	 * @param[in] nTimeOut Temps en milli-seconde indiquant le temps maximum d'attente entre deux octets envoys. Si la valeur
	 * est infrieur  0, alors le temps n'est pas modifi (l'ancienne valeur est prise en compte)
	 * @return Retourne non-zro pour OK
	 * @see Receive(void*, unsigned int, int)
	 */
	int          Send(const void* buffer, unsigned int nNbBytes, int nTimeOut = -1);

	/** Membres privs */
private:
	IComDevice* m_pComDevice; /*!< Priphrique de transfert */
	TRANSFERT   m_nTransfert; /*!< Mode de transfert */
	ERRORS      m_nError;     /*!< Dernire erreur */
	int         m_nSpeeds;    /*!< Nombre de vitesses disponibles */
	int*        m_pSpeeds;    /*!< Tableau des vitesses disponibles */
};

#endif
