
#include "ROMDISKDrive.h"
#include "ROMDISKFileInfo.h"
#include "DriveAccess.h"
#include "utils.h"

#include <math.h>

#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CROMDISKDrive::CROMDISKDrive(bool bOneFile, bool bVariableSize)
: CDriveFile(bOneFile, bVariableSize)
{
	m_bIsValid       = false;
	m_bFATModified   = false;
	m_nSizeInSectors = -1;
	m_mFiles         = new FilesMap();
	m_pOpenFiles     = new OpenFilesMap();
}

//-----------------------------------------------------------------------------------
CROMDISKDrive::~CROMDISKDrive()
{
	CloseDrive();
	delete m_mFiles;
	delete m_pOpenFiles;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::CloseDrive()
{
	if(!m_pOpenFiles->empty()) {
		_WARNING1("CROMDISKDrive::CloseDrive() found %i file(s)", m_pOpenFiles->size());
	}
	int nRet = WriteFAT();	
	if(nRet < 1) {
		return nRet;
	}
	nRet = CDriveFile::CloseDrive();
	if(nRet < 1) {
		return nRet;
	}
	m_mFiles->clear();
	m_nSizeInSectors = -1;
	m_bIsValid       = false;
	return 1;
}

//-----------------------------------------------------------------------------------
void CROMDISKDrive::ResetZone(FileTable &t)
{
	memset(&t, 0, sizeof t);
	memset(&t, 0x20, 12);
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::FormatDrive(unsigned short nNbFilesInRoot, const string& sVolumeName)
{
	if(m_bReadWriteMode)
	{
		DeleteAllFileInfo();
		m_bIsValid = true;
		//Formate le premier fichier
		//Cration du header
		static unsigned char header[] = {0xEB, 0x28, 0x90, 0x44, 0x4C, 0x52, 0x44, 0x49,
			0x53, 0x4B, 0, 0, 2, 1, 1, 0, 1, 0x10, 0, 0x3, 0, 0xF8, 2, 0,
			0, 0xF0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0};
		int i;
		//Dbut : en-tte de 512 octets
		bool bOk = SetPosition(0) > 0
			&& Write(header, sizeof header) > 0;
		unsigned long uLong = 0xFFFFFFFF;
		
		for(i = 0; i < 118 && bOk; i++)
		{
			bOk = Write(&uLong, sizeof(uLong)) > 0;
		}
		if(!bOk)
		{
			m_bIsValid = false;
			return -2;
		}
		uLong = 0xAA55FFFF;
		bOk = Write(&uLong, 4) > 0;
		
		//Ecriture de l'index de la FAT (juste le header de 3 octets)
		uLong = 0xFFFFF8;
		bOk = bOk && Write(&uLong, 3) > 0;
		
		//Rempli le reste du secteur de 0 + avec la zone racine
		char buffer[SECTOR_SIZE];
		int nRootSize = GetRootSize(nNbFilesInRoot);
		memset(buffer, 0, sizeof(buffer));
		int n = nRootSize / SECTOR_SIZE;
		bOk = bOk && Write(buffer, SECTOR_SIZE - 3) > 0
			&& SetFATSize(1) > 0
			&& SetRootSize(n) > 0//Ecrire le nombre de secteur pour l'index (racine)
			&& SetPosition(1024) > 0;//Se dplace  la racine

		//Rempli la racine de 0
		while(bOk && nRootSize > 0)
		{
			bOk = Write(buffer, sizeof(buffer)) > 0;
			nRootSize -= sizeof(buffer);
		}
		bOk = bOk && SetVolumeName(sVolumeName) > 0;
		m_bIsValid = bOk;
		if(bOk && ReadFAT() > 0)
		{
			if(m_hFile != INVALID_HANDLE_VALUE)	{
				return SetFileSize(m_hFile, GetSizeInBytes()); 
			}
			return 1;
		}
		return -3;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::SetVolumeName(const string& sVolumeName)
{
	//On se place au dbut du secteur
	if(SetPosition(GetRootPos()) < 1) {
		return -1;
	}
	FileTable t;
	ResetZone(t);
	
	//Cration du nom du volume
	for(unsigned int i = 0; i < sVolumeName.length(); i++) {
		t.name[i] = sVolumeName[i];
	}
	memset(t.ext, 0x20, 3);
	t.ext[3] = 0x08;
	return Write(&t, sizeof(t));
}

//-----------------------------------------------------------------------------------
string CROMDISKDrive::GetVolumeName()
{
	CROMDISKFileInfo* pRoot = OpenRoot();
	if(pRoot != NULL) {
		string s = pRoot->GetName();
		FreeFileInfo(pRoot);
		return s;
	}
	return "";
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::OpenDriveFile(const string& sPath, int nShift, bool bReadWriteMode)
{
	int nRet = CDriveFile::OpenDriveFile(sPath, nShift, bReadWriteMode);
	if(nRet < 1) {
		return nRet;
	}
	IsValid();
	return ReadFAT();
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::OpenDriveFile(void* pData, unsigned int nMaxSize)
{
	int nRet = CDriveFile::OpenDriveFile(pData, nMaxSize);
	if(nRet < 1) {
		return nRet;
	}
	IsValid();
	return ReadFAT();
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::InsertData(unsigned int nPos, const void* pData, unsigned int nShift)
{
	int nRet = CDriveFile::InsertData(nPos, pData, nShift);
	if(nRet < 1) {
		return nRet;
	}
	m_nSizeInSectors += (nShift / SECTOR_SIZE);
	if(nShift % SECTOR_SIZE) {
		m_nSizeInSectors++;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::RemoveData(unsigned int nPos, unsigned int nSize)
{
	int nRet = CDriveFile::RemoveData(nPos, nSize);
	if(nRet < 1) {
		return nRet;
	}
	m_nSizeInSectors -= (nSize / SECTOR_SIZE);
	return 1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::ReadFAT()
{
	m_mFiles->clear();
	m_bFATModified = false;
	m_nAddressFree = 2;
	if(m_bIsValid)
	{
		TFAT current;
		int nRead = 3;//Saute les trois premier octets
		int nFATSize = GetFATSize();
		bool bOk = SetPosition(19) > 0
			&& Read(&m_nSizeInSectors, sizeof(m_nSizeInSectors)) > 0//Lit la taille du lecteur
			&& SetPosition(SECTOR_SIZE + nRead) > 0;
		int nSector = 0;//Nbre de secteur du fichier lu
		while(bOk && nRead < nFATSize * SECTOR_SIZE)
		{
			bOk = Read(&current.nFirst, 1) > 0
				&& Read(&current.nSecond, 2) > 0;
			nRead += 3;
			nSector++;
			if(current.nFirst == 0xFF && (current.nSecond&0x0F) == 0x0F)
			{
				//Fin du fichier
				(*m_mFiles)[m_nAddressFree] = nSector;
				m_nAddressFree += nSector;
				nSector = 0;
			}
			nSector++;
			if((current.nSecond&0xFFF0) == 0xFFF0)
			{
				//Fin du fichier
				(*m_mFiles)[m_nAddressFree] = nSector;
				m_nAddressFree += nSector;
				nSector = 0;
			}
			else if((current.nSecond&0x0FFF) == 0x0FFF)
			{
				//Fin du fichier
				nSector--;
				(*m_mFiles)[m_nAddressFree] = nSector;
				m_nAddressFree += nSector ;
				nSector = 0;
			}
			if(current.nFirst == 0x00)
			{
				//Fini
				break;
			}
		}
		if(!bOk) {
			m_bIsValid = false;
		}
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::WriteFAT()
{
	DeleteAllFileInfo();
	if(m_bFATModified)
	{
		int nFATSize;
		if( (nFATSize = GetFATSize()) == -1
			|| SetPosition(SECTOR_SIZE) == -1) {
			return -1;
		}
		list<TFAT> lFAT;
		
		TFAT current = {0x03, 0x0040};
		int nCpt;
		bool bWriteFirst = true;
		FilesMap::iterator it  = m_mFiles->find(2);
		FilesMap::iterator end = m_mFiles->end();
		TFAT index = {0xF8, 0xFFFF};
		lFAT.push_back(index);
		while(it != end)
		{
			int nSize = (*it).second;
			nCpt = 1;
			while(nCpt <= nSize)
			{
				if(bWriteFirst)
				{
					if(nSize == nCpt)
					{
						index.nFirst  = 0xFF;
						index.nSecond = 0x0F;
					}
					else
					{
						index.nFirst  = current.nFirst;
						index.nSecond = 0x0;
					}
					if(current.nFirst < 0x3)
					{
						current.nSecond++;
					}
					current.nFirst += 0x2;
				}
				else
				{
					index.nSecond |= current.nSecond;
					if(nSize == nCpt)
					{
						index.nSecond |= 0xFFF0;
					}
					current.nSecond += 0x20;
					lFAT.push_back(index);
				}
				bWriteFirst ^= true;
				nCpt++;
			}
			it = m_mFiles->find((*it).first + nSize);
		}
		if(!bWriteFirst) {
			lFAT.push_back(index);
		}
		int nNewFATSize = ((lFAT.size() * 3) / SECTOR_SIZE) + 1;
		int nSize       = nNewFATSize * SECTOR_SIZE;
		
		unsigned char* pFAT = new unsigned char[nSize];
		memset(pFAT, 0, nSize);
		list<TFAT>::iterator it2  = lFAT.begin();
		list<TFAT>::iterator end2 = lFAT.end();
		unsigned int i = 0;
		for(; it2 != end2; ++it2) {
			pFAT[i] = (*it2).nFirst;
			*(unsigned short*)(pFAT + i + 1) = (*it2).nSecond;
			i += 3;
		}
		int nWrite = F100MIN(nSize, nFATSize * SECTOR_SIZE);
		
		bool bOk = Write(pFAT, nWrite) > 0;
		
		if(nNewFATSize != nFATSize) {
			if(nNewFATSize > nFATSize) {
				bOk = bOk && InsertData(nFATSize * SECTOR_SIZE + SECTOR_SIZE, pFAT + nWrite, nSize - nWrite) > 0;
			}
			else {
				int nShift = nFATSize * SECTOR_SIZE - nSize;
				bOk = bOk && RemoveData(nFATSize * SECTOR_SIZE + SECTOR_SIZE - nShift, nShift) > 0;
			}
			bOk = bOk && SetFATSize(nNewFATSize);
		}
		delete[] pFAT;
		
		bOk = bOk && SetPosition(19) > 0
			&& Write(&m_nSizeInSectors, sizeof(m_nSizeInSectors)) > 0;
		if(!bOk) {
			return -1;
		}
		m_bFATModified = false;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetDirSize(unsigned short nNumFile)
{
	return (int)(ceil((nNumFile + 2) * sizeof(FileTable) / (float)SECTOR_SIZE) * SECTOR_SIZE);
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetRootSize(unsigned short nNumFile)
{
	return (int)(ceil((nNumFile + 1) * sizeof(FileTable) / (float)SECTOR_SIZE) * SECTOR_SIZE);
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetSizeInBytes() const
{
	int nSize = GetSizeInSectors();
	if(nSize > 0) {
		return nSize * SECTOR_SIZE;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetSizeInSectors() const
{
	return m_nSizeInSectors;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::SetFATSize(unsigned short nSize)
{
	if(SetPosition(0x16) > 0 && Write(&nSize, sizeof(nSize)) > 0) {
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::SetRootSize(unsigned char nSize)
{
	nSize <<= 4;
	if(SetPosition(0x11) > 0 && Write(&nSize, 1) > 0) {
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetFATSize()
{
	unsigned short nSize;
	if(SetPosition(0x16) > 0 && Read(&nSize, sizeof(nSize)) > 0) {
		return nSize;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetRootSize()
{
	unsigned char nSize;
	if(SetPosition(0x11) > 0 && Read(&nSize, 1) > 0) {
		return SECTOR_SIZE * (nSize >> 4);
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetRootPos()
{
	//Regarde la taille de la FAT
	int nFATSize = GetFATSize();
	if(nFATSize > 0) {
		return (SECTOR_SIZE * (nFATSize + 1));
	}
	return -1;
}

//-----------------------------------------------------------------------------------
bool CROMDISKDrive::IsValid()
{
	if(!m_bIsValid)
	{
		static unsigned char header[] = {0xEB, 0x28, 0x90, 0x44, 0x4C,
			0x52, 0x44, 0x49, 0x53, 0x4B};
		unsigned char pBuffer[10];
		m_bIsValid = true;
		if(SetPosition(0) < 1 || Read(pBuffer, sizeof(pBuffer)) < 1)
		{
			return false;
		}
		int i;
		for(i = 0; i < 10 && header[i] == pBuffer[i]; i++);
		m_bIsValid = (i == 10);
	}
	return m_bIsValid;
}

//-----------------------------------------------------------------------------------
void CROMDISKDrive::FreeFileInfo(CROMDISKFileInfo* pInfo)
{
	if(pInfo != NULL) {
		if(pInfo->DecCountRef() == 0) {
			m_pOpenFiles->remove(pInfo);
			delete pInfo;
		}
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKDrive::DeleteAllFileInfo()
{
	OpenFilesMap::iterator it  = m_pOpenFiles->begin();
	OpenFilesMap::iterator end = m_pOpenFiles->end();
	for(; it != end; ++it) {
		delete (*it);
	}
	m_pOpenFiles->clear();
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::ReadInfos(unsigned int nParentAddress, unsigned int nParentSize, int nPosition)
{
	if(SetPosition(nParentAddress + nPosition * sizeof(FileTable)) < 1) {
		return NULL;
	}
	FileTable infos;
	if(Read(&infos, sizeof(infos)) < 1) {
		return NULL;
	}
	if(nParentSize <= (int)(nPosition * sizeof(FileTable)) || infos.name[0] == '\0') {
		return NULL;
	}
	int nAddress;
	if(infos.adress == 0 && infos.ext[3] != 0x08) {
		nAddress = 0;
	}
	else {
		nAddress = ConvertFATAddressToFileAddress(infos.adress);
	}
	CROMDISKFileInfo* pFileInfo = GetFileInfo(nAddress, nParentAddress, nPosition);
	if(pFileInfo != NULL && pFileInfo->m_nCountRef == 1) {
		pFileInfo->SetExtension(infos.ext);
		pFileInfo->SetName(infos.name);
		pFileInfo->m_nParentSize = nParentSize;
		pFileInfo->m_nSize       = infos.size;
		pFileInfo->SetTime(infos.date, infos.time);
		if(pFileInfo->IsDir()) {
			pFileInfo->m_nSize = (*m_mFiles)[infos.adress] * SECTOR_SIZE;
		}
		else if(pFileInfo->IsRoot()) {
			pFileInfo->m_nSize = GetRootSize();
		}
	}
	return pFileInfo;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::SetInfos(CROMDISKFileInfo* pFileInfo)
{
	if(pFileInfo == NULL) {
		return -1;
	}
	FileTable infos;
	ResetZone(infos);
	memcpy(infos.name, pFileInfo->m_pName,      sizeof(infos.name));
	memcpy(infos.ext,  pFileInfo->m_pExtension, sizeof(infos.ext));

	if(pFileInfo->IsFile()) {
		infos.size = pFileInfo->m_nSize;
	}
	infos.adress = GetAddressInSector(pFileInfo);
	pFileInfo->GetTime(infos.date, infos.time);
	if(SetPosition(pFileInfo->m_nParentAddress + pFileInfo->m_nPosition * sizeof(FileTable)) < 1 || Write(&infos, sizeof(infos)) < 1) {
		return -1;
	}
	return 1;
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::OpenRoot()
{
	return ReadInfos(GetRootPos(), GetRootSize(), 0);
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::OpenDir(CROMDISKFileInfo* pDir, bool bFreeDir)
{
	if(pDir != NULL && pDir->IsDir()) {
		CROMDISKFileInfo* pFileInfo =  ReadInfos(pDir->m_nAddress, pDir->m_nSize, 0);
		if(bFreeDir) {
			FreeFileInfo(pDir);
		}
		return pFileInfo;
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::FindFirst(CROMDISKFileInfo* pDir, bool bFreeDir)
{
	if(pDir == NULL || !pDir->IsValid()) {
		return NULL;
	}
	CROMDISKFileInfo* pFirst = ReadInfos(pDir->m_nParentAddress, pDir->m_nParentSize, 0);
	if(bFreeDir) {
		FreeFileInfo(pDir);
	}
	return pFirst;
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::FindNext(CROMDISKFileInfo* pDir, bool bFreeDir)
{
	if(pDir == NULL || !pDir->IsValid()) {
		return NULL;
	}
	CROMDISKFileInfo* pNext = ReadInfos(pDir->m_nParentAddress, pDir->m_nParentSize, pDir->m_nPosition + 1);
	if(bFreeDir) {
		FreeFileInfo(pDir);
	}
	return pNext;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::Rename(const string& sFileName, string& sNewName)
{
	CROMDISKFileInfo* pRoot = OpenRoot();
	CROMDISKFileInfo* pFile = FindFileOrDir(sFileName, pRoot);
	FreeFileInfo(pRoot);
	if(pFile == NULL) {
		return ER_FILE_NOT_FOUND;
	}
	int nRet = pFile->Rename(sNewName);
	FreeFileInfo(pFile);
	return nRet;
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::FindFileOrDir(const string& sPath, CROMDISKFileInfo* pDir)
{
	if(pDir != NULL) {
		string sName = section(sPath, '/', 0, 0);
		if(sName.empty()) {
			return NULL;
		}
		CROMDISKFileInfo* pNext = FindFirst(pDir, false);
		if(pDir->IsRoot()) {
			pNext = FindNext(pNext, true);
		}
		while(pNext != NULL) {
			if(areEqual(pNext->GetFullName(), sName, false)) {
				string s = section(sPath, '/', 1);
				if(s.empty()) {
					return pNext;
				}
				pNext = OpenDir(pNext, true);
				CROMDISKFileInfo* pResult = FindFileOrDir(s, pNext);
				FreeFileInfo(pNext);
				return pResult;
			}
			pNext = FindNext(pNext);
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetAddressInSector(const CROMDISKFileInfo* pFile)
{
	if(pFile == NULL || !pFile->IsValid()) {
		return -1;
	}
	if(pFile->IsRoot() || pFile->m_nAddress == 0) {
		return 0;
	}
	unsigned int nStart = GetRootPos() + GetRootSize();
	
	if(pFile->m_nAddress < nStart) {
		return 0;
	}
	return 2 + ((pFile->m_nAddress - nStart) / SECTOR_SIZE);
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::ConvertFATAddressToFileAddress(int nAddrSectors)
{
	int nRootPos = GetRootPos();
	if(nRootPos < 1) {
		return 0;
	}
	if(nAddrSectors < 2) {
		return nRootPos;
	}
	return (nAddrSectors - 2) * SECTOR_SIZE + nRootPos + GetRootSize();
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::GetFreePosition(CROMDISKFileInfo* pDir, bool bAutoAdd)
{
	if(pDir != NULL) {
		CROMDISKFileInfo* pNext = FindFirst(pDir, false);
		if(pNext == NULL) {
			return -1;
		}
		int nPos = 0;
		while(pNext != NULL) {
			nPos  = pNext->m_nPosition + 1;
			pNext = FindNext(pNext, true);
		}
		if((nPos * sizeof(FileTable)) == pDir->m_nSize) {
			if(!bAutoAdd) {
				return -2;
			}
			if(Resize(pDir, pDir->GetSize() + SECTOR_SIZE) < 1) {
				return -1;
			}
		}
		return nPos;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::CreateDirectory(CROMDISKFileInfo* pParent, CROMDISKFileInfo*& pDir, string& sName, unsigned short nMaxFile)
{
	if(pParent != NULL)
	{
		CROMDISKFileInfo::CheckName(sName, false);
		CROMDISKFileInfo* pFind = FindFileOrDir(sName, pParent);
		if(pFind != NULL) {
			FreeFileInfo(pFind);
			return ER_ALREADY_EXISTS;
		}
		int nPos = GetFreePosition(pParent);
		if(nPos < 1) {
			pDir = NULL;
			return -1;
		}
		pDir = GetFileInfo(ConvertFATAddressToFileAddress(m_nAddressFree), pParent->m_nParentAddress, nPos);
		if(pDir == NULL) {
			pDir = NULL;
			return -1;
		}
		pDir->m_pExtension[3] = 0x10;
		pDir->m_nSize       = GetDirSize(nMaxFile);
		pDir->m_nParentSize = pParent->m_nParentSize;
		pDir->SetCurrentTime();
		if(pDir->Rename(sName) < 1) {
			FreeFileInfo(pDir);
			pDir = NULL;
			return -1;
		}
		int nSizeInSectors = pDir->m_nSize / SECTOR_SIZE;
		(*m_mFiles)[m_nAddressFree] = nSizeInSectors;
		m_nAddressFree += nSizeInSectors;
		m_bFATModified  = true;
		
		m_nSizeInSectors += nSizeInSectors;

		//Cration des donnes du rpertoire
		char data[SECTOR_SIZE];
		memset(data, 0, sizeof(data));

		FileTable dotDir;
		ResetZone(dotDir);
		dotDir.adress = GetAddressInSector(pDir);
		dotDir.name[0] = '.';
		dotDir.ext[3]  = 0x10;
		memcpy(data, &dotDir, sizeof(dotDir));
		dotDir.name[1] = '.';
		dotDir.adress = GetAddressInSector(pParent);
		memcpy(data + sizeof(dotDir), &dotDir, sizeof(dotDir));

		if(SetPosition(pDir->m_nAddress) > 0
			&& Write(data, sizeof(data)) > 0)
		{
			memset(data, 0, 2 * sizeof(dotDir));
			while(nSizeInSectors > 1 && Write(data, SECTOR_SIZE) > 0) {
				nSizeInSectors--;
			}
			if(nSizeInSectors == 1)	{
				return 1;
			}
		}
	}
	pDir = NULL;
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::CreateFile(CROMDISKFileInfo* pParent, CROMDISKFileInfo*& pFile, string& sName)
{
	if(pParent != NULL) {
		CROMDISKFileInfo::CheckName(sName, true);
		CROMDISKFileInfo* pFind = FindFileOrDir(sName, pParent);
		if(pFind != NULL) {
			FreeFileInfo(pFind);
			return ER_ALREADY_EXISTS;
		}
		int nPos = GetFreePosition(pParent);
		if(nPos < 1) {
			pFile = NULL;
			return -1;
		}

		pFile = GetFileInfo(0, pParent->m_nParentAddress, nPos);
		if(pFile == NULL) {
			return -1;
		}
		pFile->m_pExtension[3] = 0x20;
		pFile->m_nSize       = 0;
		pFile->m_nParentSize = pParent->m_nParentSize;
		pFile->SetCurrentTime();
		if(pFile->Rename(sName) < 1) {
			FreeFileInfo(pFile);
			pFile = NULL;
			return -1;
		}
		return 1;
	}
	pFile = NULL;
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::CreateFile(CROMDISKFileInfo* pParent, CROMDISKFileInfo*& pFile, string& sName, const string& sFileName)
{
	HND h = OpenDescModeRead(sFileName.c_str());
	if(h == INVALID_HANDLE_VALUE) {
		pFile = NULL;
		return ER_CANT_READ;
	}
	int nRet = CreateFile(pParent, pFile, sName);
	if(nRet != 1) {
		pFile = NULL;
		CloseDesc(h);
		return nRet;
	}
	unsigned int nSector = 0;
	unsigned int nSize   = FileSize(h);

	if(nSize > 0) {
		char buffer[65536];
		unsigned int nWrote = 0;
		unsigned int nWrite = sizeof(buffer);
		while(nWrote < nSize) {
			if(nWrote + nWrite > nSize) {
				nWrite = nSize - nWrote;
			}
			ReadDesc(h, buffer, nWrite);
			int nRet = pFile->WriteFile(buffer, nWrote, nWrite);
			if(nRet < 1) {
				FreeFileInfo(pFile);
				pFile = NULL;
				CloseDesc(h);
				return nRet;
			}
			nWrote += nWrite;
		}
	}
#ifdef WIN32
	FILETIME nLastWriteTime;
	unsigned short nTime, nDate;
	GetFileTime(h, NULL, NULL, &nLastWriteTime);
	FileTimeToDosDateTime(&nLastWriteTime, &nDate, &nTime);
	pFile->SetTime(nDate, nTime);
	SetInfos(pFile);
#endif
	CloseDesc(h);
	return 1;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::Resize(CROMDISKFileInfo* pFileInfo, unsigned int nNewSize)
{
	if(pFileInfo != NULL) {
		_ASSERTE(Find(pFileInfo));
		if(pFileInfo->m_nSize == nNewSize) {
			return 1;
		}
		unsigned int nRealSize    = ConvertFileSizeToRealSize(pFileInfo->m_nSize);
		unsigned int nNewRealSize = ConvertFileSizeToRealSize(nNewSize);
		if(pFileInfo->m_nSize < nNewSize) {
			int nShift = nNewRealSize - nRealSize;
			if(nShift == 0) {
				_ASSERTE(pFileInfo->IsFile());
				pFileInfo->m_nSize = nNewSize;
				return SetInfos(pFileInfo);
			}
			char* pBuffer = new char[nShift];
			if(pBuffer == NULL) {
				return ER_OUT_OF_MEM;
			}
			if(pFileInfo->IsFile()) {
				memset(pBuffer, 0xFF, nShift);
			}
			else {
				memset(pBuffer, 0x00, nShift);
			}
			_ASSERTE(pFileInfo->m_nAddress != 0 || !pFileInfo->IsDir());
			if(pFileInfo->m_nAddress == 0 && pFileInfo->IsFile()) {
				pFileInfo->m_nAddress = ConvertFATAddressToFileAddress(m_nAddressFree);
				(*m_mFiles)[m_nAddressFree] = ConvertBytesToSectors(nNewRealSize);
				m_nAddressFree             += ConvertBytesToSectors(nNewRealSize);
			}
			UpdateFiles(pFileInfo, nShift, nNewSize);
			int nRet = InsertData(pFileInfo->m_nAddress + nRealSize, pBuffer, nShift);
			delete[] pBuffer;
			return nRet;
		}
		int nShift  = nRealSize - nNewRealSize;
		int nShift2 = nNewRealSize - nNewSize;
		unsigned int nAddress = pFileInfo->m_nAddress;
		if(nShift > 0) {
			UpdateFiles(pFileInfo, -nShift, nNewSize);
			int nRet = RemoveData(nAddress + nNewRealSize, nShift);
			if(nRet < 1) {
				return nRet;
			}
		}
		if(nShift2 > 0) {
			_ASSERTE(pFileInfo->IsFile());
			char* pBuffer = new char[nShift2];
			if(pBuffer == NULL) {
				return ER_OUT_OF_MEM;
			}
			memset(pBuffer, 0xFF, nShift2);
			bool bOk = SetPosition(nAddress + nNewSize) > 0
				&& Write(pBuffer, nShift2) > 0;
			delete[] pBuffer;
			if(!bOk) {
				return -1;
			}
			pFileInfo->m_nSize = nNewSize;
			SetInfos(pFileInfo);
		}
		return 1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
void CROMDISKDrive::UpdateFile(CROMDISKFileInfo* pFileModified, CROMDISKFileInfo* pInfo, int nShift, unsigned int nNewSize)
{
	if(pInfo != NULL && pFileModified != pInfo) {
		if(pInfo->m_nAddress > pFileModified->m_nAddress) {
			pInfo->m_nAddress += nShift;
			SetInfos(pInfo);
		}
		if(pInfo->m_nParentAddress == pFileModified->m_nAddress) {
			pInfo->m_nParentSize = nNewSize;
		}
		if(pInfo->m_nParentAddress > pFileModified->m_nAddress) {
			pInfo->m_nParentAddress += nShift;
		}
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKDrive::Update(CROMDISKFileInfo* pFileModified, CROMDISKFileInfo* pParent, int nShift, unsigned int nNewSize)
{
	if(pParent != NULL && (pParent->IsRoot() || pParent->IsDir())) {
		CROMDISKFileInfo* pNext = FindFirst(pParent, false);
		while(pNext != NULL) {
			if(pNext->IsDir() && pNext->GetName() != "." && pNext->GetName() != "..") {
				CROMDISKFileInfo* pDir = OpenDir(pNext, false);
				Update(pFileModified, pDir, nShift, nNewSize);
				FreeFileInfo(pDir);
			}
			CROMDISKFileInfo* pNext2 = FindNext(pNext, false);
			UpdateFile(pFileModified, pNext, nShift, nNewSize);
			FreeFileInfo(pNext);
			pNext = pNext2;
		}
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKDrive::UpdateFiles(CROMDISKFileInfo* pFileModified, int nShift, unsigned int nNewSize)
{
	_ASSERTE(pFileModified->m_nAddress != 0 || !pFileModified->IsRoot());
	if(pFileModified->IsRoot()) {
		SetRootSize(ConvertBytesToSectors(nNewSize));
		pFileModified->m_nSize = nNewSize;
		OpenFilesMap::iterator it = m_pOpenFiles->begin();
		for(; it != m_pOpenFiles->end(); ++it) {
			CROMDISKFileInfo* pInfo = *it;
			if(pFileModified != pInfo) {
				if(pInfo->m_nAddress > pFileModified->m_nAddress) {
					pInfo->m_nAddress += nShift;
				}
				if(pInfo->m_nParentAddress == pFileModified->m_nAddress) {
					pInfo->m_nParentSize = nNewSize;
				}
				if(pInfo->m_nParentAddress > pFileModified->m_nAddress) {
					pInfo->m_nParentAddress += nShift;
				}
			}
		}
	}
	else {		
		CROMDISKFileInfo* pRoot = OpenRoot();
		Update(pFileModified, pRoot, nShift, nNewSize);
		FreeFileInfo(pRoot);
		pFileModified->m_nSize = nNewSize;
		FilesMap mTmp;
		FilesMap::iterator it  = m_mFiles->find(2);
		FilesMap::iterator end = m_mFiles->end();
		int nAddr    = 2;
		int nTmpAddr = 2;
		int nAddress = GetAddressInSector(pFileModified);
		while(it != end) {
			int nSize = (*it).second;
			if(nAddr == nAddress) {
				if(pFileModified->m_nSize > 0) {
					mTmp[nTmpAddr] = ConvertBytesToSectors(pFileModified->m_nSize);
					nTmpAddr      += ConvertBytesToSectors(pFileModified->m_nSize);
				}
				else {
					pFileModified->m_nAddress = 0;
				}
			}
			else {
				mTmp[nTmpAddr] = nSize;
				nTmpAddr      += nSize;
			}
			nAddr = nAddr + nSize;
			it = m_mFiles->find(nAddr);
		}
		*m_mFiles = mTmp;
		m_nAddressFree = nTmpAddr;
		m_bFATModified = true;
		SetInfos(pFileModified);
	}
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::Find(unsigned int nAddress, unsigned int nParentAddress, int nPosition) const
{
	OpenFilesMap::iterator it  = m_pOpenFiles->begin();
	OpenFilesMap::iterator end = m_pOpenFiles->end();
	for(; it != end; ++it) {
		CROMDISKFileInfo* pInfo = *it;
		if(pInfo->m_nAddress == nAddress && pInfo->m_nParentAddress == nParentAddress && pInfo->m_nPosition == nPosition) {
			return pInfo;
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
bool CROMDISKDrive::Find(CROMDISKFileInfo* pFileInfo) const
{
	return pFileInfo != NULL && Find(pFileInfo->m_nAddress, pFileInfo->m_nParentAddress, pFileInfo->m_nPosition) != NULL;
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo* CROMDISKDrive::GetFileInfo(unsigned int nAddress, unsigned int nParentAddress, int nPosition)
{
	CROMDISKFileInfo* pInfo = Find(nAddress, nParentAddress, nPosition);
	if(pInfo != NULL) {
		pInfo->IncCountRef();
		return pInfo;
	}
	pInfo = new CROMDISKFileInfo(this);
	if(pInfo != NULL) {
		pInfo->m_nAddress       = nAddress;
		pInfo->m_nParentAddress = nParentAddress;
		pInfo->m_nPosition      = nPosition;
		m_pOpenFiles->push_back(pInfo);
	}
	return pInfo;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::ConvertBytesToSectors(unsigned int nBytes)
{
	int nSector = nBytes / SECTOR_SIZE;
	if(nBytes % SECTOR_SIZE) {
		nSector++;
	}
	return nSector;
}

//-----------------------------------------------------------------------------------
unsigned int CROMDISKDrive::ConvertFileSizeToRealSize(unsigned int nFileSize)
{
	if(nFileSize % SECTOR_SIZE) {
		return nFileSize + (SECTOR_SIZE - nFileSize % SECTOR_SIZE);
	}
	return nFileSize;
}

//-----------------------------------------------------------------------------------
int CROMDISKDrive::Kill(CROMDISKFileInfo* pInfo)
{
	if(!Find(pInfo) || pInfo->IsRoot() || (pInfo->IsDir() && (pInfo->GetName() == "." || pInfo->GetName() == ".."))) {
		return -1;
	}
	if(pInfo->m_nCountRef > 1) {
		return ER_FILE_USED;
	}

	if(pInfo->IsDir()) {
		CROMDISKFileInfo* pDir = OpenDir(pInfo, false);
		int nRet = GetFreePosition(pDir, false);
		FreeFileInfo(pDir);
		if(nRet > 2 || nRet == -2) {
			return ER_FOLDER_NOT_EMPTY;
		}
	}

	int nRet = Resize(pInfo, 0);
	if(nRet < 1) {
		return nRet;
	}

	unsigned int nAddr = pInfo->m_nParentAddress;
	unsigned int nPos  = pInfo->m_nPosition;
	CROMDISKFileInfo* pFirst = FindFirst(pInfo, false);
	CROMDISKFileInfo* pNext  = FindNext(pInfo);
	while(pNext != NULL) {
		nPos++;
		CROMDISKFileInfo* pNext2 = FindNext(pNext, false);
		pNext->m_nPosition--;
		SetInfos(pNext);
		FreeFileInfo(pNext);
		pNext = pNext2;
	}
	FileTable ft;
	memset(&ft, 0, sizeof(ft));

	SetPosition(nAddr + nPos * sizeof(FileTable));
	Write(&ft, sizeof(ft));
	unsigned int nSize;
	if(pFirst->IsRoot()) {
		nSize = GetRootSize(nPos - 1);
	}
	else {
		nSize = GetDirSize(nPos - 2);
	}
	Resize(pFirst, nSize);
	FreeFileInfo(pFirst);
	return 1;
}
