
#ifndef ROMDISKDRIVE_H
#define ROMDISKDRIVE_H

#ifdef _MSC_VER
#	pragma warning(disable:4786)
#endif

#include "DriveFile.h"

#include <map>
#include <list>
using namespace std;

#define SECTOR_SIZE         512
#define DEFAULT_VOLUME_NAME "ROM-DISK"

class CROMDISKFileInfo;

/*!
	\class CROMDISKDrive ROMDISKDrive.h
	\brief Permet de lire ou crire un lecteur ROM-DISK
	\author Quaranta David
	\version 1.00
	
	\ingroup F100TK
*/
class DLLEXPORT CROMDISKDrive : public CDriveFile
{
	friend class CROMDISKFileInfo;
public:
	/** Liste d'erreurs pouvant tre retournes par les mthodes de la classe CROMDISKDrive */
	enum Errors{
		ER_ALREADY_EXISTS   = -20, ///< Le fichier ou dossier existe dj
		ER_OUT_OF_MEM       = -21, ///< Manque de mmoire
		ER_NOT_A_FILE       = -22, ///< La zone n'est pas un fichier
		ER_FILE_TOO_SMALL   = -23, ///< Le fichier est trop petit
		ER_POS_TOO_BIG      = -24, ///< Position de dpart suprieure  la taille d'un fichier
		ER_CANT_READ        = -25, ///< Erreur de lecture sur un fichier
		ER_FILE_NOT_FOUND   = -26, ///< Fichier ou dossier non trouv
		ER_FILE_USED        = -27, ///< Fichier ou dossier en cours d'utilisation
		ER_FOLDER_NOT_EMPTY = -28, ///< Le dossier n'est pas vide
	};

public:
	/** Constructeur
	 * @param[in] bOneFile A vrai si on veut qu'un seul fichier soit cr.
	 * @param[in] bVariableSize A vrai si on veut que les fichiers crs soient au maximum de DRIVE_SIZE octets (si bOneFile == FALSE alors le fichier n'aura pas forcement une taille multiple de DRIVE_SIZE)
	 * @remarks Paramtre sans effet si l'on travaille dans une zone mmoire
	 */
	CROMDISKDrive(bool bOneFile = false, bool bVariableSize = true);

	/** Destructeur\n
	 * Ferme le lecteur
	 */
	virtual ~CROMDISKDrive();

	/** Formate le lecteur
	 * @param[in] nNbFilesInRoot Indique combien de fichiers (ou dossiers) seront contenus dans la racine
	 * @param[in] sVolumeName Nom du volume
	 * @return < 0 Erreur\n 1 Succs
	 * @warning Supprime toutes les informations sur les fichiers et dossiers ouverts
	 */
	int    FormatDrive(unsigned short nNbFilesInRoot = 0, const string& sVolumeName = DEFAULT_VOLUME_NAME);

	/** Dfini le nom du volume
	 * @param[in] sVolumeName Nom du volume
	 * @return < 0 Erreur\n 1 Succs
	 */
	int    SetVolumeName(const string& sVolumeName);

	/** @return Nom du volume */
	string GetVolumeName();

	int    OpenDriveFile(const string& sPath, int nShift, bool bReadWriteMode = true);
	int    OpenDriveFile(void* pData, unsigned int nMaxSize);
	int    CloseDrive();

	int    GetSizeInBytes() const;
	/** @return La taille du lecteur en secteurs de 512 octets ou -1 si erreur */
	int    GetSizeInSectors() const;
	bool   IsValid();

	/** Permet de signaler au lecteur que vous avez plus besoin des informations sur un fichier ou dossier
	 * @param[in] pInfo Informations qui ne sont plus utiles.
	 * @remarks \a pInfo est dtruit si et seulement si il n'y a plus de rfrence sur lui.
	 */
	void            FreeFileInfo(CROMDISKFileInfo* pInfo);

	/** Dtruit tous les informations de fichiers ou dossiers ouverts */
	void            DeleteAllFileInfo();

	/** @return La racine du lecteur (Nom du volume) ou NULL si erreur */
	CROMDISKFileInfo* OpenRoot();

	/** Permet d'ouvrir un dossier
	 * @param[in] pDir Dossier  ouvrir
	 * @param[in] bFreeDir A vrai si l'on veut que le dossier \a pDir soit libr
	 * @return La racine du lecteur (Nom du volume) ou NULL si erreur
	 * @remarks Libre \a pDir si c'est un dossier et mme si cette mthode choue
	 */
	CROMDISKFileInfo* OpenDir(CROMDISKFileInfo* pDir, bool bFreeDir);

	/** Permet de rcuprer le premier lment contenu dans la racine ou un dossier
	 * @param[in] pDir Dossier ou racine
	 * @param[in] bFreeDir A vrai si l'on veut que le dossier \a pDir soit libr
	 * @return Retourne le premier lment ou NULL si erreur
	 * @remarks Libre \a pDir si c'est un dossier et mme si cette mthode choue
	 */
	CROMDISKFileInfo* FindFirst(CROMDISKFileInfo* pDir, bool bFreeDir = true);

	/** Permet de rcuprer l'lment suivant contenu dans la racine ou un dossier
	 * @param[in] pDir Dossier ou racine
	 * @param[in] bFreeDir A vrai si l'on veut que le dossier \a pDir soit libr
	 * @return Retourne l'lment suivant ou NULL si erreur ou si le dossier est vide
	 * @remarks Libre \a pDir si c'est un dossier et mme si cette mthode choue
	 */
	CROMDISKFileInfo* FindNext(CROMDISKFileInfo* pDir, bool bFreeDir = true);
	
	/** Recherche un fichier ou dossier depuis le chemin indiqu
	 * @param[in] sPath Contient le chemin recherch relatif au dpart souhait (spar par des '/').
	 * Complet si \a est la racine du lecteur, sinon partiel si on commence depuis un dossier
	 * @param[in] pDir Dossier de dpart (Le dossier doit tre ouvert pour que la mthode dedans, sinon la mthode part sur le premier lment point)
	 * @return Retourne l'lment correspondant  la recherche ou NULL si erreur ou si non trouv
	 */
	CROMDISKFileInfo* FindFileOrDir(const string& sPath, CROMDISKFileInfo* pDir);

	/** Renomme un fichier ou un rpertoire
	 * @param[in] sFileName Nom du fichier ou dossier  renommer
	 * @param[in,out] sNewName contient le nouveau nom valide et est retourn dans cette mme variable le nom vraiment inscrit
	 * @return 1 Succs\n CROMDISKDrive::ER_FILE_NOT_FOUND\n < -1 Erreur
	 */
	int               Rename(const string& sFileName, string& sNewName);

	/** Permet de crerun rpertoire dans le lecteur
	 * @param[in] pParent Parent du dossier
	 * @param[out] pDir Pointeur sur le dossier cr
	 * @param[in] sName Nom du dossier
	 * @param[in] nMaxFile Indique combien de fichiers ou dossier devrait si trouver (Pour allouer la place ncessaire)
	 * @return -1 Erreur\n CROMDISKDrive::ER_ALREADY_EXISTS\n 1 Succs
	 * @remarks Le dossier retourn n'est pas ouvert. (ie. OpenDir(CROMDISKFileInfo*, bool))
	 */
	int               CreateDirectory(CROMDISKFileInfo* pParent, CROMDISKFileInfo*& pDir, string& sName, unsigned short nMaxFile);

	/** Permet de crer un fichier
	 * @param[in] pParent Parent du fichier
	 * @param[out] pFile Pointeur sur le fichier cr
	 * @param[in] sName Parent du fichier
	 * @return -1 Erreur\n CROMDISKDrive::ER_ALREADY_EXISTS\n 1 Succs
	 */
	int               CreateFile(CROMDISKFileInfo* pParent, CROMDISKFileInfo*& pFile, string& sName);

	/** Permet de crer un fichier
	 * @param[in] pParent Parent du fichier
	 * @param[out] pFile Pointeur sur le fichier cr
	 * @param[in] sName Parent du fichier
	 * @param[in] sFileName Fichier  copier
	 * @return -1 Erreur\n CROMDISKDrive::ER_OUT_OF_MEM\n CROMDISKDrive::ER_NOT_A_FILE\n CROMDISKDrive::ER_FILE_TOO_SMALL\n CROMDISKDrive::ER_CANT_READ\n
	 * > 0 Nombre d'octets effectivement lus
	 * @remarks Sous Windows la date de modification du fichier est celle du fichier copi, pour les autres systmes c'est la date courante qui est prise
	 */
	int               CreateFile(CROMDISKFileInfo* pParent, CROMDISKFileInfo*& pFile, string& sName, const string& sFileName);

	 /** Permet de supprimer un dossier ou un fichier du lecteur
	  * @param[in] pInfo Dossier ou fichier  supprimer
	  * @return 1 Succs\n -1 Erreur\n CROMDISKDrive::ER_OUT_OF_MEM\n CROMDISKDrive::ER_FILE_USED\n CROMDISKDrive::ER_FOLDER_NOT_EMPTY
	  * @remarks Le pointeur \a pInfo est dtruit aprs le bon droulement de la mthode.
	  * @remarks \a pInfo ne doit pas avoir plusieurs rfrences sinon l'erreur ER_FILE_USED est renvoye.
	  * @remarks Si \a pInfo est un dossier, alors celui-ci doit tre vide sinon l'erreur ER_FOLDER_NOT_EMPTY sera renvoye.
	  */
	int               Kill(CROMDISKFileInfo* pInfo);

public:
	/** Permet de connaitre la taille d'un dossier
	 * @param[in] nNumFile Nombre de fichier contenu dans le dossier
	 * @return Retourne la taille d'un rpertoire en octets en fonction du nombre de fichiers contenus
	 */
	static int          GetDirSize(unsigned short nNumFile);

	/** Permet de connaitre la taille de la racine
	 * @param[in] nNumFile Nombre de fichier contenu dans la racine
	 * @return Retourne la taille d'une racine en octets en fonction du nombre de fichiers contenus
	 */
	static int          GetRootSize(unsigned short nNumFile);

	/** Permet de convertir des octets en secteur
	 * @param[in] nBytes Nombre d'octets
	 * @return Retourne le nombre de secteurs correspondant
	 */
	static int          ConvertBytesToSectors(unsigned int nBytes);

	/** Permet de connaitre la taille relle d'un fichier (ie. La taille prise dans un lecteur)
	 * @param[in] nFileSize Taille en octets du fichier
	 * @return Retourne la taille relle du fichier en octets
	 */
	static unsigned int ConvertFileSizeToRealSize(unsigned int nFileSize);

protected:
	int               InsertData(unsigned int nPos, const void* pData, unsigned int nShift);
	int               RemoveData(unsigned int nPos, unsigned int nSize);

	/** Permet de dfinir la taille de la FAT. La valeur est crite dans l'en-tte ROM-DISK
	 * @param[in] nSize Taille de la FAT en secteurs
	 * @return 1 Succs\n -1 Erreur
	 */
	int               SetFATSize(unsigned short nSize);

	/** Permet de rcuprer la taille de la FAT.
	 * @return La taille de la FAT en secteurs ou -1 si erreur
	 */
	int               GetFATSize();

	/** Permet de dfinir la taille de la racine. La valeur est crite dans l'en-tte ROM-DISK
	 * @param[in] nSize Taille de la racine en secteurs
	 * @return 1 Succs\n -1 Erreur
	 */
	int               SetRootSize(unsigned char nSize);

	/** Permet de rcuprer la taille de la racine.
	 * @return La taille de la racine en octets ou -1 si erreur
	 */
	int               GetRootSize();

	/** Permet de rcuprer la position de la racine.
	 * @return La position de la racine en octets ou -1 si erreur
	 */
	int               GetRootPos();

	/** Permet d'avoir un fichier ou dossier en fonction de sa position dans son parent
	 * @param[in] nParentAddress Adresse du parent en octets
	 * @param[in] nParentSize Taille du parent en octets
	 * @param[in] nPosition Position du fichier ou dossier dans la table d'allocation du parent
	 * @return Les informations lues ou NULL si erreur
	 */
	CROMDISKFileInfo* ReadInfos(unsigned int nParentAddress, unsigned int nParentSize, int nPosition);

	/** Ecrit les donnes d'un fichier ou dossier dans le lecteur
	 * @param[in] pFileInfo Dossier ou fichier
	 * @return -1 Erreur\n 1 Succs
	 */
	int               SetInfos(CROMDISKFileInfo* pFileInfo);

	/** Lit la FAT du lecteur
	 * @return -1 Erreur\n 1 Succs
	 */
	int               ReadFAT();

	/** Ecrit la FAT dans le lecteur
	 * @return -1 Erreur\n 1 Succs
	 * @remarks Appelle DeleteAllFileInfo()
	 * @remarks Mthode qui doit seulement tre appele lors de la fermeture du lecteur.
	 */
	int               WriteFAT();

	/** Permet de rcuprer le dernier emplacement de libre d'un dossier ou de la racine
	 * @param[in] pDir Dossier  vrifier
	 * @param[in] bAutoAdd A vrai si on veut que le conteneur soit agrandi si ncessaire
	 * @return -1 Erreur\n -2 Plus de place (bAutoAdd == false)\n > 0 Emplacement de libre
	 */
	int               GetFreePosition(CROMDISKFileInfo* pDir, bool bAutoAdd = true);

	/** Permet d'avoir l'adresse en octets d'un fichier ou dossier au format interne FAT
	 * @param[in] pFile Fichier ou dossier concern
	 * @return Retourne l'adresse ou -1 si erreur
	 */
	int               GetAddressInSector(const CROMDISKFileInfo* pFile);

	/** Permet de convertir l'adresse interne d'un fichoer ou dossier en une adresse en octets indiquant la
	 * la position exacte du fichier
	 * @param[in] nAddrSectors Adresse en secteur de la FAT
	 * @return La position relle ou -1 si erreur
	 */
	int               ConvertFATAddressToFileAddress(int nAddrSectors);

	/** Permet de rcuprer si une zone existante
	 * @param[in] nAddress Adresse de la zone dans le fichier
	 * @param[in] nParentAddress Adresse du parent de la zone
	 * @param[in] nPosition Position interne de la zone
	 * @return Retourne un pointeur sur la zone recherche ou NULL si non trouv
	 */
	CROMDISKFileInfo* Find(unsigned int nAddress, unsigned int nParentAddress, int nPosition) const;

	/** Permet de savoir si une zone existe
	 * @param[in] pFileInfo Zone  chercher
	 * @return Retourne Vrai si la zone est trouv, sinon faux
	 */
	bool              Find(CROMDISKFileInfo* pFileInfo) const;

	/** Permet de rcuprer des informations sur un fichier en connaissant juste sa position dans le lecteur (Adresse)
	 * @param[in] nAddress Adresse du fichier en octets
	 * @param[in] nParentAddress Addresse du parent en octets
	 * @param[in] nPosition Position du dossier ou fichier dans le parent
	 * @return Retourne le fichier si trouv, sinon NULL
	 * @warning L'adresse est considre comme correcte.
	 */
	CROMDISKFileInfo* GetFileInfo(unsigned int nAddress, unsigned int nParentAddress, int nPosition);

	/** Permet de redimensionner une zone
	 * @param[in] pFileInfo Zone  redimensionner
	 * @param[in] nNewSize Nouvelle taille
	 * @return 1 Succs\n -1 Erreur\n ER_OUT_OF_MEM
	 */
	int               Resize(CROMDISKFileInfo* pFileInfo, unsigned int nNewSize);

	/** Met  jour les fichiers ouverts et la FAT
	 * @param[in] pFileModified Zone redimensionne
	 * @param[in] nShift Dcalage effectu (Multiple de SECTOR_SIZE)
	 * @param[in] nNewSize Nouvelle taille de la zone redimensionne
	 */
	void              UpdateFiles(CROMDISKFileInfo* pFileModified, int nShift, unsigned int nNewSize);

	/** Met  jour les fichiers ouverts
	 * @param[in] pFileModified Zone redimensionne
	 * @param[in] pParent Zone parente  scanner
	 * @param[in] nShift Dcalage effectu (Multiple de SECTOR_SIZE)
	 * @param[in] nNewSize Nouvelle taille de la zone redimensionne
	 */
	void              Update(CROMDISKFileInfo* pFileModified, CROMDISKFileInfo* pParent, int nShift, unsigned int nNewSize);

	/** Met  jour une zone
	 * @param[in] pFileModified Zone redimensionne
	 * @param[in] pInfo Zone  mettre  jour si ncessaire
	 * @param[in] nShift Dcalage effectu (Multiple de SECTOR_SIZE)
	 * @param[in] nNewSize Nouvelle taille de la zone redimensionne
	 */
	void              UpdateFile(CROMDISKFileInfo* pFileModified, CROMDISKFileInfo* pInfo, int nShift, unsigned int nNewSize);
		
protected:
	/** \brief Format des donnes de la FAT pour stocker les informations sur une zone. (Fichier, dossier et nom de volume) */
	struct FileTable{
		char name[8];        ///< Nom de la zone
		char ext[4];         ///< Extension
		char empty[10];      ///< Vide
		unsigned short time; ///< Heure de modification
		unsigned short date; ///< Date de modification
		short adress;        ///< Adresse de dpart en secteurs
		unsigned int size;   ///< Taille de la zone en secteurs
	};

protected:
	/** Initialise une zone de la FAT
	 * @param[in] t Zone  initialiser
	 */
	static void ResetZone(FileTable &t);

private:
	struct TFAT{
		unsigned char nFirst;
		unsigned short nSecond;
	};
	typedef map<int, int> FilesMap;
	typedef list<CROMDISKFileInfo*> OpenFilesMap;

private:
	bool           m_bIsValid;      ///< A vrai si le lecteur est valide (ie. l'en tte est valide et un fichier ou buffer a t donn)
	FilesMap*      m_mFiles;        ///< Liste tous les fichiers et dossiers dans le lecteur actuel (FAT)
	OpenFilesMap*  m_pOpenFiles;    ///< Liste tous les fichiers et dossiers ouverts
	bool           m_bFATModified;  ///< A vrai si la FAT a t modifie (m_lFiles modifie)
	int            m_nAddressFree;  ///< Valeur en octets indiquant la premire adresse libre
	unsigned short m_nSizeInSectors;///< Taille du lecteur en secteurs
};

#endif // ROMDISKDRIVE_H
