
#include "ROMDISKFileInfo.h"
#include "APIFunction.h"
#include "utils.h"
#include "ROMDISKDrive.h"

//-----------------------------------------------------------------------------------
CROMDISKFileInfo::CROMDISKFileInfo(CROMDISKDrive* pDrive)
: CFileInfo()
{
	m_pDrive = pDrive;
	m_nSize  = 0;
	memset(m_pName, 0x20, sizeof(m_pName));
	memset(m_pExtension, 0x20, sizeof(m_pExtension));
}

//-----------------------------------------------------------------------------------
CROMDISKFileInfo::~CROMDISKFileInfo()
{
}

//-----------------------------------------------------------------------------------
bool CROMDISKFileInfo::IsRoot() const
{
	static unsigned char pExt[] = {0x20, 0x20, 0x20, 0x08};
	return memcmp(pExt, m_pExtension, 4) == 0;
}

//-----------------------------------------------------------------------------------
bool CROMDISKFileInfo::IsDir() const
{
	static unsigned char pExt[] = {0x20, 0x20, 0x20, 0x10};
	return memcmp(pExt, m_pExtension, 4) == 0;
}

//-----------------------------------------------------------------------------------
bool CROMDISKFileInfo::IsFile() const
{
	return m_pExtension[3] == 0x20;
}

//-----------------------------------------------------------------------------------
bool CROMDISKFileInfo::IsValid() const
{
	return IsRoot() || IsDir() || IsFile();
}

//-----------------------------------------------------------------------------------
string CROMDISKFileInfo::GetName() const
{
	char buf[9];
	memcpy(buf, m_pName, 8);
	buf[8] = '\0';
	for(int i = 7; i >= 0 && buf[i] == ' '; i--) {
		buf[i] = '\0';
	}
	return buf;
}

//-----------------------------------------------------------------------------------
string CROMDISKFileInfo::GetExtension() const
{
	if(IsFile()) {
		char buf[4];
		memcpy(buf, m_pExtension, 3);
		buf[3] = '\0';
		for(int i = 2; i >= 0 && buf[i] == ' '; i--) {
			buf[i] = '\0';
		}
		return buf;
	}
	return "";
}

//-----------------------------------------------------------------------------------
string CROMDISKFileInfo::GetFullName() const
{
	string sExt = GetExtension();
	if(sExt.empty()) {
		return GetName();
	}
	return GetName() + '.' + sExt;
}

//-----------------------------------------------------------------------------------
unsigned int CROMDISKFileInfo::GetSize() const
{
	return m_nSize;
}

//-----------------------------------------------------------------------------------
const CROMDISKFileInfo::FileTime& CROMDISKFileInfo::GetFileTime() const
{
	return m_nTime;
}

//-----------------------------------------------------------------------------------
void CROMDISKFileInfo::SetName(const char* pName)
{
	if(pName) {
		memcpy(m_pName, pName, sizeof(m_pName));
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKFileInfo::SetExtension(const char* pExt)
{
	if(pExt) {
		memcpy(m_pExtension, pExt, sizeof(m_pExtension));
	}
}

//-----------------------------------------------------------------------------------
void CROMDISKFileInfo::CheckName(string& sName, bool bIsFile)
{
	if(bIsFile) {
		unsigned int i = sName.find_last_of('.');
		if(i == -1) {
			if(sName.length() > 8) {
				sName.resize(8);
			}
		}
		else {
			string name = sName.substr(0, i);
			string ext  = sName.substr(i);
			if(name.length() > 8) {
				name.resize(8);
			}
			if(ext.length() > 4) {
				ext.resize(4);
			}
			sName = name + ext;
		}
	}
	else {
		if(sName.length() > 8) {
			sName.resize(8);
		}
	}
	upper(sName);
}

//-----------------------------------------------------------------------------------
int CROMDISKFileInfo::Rename(string& sNewName)
{
	if(IsValid()) {
		char* pName   = m_pName;
		char* pExt    = m_pExtension;
		char* pSource = (char*)(sNewName.c_str());
		int i;
		upper(pSource);
		for(i = 0; i < 8 && pSource[i] != '\0' && pSource[i] != '.'; i++) {
			pName[i] = pSource[i];
		}
		for(int k = i; k < 8; k++) {
			pName[k] = ' ';
		}
		if(IsFile()) {
			int nExt = 0;
			for(; pSource[i] != '\0' && pSource[i] != '.'; i++);
			if(pSource[i] == '.')
			{
				for(i++; nExt < 3 && pSource[i] != '\0'; i++) {
					pExt[nExt] = pSource[i];
					nExt++;
				}
			}
			for(; nExt < 3; nExt++)	{
				pExt[nExt] = ' ';
			}
		}
		if(m_pDrive->SetInfos(this) > 0) {
			sNewName = GetFullName();
			return 1;
		}
		return -1;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKFileInfo::ReadFile(void* pBuffer, unsigned int nStartPos, unsigned int nSize)
{
	if(pBuffer != NULL && IsFile()) {
		unsigned int nMaxSize = GetSize();
		if(nStartPos > nMaxSize) {
			return CROMDISKDrive::ER_POS_TOO_BIG;
		}
		if(nStartPos + nSize > nMaxSize) {
			nSize = nMaxSize - nStartPos;
		}
		bool bOk = m_pDrive->SetPosition(m_nAddress + nStartPos) > 0
			&& m_pDrive->Read(pBuffer, nSize) > 0;
		if(bOk) {
			return nSize;
		}
		return CROMDISKDrive::ER_CANT_READ;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKFileInfo::WriteFile(const void* pBuffer, unsigned int nStartPos, unsigned int nSize, bool bEnlarge)
{
	if(nStartPos + nSize > m_nSize) {
		int nRet = CROMDISKDrive::ER_FILE_TOO_SMALL;
		if(bEnlarge) {
			nRet = ResizeFile(nStartPos + nSize);
		}
		if(nRet < 1) {
			return nRet;
		}
	}
	bool bOk = m_pDrive->SetPosition(m_nAddress + nStartPos) > 0
		&& m_pDrive->Write(pBuffer, nSize) > 0;
	if(bOk && SetCurrentTime() > 0) {
		return nSize;
	}
	return -1;
}

//-----------------------------------------------------------------------------------
int CROMDISKFileInfo::ResizeFile(unsigned int nSize)
{
	if(IsFile()) {
		int nRet =  m_pDrive->Resize(this, nSize);
		if(nRet > 0) {
			return SetCurrentTime();
		}
		return nRet;
	}
	return CROMDISKDrive::ER_NOT_A_FILE;
}

//-----------------------------------------------------------------------------------
void CROMDISKFileInfo::GetTime(unsigned short& nDate, unsigned short& nTime) const
{
	nDate   = m_nTime.year - 1980;
	nDate <<= 4;
	nDate  += m_nTime.month;
	nDate <<= 5;
	nDate  += m_nTime.day;
	nTime   = m_nTime.hour;
	nTime <<= 6;
	nTime  += m_nTime.min;
	nTime <<= 5;
	nTime  += m_nTime.sec;
}

//-----------------------------------------------------------------------------------
void CROMDISKFileInfo::SetTime(unsigned short nDate, unsigned short nTime)
{
	m_nTime.day   = nDate & 31;
	m_nTime.month = (nDate >> 5) & 15;
	m_nTime.year  = ((nDate >> 9) & 255) + 1980;
	m_nTime.sec  = (nTime & 31) << 1;
	m_nTime.min  = (nTime >> 5) & 63;
	m_nTime.hour = (nTime >> 11) & 31;
}

//-----------------------------------------------------------------------------------
int CROMDISKFileInfo::SetCurrentTime()
{
	time_t t;
	time(&t);
	tm* pTm = gmtime(&t);
	m_nTime.sec   = (pTm->tm_sec >> 1);
	m_nTime.min   = pTm->tm_min;
	m_nTime.hour  = pTm->tm_hour + 1;
	m_nTime.year  = pTm->tm_year + 1900;
	m_nTime.month = pTm->tm_mon + 1;
	m_nTime.day   = pTm->tm_mday;
	return m_pDrive->SetInfos(this);
}
