
#ifndef _ROMDISKFILEINFO_H
#define _ROMDISKFILEINFO_H

#include <F100_Op.h>

#include <time.h>

#include "FileInfo.h"

class CROMDISKDrive;

/*!
	\class CROMDISKFileInfo ROMDISKFileInfo.h
	\brief Permet de grer les fichiers ou dossiers inclus dans un lecteur ROM-DISK
	\author Quaranta David
	\version 1.00

	\ingroup F100TK

	Classe utilse par la classe CROMDISKDrive
	
*/
class DLLEXPORT CROMDISKFileInfo : public CFileInfo
{
	friend class CROMDISKDrive;
public:
	/** \brief Donnes pour la date et heure de modification de la zone pointe */
	struct FileTime{
		char hour;  ///< Heures   (0..23)
		char min;   ///< Minutes  (0..59)
		char sec;   ///< Secondes (0..59)
		char day;   ///< Jours    (1..31)
		char month; ///< Mois     (1..12)
		int  year;  ///< Annes (Valeurs relles)
	};

protected:
	/** Constructeur
	 * @param[in] pDrive Lecteur contenant cette zone
	 */
	CROMDISKFileInfo(CROMDISKDrive* pDrive);

public:
	/** Destructeur */
	virtual ~CROMDISKFileInfo();

	/** Permet de savoir si la zone est la racine (Nom de volume)
	 * @return Retourne Vrai si c'est le nom de volume
	 */
	bool         IsRoot() const;

	/** Permet de savoir si c'est un dossier
	 * @return Retourne Vrai si c'est un dossier
	 */
	bool         IsDir() const;

	/** Permet de savoir si c'est un fichier
	 * @return Retourne Vrai si c'est un fichier
	 */
	bool         IsFile() const;

	/** Permet de savoir si c'est la zone est valide
	 * @return Retourne Vrai si la zone est valide
	 */
	bool         IsValid() const;

	/** Permet de connaitre le nom de la zone
	 * @return Retourne le nom
	 */
	string       GetName() const;

	/** Permet de connaitre l'extension de la zone
	 * @return Retourne l'extension
	 */
	string       GetExtension() const;

	/** Permet de connaitre le nom complet de la zone
	 * @return Retourne le nom complet
	 */
	string       GetFullName() const;

	/** Permet de connaitre la taille de la zone
	 * @return Retourne la taille en octets du fichier ou dossier
	 */
	unsigned int GetSize() const;

	/** @return La date de modification */
	const FileTime& GetFileTime() const;

	/** Permet de renommer le fichier ou dossier
	 * @param[in,out] sNewName contient le nouveau nom valide et est retourn dans cette meme variable le vrai nom inscrit
	 * @return 1 OK\n -1 Erreur
	 */
	int          Rename(string& sNewName);

	/** Permet de lire les donnes d'un fichier
	 * @param[in] pBuffer Zone mmoire qui va recevoir les donnes.
	 * @param[in] nStartPos Position en octets pour indiquer o demarre la lecture
	 * @param[in] nSize Taille indiquant le nombre d'octets  lire
	 * @return -1 Erreur d'argument\n CROMDISKDrive::ER_POS_TOO_BIG\n CROMDISKDrive::ER_CANT_READ\n
	 * > 0 Nombre d'octets effectivement lus
	 */
	int          ReadFile(void* pBuffer, unsigned int nStartPos, unsigned int nSize);

	/** Permet de lire les donnes d'un fichier
	 * @param[in] pBuffer Donnes  crire
	 * @param[in] nStartPos Position en octets pour indiquer o demarre la lecture
	 * @param[in] nSize Taille indiquant le nombre d'octets  crire
	 * @param[in] bEnlarge A vrai si on veut que le fichier soit automatiquement agrandi, sinon le fichier ne sera pas agrandi s'il est trop petit
	 * @return -1 Erreur\n CROMDISKDrive::ER_OUT_OF_MEM\n CROMDISKDrive::ER_NOT_A_FILE\n CROMDISKDrive::ER_FILE_TOO_SMALL\n
	 * > 0 Nombre d'octets effectivement lus
	 */
	int          WriteFile(const void* pBuffer, unsigned int nStartPos, unsigned int nSize, bool bEnlarge = true);

	/** Permet de redimensionner un fichier uniquement
	 * @param[in] nSize Nouvelle taille
	 * @return 1 Succs\n -1 Erreur\n CROMDISKDrive::ER_OUT_OF_MEM\n CROMDISKDrive::ER_NOT_A_FILE
	 */
	int          ResizeFile(unsigned int nSize);

public:
	/** Permet de vrifier et corriger le nom d'une zone
	 * @param[in, out] sName Nom de la zone
	 * @param[in] bIsFile A vrai si la zone est un fichier, sinon  faux
	 */
	static void  CheckName(string& sName, bool bIsFile);

protected:
	/** Permet de dfinir le nom du fichier ou dossier
	 * @param[in] pName Nom
	 * @remarks Les donnes sont copies
	 */
	void    SetName(const char* pName);	

	/** Permet de dfinir l'extension du fichier
	 * @param[in] pExt Extension
	 * @remarks Les donnes sont copies
	 */
	void    SetExtension(const char* pExt);

	/** Permet d'obtenir la date de modification de la zone au format FAT
	 * @param[out] nDate Date au format FAT
	 * @param[out] nTime Heure au format FAT
	 */
	void    GetTime(unsigned short& nDate, unsigned short& nTime) const;

	/** Dfini la date de modification de la zone
	 * @param[in] nDate Date au format FAT
	 * @param[in] nTime Heure au format FAT
	 */
	void    SetTime(unsigned short nDate, unsigned short nTime);

	/** Dfini l'heure actuelle du systme comme la dernire date de modification du fichier ou dossier
	 * @return 1 Succs\n < 0 Erreur
	 */
	int     SetCurrentTime();

protected:
	CROMDISKDrive* m_pDrive;        ///< Lecteur contenant ce fichier
	char           m_pName[8];      ///< Nom
	char           m_pExtension[4]; ///< Extension
	unsigned int   m_nSize;         ///< Taille (octets)
	unsigned int   m_nParentAddress;///< Adresse du parent (octets)
	unsigned int   m_nParentSize;   ///< Taille du parent (octets)
	int            m_nPosition;     ///< Position de la zone dans son parent
	FileTime       m_nTime;         ///< Date de modification du fichier
};

#endif
