// SerialDevice.cpp: implementation of the CSerialDevice class.
//
//////////////////////////////////////////////////////////////////////

#include "SerialDevice.h"

#ifndef WIN32
#	define SCOM "/dev/ttyS%i"
#	include <stdio.h>
#	include <fstream>
#	include <termios.h>
#	include <sys/fcntl.h>
#	include <sys/ioctl.h>
#	define RTS_CONTROL_DISABLE		0
#	define RTS_CONTROL_ENABLE		1
#	define RTS_CONTROL_HANDSHAKE	2
#	define NOPARITY					0
#	define ONESTOPBIT				1
#	define TWOSTOPBITS				2
#	define DTR_CONTROL_DISABLE		0
#	define DTR_CONTROL_ENABLE		1
#	define DTR_CONTROL_HANDSHAKE	2
#endif

#include <stdio.h>


#ifdef MAC
#include <sys/param.h>

#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/serial/IOSerialKeys.h>
#include <IOKit/IOBSD.h>

#define MAXPORTS 9

int nbMacPorts;
char MacPorts[MAXPORTS][MAXPATHLEN];

//Fonctions pour trouver les ports dans un Iterateur
int MyFindModems(io_iterator_t *matchingServices)
{
    kern_return_t       kernResult; 
    mach_port_t         masterPort;
    CFMutableDictionaryRef  classesToMatch;
	
    kernResult = IOMasterPort(MACH_PORT_NULL, &masterPort);
    if (KERN_SUCCESS != kernResult)
        return -1;
	
    // Serial devices are instances of class IOSerialBSDClient.
    classesToMatch = IOServiceMatching(kIOSerialBSDServiceValue);
    if (classesToMatch == NULL)
        return -1;
    else
        CFDictionarySetValue(classesToMatch,
                             CFSTR(kIOSerialBSDTypeKey),
                             CFSTR(kIOSerialBSDRS232Type));
	
    kernResult = IOServiceGetMatchingServices(masterPort, classesToMatch, matchingServices);    
    if (KERN_SUCCESS != kernResult)
		return -1;
		
    return 0;
}

int MyGetPortsPath(io_iterator_t serialPortIterator)
{
    io_object_t     modemService;
	int				i = 0;
    // Iterate across all ports found
    while ((modemService = IOIteratorNext(serialPortIterator)) && i < 9)
    {
		CFTypeRef   deviceFilePathAsCFString;
		deviceFilePathAsCFString = IORegistryEntryCreateCFProperty(modemService,
				CFSTR(kIOCalloutDeviceKey),kCFAllocatorDefault,0);
		if (deviceFilePathAsCFString)
		{
			Boolean result;
			result = CFStringGetCString(deviceFilePathAsCFString,MacPorts[i],MAXPATHLEN, kCFStringEncodingASCII);
			CFRelease(deviceFilePathAsCFString);
			if (result)
				++i;
		}
		(void) IOObjectRelease(modemService);
	}
	
	nbMacPorts = i;
	return 0;
}

#endif

//-----------------------------------------------------------------------------------
CSerialDevice::CSerialDevice()
{
	m_nPort    =  0;
	m_nCable   =  0;
	m_nTimeOut = -1;
	m_hCom     = INVALID_HANDLE_VALUE;
	m_nError   = ER_OK;
	
#ifdef MAC
	io_iterator_t	serialPortIterator;
    MyFindModems(&serialPortIterator);
    MyGetPortsPath(serialPortIterator);
#endif
}

//-----------------------------------------------------------------------------------
CSerialDevice::~CSerialDevice()
{
	ClosePort();
}

//-----------------------------------------------------------------------------------
void CSerialDevice::Init(int nPort, int nCable)
{
	ClosePort();
	m_nPort  = nPort;
	m_nCable = nCable;
}

//-----------------------------------------------------------------------------------
int CSerialDevice::OpenPort(int nSpeed)
{
	if(m_nPort < 0)
	{
		m_nError = ER_OPENPORT;
		return 0;
	}
	int nDTR_Control = DTR_CONTROL_HANDSHAKE;
	int nRTS_Control = RTS_CONTROL_HANDSHAKE;
	switch(m_nCable)
	{
	case 1://FXI
		nRTS_Control = RTS_CONTROL_ENABLE;
		break;
	case 2://SB87
		nRTS_Control = RTS_CONTROL_DISABLE;
		break;
	case 3://MAX232
		nRTS_Control = RTS_CONTROL_ENABLE;
		nDTR_Control = DTR_CONTROL_ENABLE;
		break;
	}
	if(nSpeed > GetMaxSpeed())
	{
		nSpeed = GetMaxSpeed();
	}
	ClosePort();
	char sPortName[128];
	GetPortName(m_nPort, sPortName, 128);
#ifdef WIN32
	m_hCom = OpenDesc(sPortName, GENERIC_READ | GENERIC_WRITE, OPEN_EXISTING);
#else
#	ifdef MAC
		m_hCom = open(sPortName, O_RDWR | O_NOCTTY | O_NDELAY);
#	else
	char sPort[128];
	sprintf(sPort, SCOM, atoi(sPortName+3) - 1);
	m_hCom = open(sPort, O_RDWR | O_NOCTTY | O_NDELAY);
#	endif
#endif
	
	if(m_hCom == INVALID_HANDLE_VALUE)
	{
		m_nError = ER_OPENPORT;
		return 0;
	}
#ifdef WIN32
	DCB dcb;
	//Lecture Configuration initiale
	if (!GetCommState(m_hCom, &dcb))
	{
		m_nError = ER_GETPORT;
		ClosePort();
		return 0;
	}
	/* Gestion de la vitesse */
	dcb.BaudRate = nSpeed;
	/* Gestion du nombre de bits */
	dcb.ByteSize = 8;
	/* Gestion de la parit */
	dcb.Parity   = NOPARITY;
	/* Gestion du Stop Bit */
	dcb.StopBits = TWOSTOPBITS;
	
	dcb.fBinary = 1;
	dcb.fParity = 0;
	dcb.fOutxCtsFlow = 0;
	dcb.fOutxDsrFlow = 0;
	dcb.fDtrControl = nDTR_Control;
	dcb.fDsrSensitivity=0;
	
	dcb.fTXContinueOnXoff = 0;
	dcb.fRtsControl = nRTS_Control;

	// Configure le port
	if(!SetCommState(m_hCom, &dcb))
	{
		ClosePort();
		m_nError = ER_SETPORT;
		return 0;
	}
#else
	fcntl(m_hCom, F_SETFL, 0);	//blocking mode (really usefull ?)
	
	//recupere la configuration actuelle
	struct termios current;
	if(tcgetattr (m_hCom, &current) == -1)
	{
		ClosePort();
		m_nError = ER_GETPORT;
		return 0;
	}
	//Nouvelle configuration
	current.c_iflag = IGNBRK | IGNPAR;
	current.c_oflag = 0;
	current.c_cflag = CS8 | CSTOPB | CREAD | CLOCAL;
	current.c_lflag = 0;
	current.c_cc[VMIN] = 1;
	current.c_cc[VTIME] = 0;
	speed_t speed = B9600;
	switch(nSpeed)
	{
	case 14400:
		speed = B9600;
		break;
	case 38400:
		speed = B38400;
		break;
	case 57600:
		speed = B57600;
		break;
	case 115200:
		speed = B115200;
		break;
	}
	cfsetispeed(&current, speed);
	cfsetospeed(&current, speed);

	tcflush(m_hCom, TCIOFLUSH);	//free the current buffer

	//Sauve cette config
	if(tcsetattr (m_hCom, TCSANOW, &current) == -1)
	{
		ClosePort();
		m_nError = ER_SETPORT;
		return 0;
	}

	int nControl;
	ioctl(m_hCom, TIOCMGET, &nControl);
	if(nRTS_Control == RTS_CONTROL_ENABLE)
	{
		nControl |= TIOCM_RTS;
	}
	else if(nRTS_Control == RTS_CONTROL_DISABLE)
	{
		nControl &= ~TIOCM_RTS;
	}
	if(nDTR_Control == DTR_CONTROL_ENABLE)
	{
		nControl |= TIOCM_DTR;
	}
	else if(nDTR_Control == DTR_CONTROL_DISABLE)
	{
		nControl &= ~TIOCM_DTR;
	}
	ioctl(m_hCom, TIOCMSET, &nControl);
#endif
	Wait(500);
	return SetTimeOut(4000);
}

//-----------------------------------------------------------------------------------
void CSerialDevice::ClosePort()
{
	CloseDesc(m_hCom);	
	m_nTimeOut = -1;
	m_nError   = ER_OK;
}

//-----------------------------------------------------------------------------------
int CSerialDevice::Send(const void* buffer, unsigned int nNbBytes, int nTimeOut)
{
	//Ecrire dans le port srie
	if(m_hCom != INVALID_HANDLE_VALUE)
	{
		SetTimeOut(nTimeOut);
#	ifdef WIN32
		return WriteDesc(m_hCom, buffer, nNbBytes);
#	else
		for(unsigned int i = 0; i < nNbBytes; i++)
		{
			WriteDesc(m_hCom, ((char*)buffer) + i, 1);
		}
		return 1;
#	endif
	}
	return 0;	
}

//-----------------------------------------------------------------------------------
int CSerialDevice::Receive(void* buffer, unsigned int nNbBytes, int nTimeOut)
{
	if(m_hCom != INVALID_HANDLE_VALUE)
	{
		SetTimeOut(nTimeOut);
#	ifdef WIN32
		return ReadDesc(m_hCom, buffer, nNbBytes);
#	else
		unsigned int count  = 0;
		while(count < nNbBytes)
		{
			fd_set rfds;
			struct timeval tv;
			int retval;
			/* Surveiller m_hCom en attente d'octets */
			FD_ZERO(&rfds);
			FD_SET(m_hCom, &rfds);
			/* Pendant timeout secondes maxi */
			tv.tv_sec  = nTimeOut / 1000;
			tv.tv_usec = (nTimeOut % 1000) * 1000;
			retval = select(m_hCom+1, &rfds, NULL, NULL, &tv);
			if(retval == 0)
			{
				return 0;
			}
			if(ReadDesc(m_hCom, (char*)(buffer) + count, 1))
			{
				count++;
			}
		}
		return 1;
#	endif
	}
	return 0;	
}

//-----------------------------------------------------------------------------------
CSerialDevice::ERRORS CSerialDevice::GetLastError() const
{
	return m_nError;	
}

//-----------------------------------------------------------------------------------
const char** CSerialDevice::GetCables(int& nNbCable) const
{
	static const char* cables[] = {"Auto", "FXI", "SB87", "MAX232"};
	nNbCable = 4;
	return cables;
}

//-----------------------------------------------------------------------------------
int CSerialDevice::GetMaxSpeed() const
{
	return 115200;
}

//-----------------------------------------------------------------------------------
int CSerialDevice::SetTimeOut(int nTimeOut)
{
	m_nError = ER_OK;
	if(m_nTimeOut == nTimeOut || nTimeOut < 0)
	{
		return 1;
	}
	m_nTimeOut = nTimeOut;
#ifdef WIN32
	COMMTIMEOUTS tTimeout;
	
	tTimeout.ReadIntervalTimeout         = m_nTimeOut;
	tTimeout.ReadTotalTimeoutMultiplier  = 1;
	tTimeout.ReadTotalTimeoutConstant    = m_nTimeOut;// time_out secondes
	tTimeout.WriteTotalTimeoutMultiplier = 0;
	tTimeout.WriteTotalTimeoutConstant   = 0;
	
	//Configurer le timeout
	return SetCommTimeouts(m_hCom, &tTimeout);
#else
	return 1;
#endif
}

#define KEY_PATH "HARDWARE\\DEVICEMAP\\SERIALCOMM"

//-----------------------------------------------------------------------------------
int CSerialDevice::GetNbPort() const
{
#ifdef WIN32
	HKEY h;
	RegOpenKeyEx(HKEY_LOCAL_MACHINE, KEY_PATH, 0, KEY_READ, &h);
	if(h != NULL)
	{
		DWORD nNbPort = 0;
		RegQueryInfoKey(h, 0, 0, 0, 0, 0, 0, &nNbPort, 0, 0, 0, 0);
		RegCloseKey(h);
		if(nNbPort > 0) {
			return nNbPort;
		}
	}
#endif
#ifdef MAC
	return nbMacPorts;
#else
	return 9;
#endif
}

//-----------------------------------------------------------------------------------
void CSerialDevice::GetPortName(int nPort, char* pBuffer, int nBufferSize) const
{
#ifdef WIN32
	HKEY h;
	RegOpenKeyEx(HKEY_LOCAL_MACHINE, KEY_PATH, 0, KEY_READ, &h);
	if(h != NULL)
	{
		char name[128];
		DWORD nNameSize = sizeof(name);
		pBuffer[0] = '\0';
		RegEnumValue(h, nPort, name, &nNameSize, 0, 0, (LPBYTE)pBuffer, (LPDWORD)&nBufferSize);
		RegCloseKey(h);
		return;
	}
#endif
#ifdef MAC
	sprintf(pBuffer, "%s", MacPorts[nPort]);
#else
	sprintf(pBuffer, "COM%i", nPort + 1);
#endif
}
