// ConsoleWindow.cpp: implementation of the ConsoleWindow class.
//
//////////////////////////////////////////////////////////////////////

#include "ConsoleWindow.h"

#include <qlayout.h>
#include <qtooltip.h>

#include <assert.h>

#include <OpacityButton.h>
#include <IMainWindow.h>
#include <image_defs.h>

//-----------------------------------------------------------------------------------
ConsoleWindow* ConsoleWindow::m_pConsole = NULL;

#if defined (_DEBUG) && defined(WIN32)
#include <crtdbg.h>

//-----------------------------------------------------------------------------------
int OurReportingFunction(int nReportType, char* pUserMessage, int* pRetVal)
{
	if(ConsoleWindow::m_pConsole != NULL) {
		if(nReportType == _CRT_ASSERT) {
			ConsoleWindow::m_pConsole->OnInformation(pUserMessage);
			return 1;
		}
		else if(nReportType == _CRT_ERROR) {
			ConsoleWindow::m_pConsole->OnError(pUserMessage);
			return 1;
		}
		else if(nReportType == _CRT_WARN) {
			ConsoleWindow::m_pConsole->OnWarning(pUserMessage);
			return 1;
		}
	}
	return 0;
}
#endif

//-----------------------------------------------------------------------------------
void QtHandler(QtMsgType type, const char * msg)
{
	if(ConsoleWindow::m_pConsole != NULL) {
		switch(type)
		{
		case QtDebugMsg:
			ConsoleWindow::m_pConsole->OnInformation(msg);
			break;
		case QtWarningMsg:
			ConsoleWindow::m_pConsole->OnWarning(msg);
			break;
		case QtFatalMsg:
			ConsoleWindow::m_pConsole->OnError(msg);
			break;
		}
	}
}

//-----------------------------------------------------------------------------------
ConsoleWindow::ConsoleWindow(QWidget* parent)
	: QDockWindow(parent)
	, m_mainWidget(this)
	, m_output(&m_mainWidget)
	, m_clear(&m_mainWidget)
	, m_dOpacity(1.0)
	, m_pOpacity(NULL)
{
	QPixmap pixmap;
	setWidget(&m_mainWidget);
	setResizeEnabled(TRUE);
	setCloseMode(QDockWindow::Always);

	m_output.setReadOnly(TRUE);
	m_output.setHScrollBarMode(QScrollView::Auto);
	m_output.setVScrollBarMode(QScrollView::Auto);
	m_output.setWordWrap(QTextEdit::NoWrap);

	QBoxLayout*	pMainLayout   = new QBoxLayout(&m_mainWidget, QBoxLayout::LeftToRight);
	QBoxLayout* pButtonLayout = new QBoxLayout(pMainLayout,   QBoxLayout::TopToBottom);

	pMainLayout->addWidget(&m_output);
	m_clear.setAutoRaise(true);

	pButtonLayout->addWidget(&m_clear);

	if(COpacityButton::OpacityIsAvalaible()) {
		m_pOpacity = new COpacityButton(&m_mainWidget, this);
		m_pOpacity->setAutoRaise(true);
		pButtonLayout->addWidget(m_pOpacity);
		connect(m_pOpacity, SIGNAL(OpacityChanged()), SLOT(OnOpacityChanged()));
	}
	pButtonLayout->addStretch(100);

	m_clear.setIconSet(QPixmap::fromMimeSource(IMG_CLEAR));

	connect(&m_clear, SIGNAL(clicked()), &m_output, SLOT(clear()));
	languageChange();
}

//-----------------------------------------------------------------------------------
ConsoleWindow::~ConsoleWindow()
{
	if(this == m_pConsole) {
		m_pConsole = NULL;
	}
}

//-----------------------------------------------------------------------------------
ConsoleWindow* ConsoleWindow::GetMainInstance(QWidget* parent)
{
	if(m_pConsole == NULL) {
		m_pConsole = new ConsoleWindow(parent);
		qInstallMsgHandler( QtHandler );
#if defined (_DEBUG) && defined (WIN32)
		_CrtSetReportHook(OurReportingFunction);
#endif
	}
	return m_pConsole;
}

//-----------------------------------------------------------------------------------
double ConsoleWindow::GetOpacity() const
{
	return m_dOpacity;
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::SetOpacity(double dOpacity)
{
	if(dOpacity != m_dOpacity && dOpacity > 0.0) {
		m_dOpacity = dOpacity;
		Update();
	}
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::OnWarning(const QString& msg)
{
	m_output.setColor(Qt::blue);
	m_output.append(tr("Attention :") + " " + msg);
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::OnError(const QString& msg)
{
	assert(parentWidget() != NULL);
	parentWidget()->show();
	m_output.setColor(Qt::red);
	m_output.append(tr("Erreur :") + " " + msg);
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::OnInformation(const QString& msg)
{
	m_output.setColor(m_output.colorGroup().text());
	m_output.append(msg);
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::undock()
{
	QDockWindow::undock();
	Update();
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::dock()
{
	QDockWindow::dock();
	Update();
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::Update(bool bForceUpdate)
{
	if(m_pOpacity != NULL && (isShown() || bForceUpdate)) {
		if(area() == NULL) {
			m_pOpacity->SetOpacity(m_dOpacity);
			m_pOpacity->show();
		}
		else {
			m_pOpacity->hide();
		}
	}
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::show()
{
	Update(TRUE);
	QDockWindow::show();
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::OnOpacityChanged()
{
	assert(m_pOpacity != NULL);
	m_dOpacity = m_pOpacity->GetOpacity();
	assert(m_dOpacity != 0.0);
}

//-----------------------------------------------------------------------------------
void ConsoleWindow::languageChange()
{
	QToolTip::add(&m_clear, tr("Efface le texte"));
}
