
#include "PainterDevice.h"

#include <qpainter.h>
#include <qimage.h>
#include <qfontmetrics.h>

#include <math.h>

//-----------------------------------------------------------------------------------
CPainterDevice::CPainterDevice(QWidget* parent, bool bEnableSmoothScale)
:QWidget(parent, 0, WNoAutoErase)
{
	m_bEnableSmoothScale = bEnableSmoothScale;
	setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
	ClearTempLine();
	ClearTempText();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::SetSize(int w, int h)
{
	pix.create(w, h, 1, 2, QImage::BigEndian);
	pix.setColor(0, qRgb(255,255,255));
	pix.setColor(1, 0);
	pix.fill(0);
	update();
	UpdateRatio();
}

//-----------------------------------------------------------------------------------
QSize CPainterDevice::GetSize() const
{
	return pix.size();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::SetPixel(int x, int y, int state)
{
	pix.setPixel(x, y, state);
	update();
}

//-----------------------------------------------------------------------------------
int CPainterDevice::GetPixel(int x, int y)
{
	return !pix.pixel(x, y);
}

//-----------------------------------------------------------------------------------
void CPainterDevice::InvertPixels()
{
	pix.invertPixels();
	update();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::InvertPixels(int x, int y, int w, int h)
{
	for(int i=x;i<x+w;i++)
	{
		for(int j=y; j < y + h; j++)
		{
			if(pix.pixel(i, j) != -1)
			{
				pix.setPixel(i, j, 0);
			}
			else
			{
				pix.setPixel(i, j, 1);
			}
		}
	}
	update();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::Fill(int value)
{
	pix.fill(value);
	update();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::GetWorldPos(int x, int y, int& nWorldX, int& nWorldY)
{ 
	int nStartX = (width()  - pix.width()  * m_fHRatio) / 2;
	int nStartY = (height() - pix.height() * m_fVRatio) / 2;
	nWorldX = (x - nStartX) / m_fHRatio;
	nWorldY = (y - nStartY) / m_fVRatio;
}

//-----------------------------------------------------------------------------------
void CPainterDevice::DrawImage(QPainter& p, const QImage& image)
{
	QImage tmp;
	if(m_bEnableSmoothScale) {
		tmp = image.smoothScale(image.width() * m_fHRatio, image.height() * m_fVRatio);
	}
	else{
		tmp = image.scale(image.width() * m_fHRatio, image.height() * m_fVRatio);
	}
	p.drawImage(GetStartX(), GetStartY(), tmp);
}

//-----------------------------------------------------------------------------------
int CPainterDevice::GetStartX() const
{
	return (width()  - (int)(pix.width()  * m_fHRatio)) >> 1;
}

//-----------------------------------------------------------------------------------
int CPainterDevice::GetStartY() const
{
	return (height() - (int)(pix.height() * m_fVRatio)) >> 1;
}

//-----------------------------------------------------------------------------------
void CPainterDevice::DrawGrid(QPainter& p)
{
	p.save();
	int x = GetStartX();
	int y = GetStartY();
	int w = pix.width();
	int h = pix.height();
	p.scale(m_fHRatio, m_fVRatio);
	if(m_fHRatio >= 4.0f && m_fVRatio >= 4.0f)
	{
		p.setPen(QColor(192, 192, 192));
		for(int xx = 0; xx < w; xx += 1) {
			p.drawLine(xx+x, y, xx+x, y+h);
		}
		for(int yy = 0; yy < h; yy += 1) {
			p.drawLine(x, yy+y, x+w, yy+y);
		}
	}
	p.restore();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::paintEvent(QPaintEvent* e)
{
	QPixmap pm(size());
	pm.fill(this, 0, 0);
	QPainter p(&pm, this);
	QImage img = pix.copy();
	if(m_qP1.x() >= 0) {
		DrawLineInImage(img, m_qP1.x(), m_qP1.y(), m_qP2.x(), m_qP2.y(), m_bLineColor);
	}
	DrawImage(p, img);
	if(!m_sTmpText.isEmpty()) {
		p.save();
		p.scale(m_fHRatio, m_fVRatio);
		p.drawImage(m_nTmpTextX, m_nTmpTextY, ConvertStringToImage(m_sTmpText, p.font()));
		p.restore();
	}
	DrawGrid(p);
	p.end();
	bitBlt(this, 0, 0, &pm);
}

//-----------------------------------------------------------------------------------
void CPainterDevice::resizeEvent(QResizeEvent* e)
{
	QWidget::resizeEvent(e);
	UpdateRatio();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::UpdateRatio()
{
	m_fHRatio = width()  / (float)pix.width();
	m_fVRatio = height() / (float)pix.height();
	if(m_fHRatio > m_fVRatio)
	{
		m_fHRatio = m_fVRatio;
	}
	if(m_fHRatio < m_fVRatio)
	{
		m_fVRatio = m_fHRatio;
	}
}

//-----------------------------------------------------------------------------------
void CPainterDevice::mousePressEvent(QMouseEvent* e)
{
	QPoint p = e->pos();
	emit mousePressed(e->button(), p.x(), p.y());
}

//-----------------------------------------------------------------------------------
void CPainterDevice::mouseMoveEvent(QMouseEvent* e)
{
	QPoint p = e->pos();
	emit mouseMoved(e->state(), p.x(), p.y());
}

//-----------------------------------------------------------------------------------
void CPainterDevice::mouseReleaseEvent(QMouseEvent* e)
{
	QPoint p = e->pos();
	emit mouseReleased(e->button(), p.x(), p.y());
}

//-----------------------------------------------------------------------------------
bool CPainterDevice::LoadFile(const QString& sFileName)
{
	QImage image;
	if(!image.load(sFileName))
	{
		return false;
	}
	image = image.smoothScale(pix.width(), pix.height());
	image = image.convertDepth(1);
	image = image.convertBitOrder(QImage::BigEndian);
	if(image.isNull())
	{
		return false;
	}
	image.setColor(0, qRgb(255, 255, 255));
	image.setColor(1, 0);
	pix = image;
	update();
	return true;
}

//-----------------------------------------------------------------------------------
bool CPainterDevice::SaveFile(const QString& sFileName)
{
	QString sExt = sFileName.section('.', -1);
	if(sExt == sFileName) {
		sExt = "BMP";
	}
	return pix.save(sFileName, sExt.upper());
}

//-----------------------------------------------------------------------------------
void CPainterDevice::DrawLineInImage(QImage& image, float x1, float y1, float x2, float y2, bool bColor)
{
	float xx = x2 - x1;
	float yy = y2 - y1;
	int max = fabs(xx);
	if(max < fabs(yy)) {
		max = fabs(yy);
	}
	xx /= max;
	yy /= max;
	for(int i = 0; i < max; i++)
	{
		image.setPixel((int)(x1), (int)y1, bColor);
		x1 += xx;
		y1 += yy;
	}
	image.setPixel((int)(x2), (int)y2, bColor);
}

//-----------------------------------------------------------------------------------
QImage CPainterDevice::ConvertStringToImage(const QString& text, const QFont& font)
{
	QFontMetrics qfm(font);
	QPixmap pm(qfm.width(text), qfm.height());
	pm.fill(Qt::white);
	QPainter p(&pm);
	p.drawText(QPoint(0, qfm.ascent() - qfm.descent()), text);
	p.end();
	return pm.convertToImage();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::SetTempLine(int x1, int y1, int x2, int y2, bool bColor)
{
	m_qP1.setX(x1);
	m_qP1.setY(y1);
	m_qP2.setX(x2);
	m_qP2.setY(y2);
	m_bLineColor = bColor;
	update();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::SetTempText(int x, int y, const QString& text)
{
	m_sTmpText  = text;
	m_nTmpTextX = x;
	m_nTmpTextY = y;
	update();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::DrawLine(int x1, int y1, int x2, int y2, bool bColor)
{
	DrawLineInImage(pix, x1, y1, x2, y2, bColor);
	update();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::SetData(uchar* pData, unsigned int nSize)
{
	memcpy(pix.bits(), pData, nSize);
	update();
}

//-----------------------------------------------------------------------------------
uchar* CPainterDevice::GetData()
{
	return pix.bits();
}

//-----------------------------------------------------------------------------------
void CPainterDevice::ClearTempLine()
{
	m_qP1.setX(-1);
}

//-----------------------------------------------------------------------------------
void CPainterDevice::ClearTempText()
{
	m_sTmpText = QString::null;
}
