// BasicEditor.cpp: implementation of the CBasicEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "BasicEditor.h"
#include "BasicZone.h"
#include "MLCSyntaxHighlighter.h"
#include "RamManagerOption.h"
#include "Wrapper.h"
#include "SyntaxHighligterFactory.h"
#include "SyntaxProperties.h"
#include "FindDialog.h"
#include "TextEditor.h"

#include <qcstring.h>
#include <qaction.h>
#include <qcombobox.h>
#include <qfontdatabase.h>
#include <qstatusbar.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qsimplerichtext.h>
#include <qprinter.h>
#include <qlayout.h>

#include <assert.h>

#include <FixedToolBar.h>
#include <MessageBox.h>
#include <image_defs.h>
#include <language.h>
#include <IOptionWindow.h>
#include <IMainWindow.h>

#define IMG_PRINT "rammanager/print.png"

#include "BASICWrapper.h"

//-----------------------------------------------------------------------------------
CBasicEditor::CBasicEditor(QWidget* parent)
	: CEditor(parent)
	, m_pBasicZone(NULL)
	, m_bModified(FALSE)
	, m_bBlockRefresh(FALSE)
	, m_pSyntax(NULL)
	, m_pPrinter(NULL)
	, m_bNoChanges(TRUE)
{
	QWidget* pCentral = new QWidget(this);

	m_pTextArea   = new CTextEditor(pCentral);
	m_pFindDialog = new CFindDialog(m_pTextArea);
	m_pLblSize    = new QLabel(statusBar());
	m_pLblCursor  = new QLabel(statusBar());

	setCentralWidget(pCentral);

	QHBoxLayout* pLayout = new QHBoxLayout(pCentral, 2);

	pLayout->addWidget(m_pTextArea, 4);

	CFixedToolBar* pToolBar = new CFixedToolBar(this);
	m_pSaveAction      = new QAction(this);
	m_pUndoAction      = new QAction(this);
	m_pRedoAction      = new QAction(this);
	m_pPrintAction     = new QAction(this);
	m_pOptionAction    = new QAction(this);
	m_pFindAction      = new QAction(this);

	m_pSaveAction->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pUndoAction->addTo(pToolBar);
	m_pRedoAction->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pFindAction->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pPrintAction->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pOptionAction->addTo(pToolBar);
	pToolBar->addSeparator();
	m_pSyntaxList = new QComboBox(pToolBar);

	pToolBar->AddStretchableWidget();

	m_pSaveAction->setIconSet(QPixmap::fromMimeSource(IMG_SAVE));
	m_pUndoAction->setIconSet(QPixmap::fromMimeSource(IMG_UNDO));
	m_pRedoAction->setIconSet(QPixmap::fromMimeSource(IMG_REDO));
	m_pPrintAction->setIconSet(QPixmap::fromMimeSource(IMG_PRINT));
	m_pOptionAction->setIconSet(QPixmap::fromMimeSource(IMG_OPTION));
	m_pFindAction->setIconSet(QPixmap::fromMimeSource(IMG_SEE));

	m_pSaveAction->setAccel(Qt::CTRL | Qt::Key_S);
	m_pUndoAction->setAccel(Qt::CTRL | Qt::Key_Z);
	m_pRedoAction->setAccel(Qt::CTRL | Qt::Key_Y);
	m_pPrintAction->setAccel(Qt::CTRL | Qt::Key_P);
	m_pFindAction->setAccel(Qt::CTRL | Qt::Key_F);

	setUsesTextLabel(TRUE);

	m_pTextArea->setWordWrap(QTextEdit::NoWrap);
	m_pTextArea->setFocus();

	connect(m_pTextArea,     SIGNAL(textChanged()), this,        SLOT(OnTextModified()));
	connect(m_pSaveAction,   SIGNAL(activated()),   this,        SLOT(OnSave()));
	connect(m_pUndoAction,   SIGNAL(activated()), m_pTextArea,   SLOT(undo()));
	connect(m_pRedoAction,   SIGNAL(activated()), m_pTextArea,   SLOT(redo()));
	connect(m_pPrintAction,  SIGNAL(activated()),   this,        SLOT(OnPrint()));
	connect(m_pOptionAction, SIGNAL(activated()),   this,        SLOT(OnShowOption()));
	connect(m_pFindAction,   SIGNAL(activated()), m_pFindDialog, SLOT(show()));
	connect(m_pSyntaxList,   SIGNAL(activated(int)), this,       SLOT(OnSyntaxChanged(int)));
	connect(m_pTextArea,     SIGNAL(cursorPositionChanged(int, int)), SLOT(OnCursorPositionChanged(int, int)));

	DoInitWords();
	
	statusBar()->addWidget(m_pLblCursor, 0, TRUE);
	statusBar()->addWidget(m_pLblSize,   0, TRUE);

	resize(420, 300);
	GetOptionWindow()->AddListener(this);
	OnOptionChanged();
}

//-----------------------------------------------------------------------------------
CBasicEditor::~CBasicEditor()
{
	GetOptionWindow()->RemoveListener(this);
	delete m_pSyntax;
	if(m_pPrinter != NULL) {
		delete m_pPrinter;
	}
}

//-----------------------------------------------------------------------------------
bool CBasicEditor::close(bool alsoDelete)
{
	if(m_bModified) {
		int nRet = CMessageBox::MsgYesNoCancel(this, tr("Voulez-vous sauver les changements ?"));
		if(nRet == QMessageBox::Yes) {
			Save();
		}
		else if(nRet == QMessageBox::Cancel) {
			return false;
		}
	}
	return CEditor::close(alsoDelete);
}

//-----------------------------------------------------------------------------------
bool CBasicEditor::SetFileZone(CFileZone* pFile)
{
	CBasicZone* pBasicZone = dynamic_cast<CBasicZone*>(pFile);
	if(pBasicZone != NULL) {
		if(m_pBasicZone != NULL) {
			disconnect(m_pBasicZone);
		}
		m_pBasicZone = pBasicZone;
		connect(m_pBasicZone, SIGNAL(DataChanged()), SLOT(OnDataChanged()));
		connect(m_pBasicZone, SIGNAL(Renamed()),     SLOT(OnFileRenamed()));
		RefreshData();
		UpdateCaption();
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------------
void CBasicEditor::UpdateSize()
{
	if(m_pBasicZone != NULL) {
		m_pLblSize->setText(tr("%1 octets").arg(m_pBasicZone->GetSize() + 24));
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::RefreshData()
{
	assert(m_pBasicZone != NULL);
	if(m_pSyntax == NULL) {
		m_pSyntax = CSyntaxHighligterFactory::GetInstance()->CreateSyntax(m_pBasicZone->GetData(), m_pBasicZone->GetSize(), m_pTextArea);
		m_pSyntaxList->setCurrentText(m_pSyntax->GetProperties()->GetName());
	}
	assert(m_pSyntax != NULL);
	UpdateKeywords();
	QString s = m_pSyntax->GetWrapper()->ConvertCasioDataToText(m_pBasicZone->GetData(), m_pBasicZone->GetSize());
	m_bModified = TRUE;
	m_pTextArea->setText(s);
	m_bModified = FALSE;
	UpdateCaption();
	UpdateSize();
}

//-----------------------------------------------------------------------------------
void CBasicEditor::DoInitWords()
{
	CEditor::DoInitWords();

	m_pSaveAction->setText(tr("Sauver"));
	m_pUndoAction->setText(tr("Annuler"));
	m_pRedoAction->setText(tr("Rpter"));
	m_pPrintAction->setText(tr("Imprimer"));
	m_pFindAction->setText(tr("Rechercher"));
	m_pOptionAction->setText(CLanguage::GetWordTranslation(CLanguage::WO_OPTION));

	m_pSyntaxList->clear();
	SyntaxPropertiesList::const_iterator it  = CSyntaxHighligterFactory::GetInstance()->GetSyntaxPropertiesList().begin();
	SyntaxPropertiesList::const_iterator end = CSyntaxHighligterFactory::GetInstance()->GetSyntaxPropertiesList().end();

	for(; it != end; it++) {
		m_pSyntaxList->insertItem((*it)->GetName());
	}
	if(m_pSyntax != NULL) {
		m_pSyntaxList->setCurrentText(m_pSyntax->GetProperties()->GetName());
	}
	UpdateStatusBar();
}

//-----------------------------------------------------------------------------------
void CBasicEditor::UpdateStatusBar()
{
	int para, index;
	m_pTextArea->getCursorPosition(&para, &index);
	OnCursorPositionChanged(para, index);
	UpdateSize();
}

//-----------------------------------------------------------------------------------
void CBasicEditor::UpdateCaption()
{
	if(m_pBasicZone != NULL) {
		QString s = tr("Editeur") + " - " + m_pBasicZone->GetName(); 
		if(m_bModified) {
			s += " *";
		}
		setCaption(s);
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnDataChanged()
{
	if(!m_bBlockRefresh) {
		RefreshData();
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnFileRenamed()
{
	UpdateCaption();
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnTextModified()
{
	if(!m_bModified) {
		m_bNoChanges = FALSE;
		m_bModified  = TRUE;
		UpdateCaption();
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnSave()
{
	if(m_bModified) {
		assert(m_pSyntax != NULL && m_pBasicZone != NULL);
		QString s = m_pSyntax->GetWrapper()->ConvertTextToCasioData(m_pTextArea->text());
		char* pData = new char[s.length()];
		if(pData != NULL) {
			memcpy(pData, s.latin1(), s.length());
			m_bBlockRefresh = TRUE;
			m_pBasicZone->SetData(pData, s.length(), TRUE);
			m_bBlockRefresh = FALSE;
			m_bModified = FALSE;
			UpdateCaption();
			UpdateSize();
		}
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnPrint()
{
	if(m_pPrinter == NULL) {
		m_pPrinter = new QPrinter(QPrinter::PrinterResolution);
		m_pPrinter->setOrientation(QPrinter::Portrait);
		m_pPrinter->setColorMode(QPrinter::GrayScale);
		m_pPrinter->setPageSize(QPrinter::A4);
		m_pPrinter->setMinMax(1, 1000);
		m_pPrinter->setPrintRange(QPrinter::AllPages);
		m_pPrinter->setOptionEnabled(QPrinter::PrintToFile,    false);
		m_pPrinter->setOptionEnabled(QPrinter::PrintPageRange, true);
		m_pPrinter->setNumCopies(1);
		m_pPrinter->setOutputToFile(FALSE);
		m_pPrinter->setFromTo(1, 1);
	}
	m_pPrinter->setFullPage(TRUE);
	m_pPrinter->setOptionEnabled(QPrinter::PrintSelection, m_pTextArea->hasSelectedText());

	QString sText = QStyleSheet::convertFromPlainText(m_pTextArea->text());
	if ( m_pPrinter->setup(this) ) { // printer dialog
		statusBar()->message(tr("Impression..."));
		QPainter p;
		if( !p.begin( m_pPrinter ) ) { // paint on printer
			statusBar()->message(tr("Impression annule"), 2000);
			return;
		}
		if(m_pPrinter->printRange() == QPrinter::Selection) {
			sText = QStyleSheet::convertFromPlainText(m_pTextArea->selectedText());
		}
		QPaintDeviceMetrics metrics(p.device());
		uint top, left, bottom, right;
		m_pPrinter->margins(&top, &left, &bottom, &right);
		uint nXMargin = (uint)((1/2.54)*metrics.logicalDpiX());
		uint nYMargin = (uint)((1/2.54)*metrics.logicalDpiY());

		top    = QMAX(nYMargin,    top);
		bottom = QMAX(nYMargin, bottom);
		left   = QMAX(nXMargin,   left);
		right  = QMAX(nXMargin,  right);

		QRect view(left, top, metrics.width()-left-right, metrics.height()-top-bottom);
		QSimpleRichText richText(sText,
			m_pTextArea->QWidget::font(),
			m_pTextArea->context(),
			m_pTextArea->styleSheet(),
			m_pTextArea->mimeSourceFactory(),
			view.height());
		richText.setWidth(&p, view.width());
		int page = m_pPrinter->fromPage();
		int to   = m_pPrinter->toPage();
		if(page == 0) {
			page = 1;
		}
		p.translate(0, -view.height() * (page - 1));
		view.moveBy(0,  view.height() * (page - 1));
		bool bNewPage = FALSE;
		int nYShift = p.fontMetrics().ascent();
		while((view.top() - (int)top < richText.height()) && (to <= 0 || page <= to)) {
			if(bNewPage) {
				m_pPrinter->newPage();
			}
			bNewPage = TRUE;
			view.setHeight(view.height() - nYShift);
			richText.draw( &p, left, top, view, colorGroup() );
			QString sPage = QString::number( page );
			view.setHeight(view.height() + nYShift);
			p.drawText(view.right() - p.fontMetrics().width(sPage), view.bottom(), sPage);

			view.moveBy(0,  view.height());
			p.translate(0, -view.height());
			page++;
		}
		if(!bNewPage) { //No page print, abort to avoid blank page
			m_pPrinter->abort();
		}
		statusBar()->message(tr("Impression termine"), 2000);
	}
	else {
		statusBar()->message(tr("Impression annule"), 2000);
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::Save()
{
	OnSave();
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnShowOption()
{
	GetOptionWindow()->GetMainWindow()->GetOptionWindow()->ShowSettingsWidget(GetOptionWindow());
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnOptionChanged()
{
	QPalette p = m_pTextArea->palette();
	p.setColor(QPalette::Active,   QColorGroup::Base, GetOptionWindow()->GetBackColor());
	p.setColor(QPalette::Inactive, QColorGroup::Base, GetOptionWindow()->GetBackColor());
	m_pTextArea->setPalette(p);
	m_pTextArea->SetSyntax(m_pSyntax);
	if(m_pSyntax != NULL) {
		m_pSyntax->rehighlight();
	}
}

//-----------------------------------------------------------------------------------
bool CBasicEditor::event(QEvent* pEvent)
{
	switch(pEvent->type()) {
	case QEvent::ApplicationFontChange:
	case QEvent::ApplicationPaletteChange:
		OnOptionChanged();
	}
	return CEditor::event(pEvent);
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnSyntaxChanged(int nIndex)
{
	CSyntaxProperties* pProperties = CSyntaxHighligterFactory::GetInstance()->GetSyntaxPropertiesList()[nIndex];
	if(pProperties != NULL) {
		QString sData = m_pTextArea->text();
		if(m_pSyntax != NULL) {
			delete m_pSyntax;
			m_pSyntax = NULL;
		}
		CSyntaxHighlighter* pSyntax = pProperties->CreateSyntax(m_pTextArea);
		if(pSyntax != NULL) {
			m_pSyntax = pSyntax;
			m_pSyntax->rehighlight();
		}
		if(m_bNoChanges) {
			RefreshData();
		}
		else {
			UpdateKeywords();
		}
	}
}

//-----------------------------------------------------------------------------------
void CBasicEditor::UpdateKeywords()
{
	m_pTextArea->SetSyntax(m_pSyntax);
}

//-----------------------------------------------------------------------------------
void CBasicEditor::OnCursorPositionChanged(int para, int index)
{
	m_pLblCursor->setText(tr("Ln %1, Col %2").arg(para).arg(index));
}
