// LeftPanel.cpp: implementation of the LeftPanel class.
//
//////////////////////////////////////////////////////////////////////

#include "LeftPanel.h"

#include <qheader.h>
#include <qlistview.h>
#include <qpopupmenu.h>

#include <F100_Debug.h>
#include <CalcInfo.h>
#include <FileList.h>
#include <language.h>
#include <MessageBox.h>

#include "FileZone.h"
#include "FileZoneItem.h"
#include "FileLoader.h"
#include "RAMManager.h"

//-----------------------------------------------------------------------------------
LeftPanel::LeftPanel(RAMManager* pRamManager, CasioType ct)
	: QDockWindow(pRamManager)
{
	m_pRamManager = pRamManager;
	m_pList = new QListView(this);

	setWidget(m_pList);
	setResizeEnabled(TRUE);
	setCloseMode(QDockWindow::Never);

	m_pList->addColumn(QString::null);
	m_pList->header()->hide();
	m_pList->setRootIsDecorated(TRUE);
	m_pList->setSorting(0);

	m_pRootItem   = new CFileItem(CFileItem::Directory, m_pList);
	m_pBASICItem  = new CFileItem(CFileItem::Directory, m_pRootItem);
	m_pListItem   = new CFileItem(CFileItem::Directory, m_pRootItem);
	m_pMatrixItem = new CFileItem(CFileItem::Directory, m_pRootItem);
	m_pImageItem  = new CFileItem(CFileItem::Directory, m_pRootItem);

	m_pList->ensureItemVisible(m_pBASICItem);

	SetCalc(ct);

	connect(m_pList, SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), SLOT(OnDoubleClick(QListViewItem*)));
	connect(m_pList, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)), SLOT(OnMenuRequested(QListViewItem*, const QPoint&)));
}

//-----------------------------------------------------------------------------------
void LeftPanel::DoInitWords()
{
	setCaption(tr("Navigateur"));
	m_pRootItem->setText(0,   CCalcInfo::GetCalcType(CFileZone::GetCasioType()));
	m_pBASICItem->setText(0,  "BASIC");
	m_pListItem->setText(0,   tr("Liste"));
	m_pMatrixItem->setText(0, tr("Matrice"));
	m_pImageItem->setText(0,  tr("Image"));
}

//-----------------------------------------------------------------------------------
void LeftPanel::SetCalc(CasioType ct)
{
	CFileZone::SetCasioType(ct);

	memset(m_pZones, 0, sizeof(m_pZones));
	
	m_pZones[1] = m_pBASICItem;
	m_pZones[5] = m_pListItem;
	m_pZones[6] = m_pMatrixItem;
	m_pZones[7] = m_pImageItem;
	DoInitWords();
}

//-----------------------------------------------------------------------------------
bool LeftPanel::IsSupported(unsigned short nZone) const
{
	return nZone <= 12 && m_pZones[nZone] != NULL;
}

//-----------------------------------------------------------------------------------
void LeftPanel::AddFile(CFileZone* pFile)
{
	assert(pFile != NULL && IsSupported(pFile->GetZone()));
	FileMap::Iterator it = m_mFiles.find(pFile);

	if(it == m_mFiles.end()) {
		m_mFiles[pFile] = new CFileZoneItem(m_pZones[pFile->GetZone()]);
	}
	UpdateFile(pFile);
}

//-----------------------------------------------------------------------------------
void LeftPanel::RemoveFile(CFileZone* pFileToRemove)
{
	FileMap::iterator it = m_mFiles.find(pFileToRemove);
	if(it != m_mFiles.end()) {
		delete it.data();
		m_mFiles.erase(it);
	}
}

//-----------------------------------------------------------------------------------
void LeftPanel::UpdateFile(CFileZone* pFile)
{
	FileMap::Iterator it = m_mFiles.find(pFile);
	if(it != m_mFiles.end()) {
		it.data()->setText(0, pFile->GetName());
	}
}

//-----------------------------------------------------------------------------------
void LeftPanel::RemoveAllFiles()
{
	FileMap::Iterator it = m_mFiles.begin();
	for(; it != m_mFiles.end(); ++it) {
		delete it.data();
	}
	m_mFiles.clear();
}

//-----------------------------------------------------------------------------------
unsigned int LeftPanel::GetFileSelected(FileList& list)
{
	list.clear();
	unsigned int nSize = 0;
	FileMap::Iterator it = m_mFiles.begin();
	for(; it != m_mFiles.end(); ++it) {
		if(it.data()->IsSelected()) {
			list.push_back(it.key());
			nSize += it.key()->GetSize();
		}
	}
	return nSize;
}

//-----------------------------------------------------------------------------------
void LeftPanel::ToggleSelection(QListViewItem* pItem)
{
	if(pItem != NULL) {
		if(pItem->rtti() == CFileItem::RTTI()) {
			QListViewItem* pChild = pItem->firstChild();
			while(pChild != NULL) {
				((CFileZoneItem*)pChild)->SetSelected(!((CFileZoneItem*)pChild)->IsSelected());
				pChild = pChild->nextSibling();
			}
		}
		else {
			((CFileZoneItem*)pItem)->SetSelected(!((CFileZoneItem*)pItem)->IsSelected());
		}
	}
}

//-----------------------------------------------------------------------------------
void LeftPanel::OnSelect()
{
	ToggleSelection(m_pList->selectedItem());
}

//-----------------------------------------------------------------------------------
CFileZone* LeftPanel::GetFile(QListViewItem* pItem)
{
	if(pItem != NULL) {
		FileMap::Iterator it = m_mFiles.begin();
		for(; it != m_mFiles.end(); ++it) {
			if(it.data() == pItem) {
				return it.key();
			}
		}
	}
	return NULL;
}

//-----------------------------------------------------------------------------------
void LeftPanel::OnDoubleClick(QListViewItem* pItem)
{
	CFileZone* pFile = GetFile(pItem);
	if(pFile != NULL) {
		m_pRamManager->CreateEditor(pFile);
	}
}

//-----------------------------------------------------------------------------------
void LeftPanel::OnMenuRequested(QListViewItem* pItem, const QPoint& pos)
{
	if(pItem != NULL) {
		int i;
		for(i = 0; i < 12; i++) {
			if(m_pZones[i] == pItem) {
				break;
			}
		}
		QPopupMenu menu(this);
		if(i < 12) {
			menu.insertItem(tr("Crer un fichier"), 0);
			menu.insertItem(tr("Slectionner/Dslectionner les fichiers du dossier"), 1);
		}
		else {
			menu.insertItem(tr("Renommer"),  2);
			menu.insertItem(tr("Supprimer"), 3);
			menu.insertItem(tr("Slectionner/Dslectionner le fichier"), 1);
		}
		switch(menu.exec(pos))
		{
		case 0://Ajouter un fichier
			{
				QString s = CFileLoader::ChooseFileName(i, tr("Nouveau"), QString::null, this);
				if(!s.isEmpty()) {
					m_pRamManager->AddFile(s, i);
				}
			}
			break;
		case 1://Toggle Selection
			ToggleSelection(pItem);
			break;
		case 2://Renommer
			{
				CFileZone* pFile = GetFile(pItem);
				assert(pFile != NULL);
				QString s = CFileLoader::ChooseFileName(pFile->GetZone(), tr("Renommer"), pFile->GetName(), this);
				if(!s.isEmpty() && s != pFile->GetName()) {
					m_pRamManager->RenameFile(pFile, s);
					UpdateFile(pFile);
					pItem->parent()->sort();
				}
			}
			break;
		case 3://Supprimer
			m_pRamManager->RemoveFile(GetFile(pItem));
			break;
		}
	}
}
