// ListEditor.cpp: implementation of the CListEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "ListEditor.h"
#include "ListZone.h"
#include "ListTableItem.h"

#include <qtable.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qlayout.h>

#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CListEditor::CListEditor(QWidget* parent)
	: CEditor(parent)
	, m_pListZone(NULL)
{
	QWidget* pCentralWidget = new QWidget(this);
	QGridLayout* pLayout    = new QGridLayout(pCentralWidget, 2, 4, 5);

	m_pAddButton    = new QPushButton(pCentralWidget);
	m_pRemoveButton = new QPushButton(pCentralWidget);
	m_pInsertButton = new QPushButton(pCentralWidget);
	m_pTable        = new QTable(pCentralWidget);

	pLayout->addWidget(m_pAddButton,    1, 0);
	pLayout->addWidget(m_pInsertButton, 1, 1);
	pLayout->addWidget(m_pRemoveButton, 1, 2);
	pLayout->addMultiCellWidget(m_pTable, 0, 0, 0, 3);

	setCentralWidget(pCentralWidget);

	m_pTable->setNumCols(1);
	m_pTable->setReadOnly(FALSE);
	m_pTable->setSorting(FALSE);
	m_pTable->setRowMovingEnabled(FALSE);
	m_pTable->setColumnMovingEnabled(FALSE);
	m_pTable->verticalHeader()->setResizeEnabled(FALSE);
	m_pTable->setSelectionMode(QTable::SingleRow);

	connect(m_pAddButton,    SIGNAL(clicked()), SLOT(OnAddElement()));
	connect(m_pInsertButton, SIGNAL(clicked()), SLOT(OnInsertElement()));
	connect(m_pRemoveButton, SIGNAL(clicked()), SLOT(OnRemoveElement()));
	connect(m_pTable,        SIGNAL(valueChanged(int, int)), SLOT(OnValueModified(int, int)));

	DoInitWords();
	RefreshData();
}

//-----------------------------------------------------------------------------------
bool CListEditor::SetFileZone(CFileZone* pFile)
{
	CListZone* pListZone = dynamic_cast<CListZone*>(pFile);
	if(pListZone == NULL) {
		return false;
	}
	if(m_pListZone != NULL) {
		disconnect(m_pListZone);
	}
	m_pListZone = pListZone;
	connect(m_pListZone, SIGNAL(DataChanged()), SLOT(OnDataChanged()));
	connect(m_pListZone, SIGNAL(Renamed()),     SLOT(OnFileRenamed()));
	RefreshData();
	return true;
}

//-----------------------------------------------------------------------------------
void CListEditor::DoInitWords()
{
	CEditor::DoInitWords();
	m_pTable->horizontalHeader()->setLabel(0, tr("Valeurs"));
	m_pAddButton->setText(tr("Ajouter"));
	m_pRemoveButton->setText(tr("Supprimer"));
	m_pInsertButton->setText(tr("Insrer"));
}

//-----------------------------------------------------------------------------------
void CListEditor::RefreshData()
{
	if(m_pListZone != NULL) {
		UpdateCaption();
		m_pTable->setNumRows(m_pListZone->GetNbElements());
		for(int i = 0; i < m_pListZone->GetNbElements(); i++) {
			m_pTable->verticalHeader()->setLabel(i, m_pListZone->GetLabel(i));
			QTableItem* pItem = m_pTable->item(i, 0);
			if(pItem == NULL) {
				pItem = new CListTableItem(m_pTable, QTableItem::OnTyping, QString::number(m_pListZone->GetElement(i), 'g', 15));
				m_pTable->setItem(i, 0, pItem);
			}
			else {
				pItem->setText(QString::number(m_pListZone->GetElement(i), 'g', 15));
				m_pTable->updateCell(i, 0);
			}
		}
	}
	else {
		m_pTable->setNumRows(0);
	}
	m_pAddButton->setEnabled(m_pListZone != NULL && m_pListZone->IsRenameable());
	m_pRemoveButton->setEnabled(m_pListZone != NULL && m_pListZone->IsRenameable());
	m_pInsertButton->setEnabled(m_pListZone != NULL && m_pListZone->IsRenameable());
}

//-----------------------------------------------------------------------------------
void CListEditor::UpdateCaption()
{
	if(m_pListZone != NULL) {
		setCaption(tr("Editeur de liste") + " - " + m_pListZone->GetName());
	}
}

//-----------------------------------------------------------------------------------
void CListEditor::OnAddElement()
{
	assert(m_pListZone != NULL);
	m_pListZone->AddElement(0.0);
}

//-----------------------------------------------------------------------------------
void CListEditor::OnInsertElement()
{
	assert(m_pListZone != NULL);
	m_pListZone->AddElement(0.0, m_pTable->currentRow());
}

//-----------------------------------------------------------------------------------
void CListEditor::OnRemoveElement()
{
	assert(m_pListZone != NULL);
	m_pListZone->RemoveElement(m_pTable->currentRow());
}

//-----------------------------------------------------------------------------------
void CListEditor::OnValueModified(int nRow, int nCol)
{
	assert(m_pListZone != NULL);
	m_pListZone->SetElement(nRow, m_pTable->text(nRow, nCol).toDouble());
}

//-----------------------------------------------------------------------------------
void CListEditor::OnDataChanged()
{
	RefreshData();
}

//-----------------------------------------------------------------------------------
void CListEditor::OnFileRenamed()
{
	UpdateCaption();
}
