// ListZone.cpp: implementation of the CListZone class.
//
//////////////////////////////////////////////////////////////////////

#include "ListZone.h"
#include "CasioIO.h"

#include <qinputdialog.h>

#include <math.h>

#include <F100_Debug.h>

#define HEADER_SIZE 14
#define SIZE_POS     8

unsigned char header[] = {0x00, 0x75, 0x1F, 0x7C, 0x00, 0x00, 0x5C, 0xB5, 0x01, 0x00, 0xF4, 0x02, 0x5A, 0x7A};

const char* data_type[] = {"LT", "WD", "GFA"};
int fixed_element[]     = {0, 11, 2};

//-----------------------------------------------------------------------------------
CListZone::CListZone()
	: CFileZone(5)
	, m_bModified(FALSE)
	, m_nDataType(DT_LIST)
{

}

//-----------------------------------------------------------------------------------
unsigned int CListZone::CalculateSize(int nElements) const
{
	unsigned int nSize = 0;
	if(m_nDataType == DT_LIST) {
		nSize = HEADER_SIZE;
	}
	return nSize + nElements * VAR_SIZE - 1;
}

//-----------------------------------------------------------------------------------
bool CListZone::ReadDataFrom(QTextStream& stream, unsigned int nCapacity)
{	
	int nElements;
	if(fixed_element[m_nDataType] != 0) {
		nElements = fixed_element[m_nDataType];
	}
	else {
		stream.readLine();
		QString sElement = stream.readLine();
		if(sElement.section(':', 0, 0) != "Elements") {
			return false;
		}
		nElements = sElement.section(':', 1, 1).toInt();
	}
	unsigned int nSize = CalculateSize(nElements);
	if(nCapacity == 0 || nElements == 0) {
		SetData(NULL, 0, TRUE);
		return true;
	}
	m_lElements.clear();
	for(int i = 0; i < nElements; i++) {
		QString s = stream.readLine();
		if(m_nDataType != DT_LIST) {
			s = s.section(' ', 1, 1);
		}
		else {
			s = s.section(':', 1, 1);
			s = s.section(' ', 2, 2);
		}
		double n = s.section("\\EE", 0, 0).toDouble();
		int e = s.section("\\EE", 1, 1).toInt();
		
		m_lElements.push_back(n * pow(10, e));
	}
	OnDataChanged();
	return true;
}

//-----------------------------------------------------------------------------------
void CListZone::WriteData(QTextStream& stream)
{
	if(m_nDataType == DT_LIST) {
		stream << "File Name:\nElements:" << m_lElements.size() << '\n';
	}
	List::Iterator it = m_lElements.begin();
	for(int i = 0; it != m_lElements.end(); ++i) {
		if(m_nDataType != DT_LIST) {
			stream << GetLabel(i);
		}
		else {
			stream << "Value : " << i;
		}
		stream << " " << CCasioIO::ConvertDoubleToCasioString(*it) << '\n';
		++it;
	}
}

//-----------------------------------------------------------------------------------
QString CListZone::GetDataType() const
{
	return data_type[m_nDataType];
}

//-----------------------------------------------------------------------------------
void CListZone::SetName(const QString& sName)
{
	CFileZone::SetName(sName);
	if(GetName() == "VWIN" || GetName() == "VWINR") {
		m_nDataType = DT_WD;
	}
	else if(GetName() == "FACTOR") {
		m_nDataType = DT_GFA;
	}
}

//-----------------------------------------------------------------------------------
void CListZone::SetData(void* pData, unsigned int nSize, bool bEmitDataChanged)
{
	m_lElements.clear();
	if(pData == NULL || nSize == 0) {
		if(fixed_element[m_nDataType] != 0) {
			for(int i = 0; i < fixed_element[m_nDataType]; i++) {
				m_lElements.push_back(0.0);
			}
			m_bModified = TRUE;
		}
		CFileZone::SetData(pData, nSize, bEmitDataChanged);
		return;
	}
	int nElements;
	int nPos = 0;
	if(fixed_element[m_nDataType] != 0) {
		nElements = fixed_element[m_nDataType];
	}
	else {
		nElements = ((unsigned char*)pData)[SIZE_POS];
		nPos = HEADER_SIZE;
	}

	for(int i = 0; i < nElements; i++) {
		unsigned char var[VAR_SIZE];
		if(i == 0) {
			memcpy(var + 1, (char*)pData + nPos, sizeof(var) - 1);
			nPos += sizeof(var) - 1;
		}
		else {
			memcpy(var, (char*)pData + nPos, sizeof(var));
			nPos += sizeof(var);
		}
		m_lElements.push_back(CCasioIO::ConvertCasioDataToDouble(var));
	}
	m_bModified = FALSE;
	CFileZone::SetData(pData, nSize, bEmitDataChanged);
}

//-----------------------------------------------------------------------------------
int CListZone::GetNbElements() const
{
	return m_lElements.size();
}

//-----------------------------------------------------------------------------------
double CListZone::GetElement(int nIndex) const
{
	return m_lElements[nIndex];
}

//-----------------------------------------------------------------------------------
void CListZone::AddElement(double dValue, int nIndex)
{
	if(m_lElements.size() < 255) {
		if(nIndex < 0) {
			m_lElements.push_back(dValue);
		}
		else {
			if(nIndex >= GetNbElements()) {
				return;
			}
			m_lElements.insert(m_lElements.at(nIndex), dValue);
		}
		OnDataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CListZone::RemoveElement(int nIndex)
{
	if(nIndex >= 0 && nIndex < GetNbElements()) {
		m_lElements.remove(m_lElements.at(nIndex));
		OnDataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CListZone::SetElement(int nIndex, double dValue)
{
	if(nIndex >= 0 && nIndex < GetNbElements() && m_lElements[nIndex] != dValue) {
		m_lElements[nIndex] = dValue;
		OnDataChanged();
	}
}

//-----------------------------------------------------------------------------------
void CListZone::FlushData()
{
	if(m_bModified) {
		int nElements = m_lElements.size();
		unsigned int nSize = CalculateSize(nElements);
		char* pData = new char[nSize];
		char var[VAR_SIZE];
		if(pData != NULL) {
			int pos = 0;
			if(fixed_element[m_nDataType] == 0) {
				memcpy(pData, header, HEADER_SIZE);
				pData[SIZE_POS] = (unsigned char)nElements;
				pos = HEADER_SIZE;
			}
			List::Iterator it = m_lElements.begin();
			for(int i = 0; i < nElements; i++) {
				CCasioIO::WriteCasioNumberFormat(var, *it);
				if(i == 0) {
					memcpy(pData + pos, var + 1, sizeof(var) - 1);
					pos += sizeof(var) - 1;
				}
				else {
					memcpy(pData + pos, var, sizeof(var));
					pos += sizeof(var);
				}
				++it;
			}
			CFileZone::SetData(pData, nSize, FALSE);
			m_bModified = FALSE;
		}
	}
}

//-----------------------------------------------------------------------------------
void CListZone::OnDataChanged()
{
	m_bModified = TRUE;
	emit DataChanged();
}

//-----------------------------------------------------------------------------------
bool CListZone::IsRenameable() const
{
	return m_nDataType == DT_LIST;
}

//-----------------------------------------------------------------------------------
QString CListZone::GetLabel(unsigned int nIndex)
{
	if(nIndex < m_lElements.size()) {
		static const char* gfa[] = {"Xfact", "Yfact"};
		static const char* wd[]  = {"Xmin", "Xmax", "Xscale", "Xdot", "Ymin", "Ymax ", "Yscale", "Tthetamin", "Tthetamax", "Tthetaptch", "XdotDual"};
		switch(m_nDataType)
		{
		case DT_LIST:
			return QString::number(nIndex + 1);
		case DT_WD:
			return wd[nIndex];
		case DT_GFA:
			return gfa[nIndex];
		}
	}
	return QString::null;
}

//-----------------------------------------------------------------------------------
void CListZone::AddSpecialLists(QStringList& lList)
{
	lList.push_back("VWIN");
	lList.push_back("VWINR");
	lList.push_back("FACTOR");
}

//-----------------------------------------------------------------------------------
int CListZone::FindIndex(int nFlags, const QString& sText)
{
	if(nFlags&2){
		int nShift = (nFlags&1)?120:0;
		if(sText == "VWIN") {
			return 0 + nShift;
		}
		else if(sText == "VWINR") {
			return 1 + nShift;
		}
		else if(sText == "FACTOR") {
			return 2 + nShift;
		}
	}
	if(nFlags&1) {
		int nFile = sText.section("LIST", 0, 0).toInt() - 1;
		int nList = sText.section("LIST", 1, 1).toInt() - 1;
		return nFile * 20 + nList;
	}
	return 0;
}

//-----------------------------------------------------------------------------------
QString CListZone::GetText(int nFlags, int nIndex)
{
	int i = nIndex;
	if(nFlags&1) {
		if(nIndex < 120) {
			return QString("%1LIST%2").arg(1 + nIndex / 20).arg(1 + nIndex % 20);
		}
		i -= 120;
	}
	if(nFlags&2){
		if(i == 0) {
			return "VWIN";
		}
		else if(i == 1) {
			return "VWINR";
		}
		return "FACTOR";
	}
	return QString::null;
}

//-----------------------------------------------------------------------------------
QString CListZone::ChooseFileName(const QString& sCaption, const QString& sDefaultText, QWidget* parent)
{
	bool bOk;
	QStringList lList;
	int nFlags = 0;
	if(sDefaultText != "VWIN" && sDefaultText != "VWINR" && sDefaultText != "FACTOR") {
		for(int i = 1; i <= 6; i++) {
			for(int j = 1; j <= 20; j++) {
				lList.push_back(CFileZone::tr("Fichier n%1, Liste %2").arg(i).arg(j));
			}
		}
		nFlags |= 1;
		if(sDefaultText.isEmpty()) {
			AddSpecialLists(lList);
		}
		nFlags |= 2;
	}
	else {
		AddSpecialLists(lList);
		nFlags |= 2;
	}
	int nCurrent = FindIndex(nFlags, sDefaultText);
	QString s = QInputDialog::getItem(sCaption, CFileZone::tr("Choisissez une liste possible"), lList, nCurrent, FALSE, &bOk, parent);
	if(bOk) {
		return GetText(nFlags, lList.findIndex(s));
	}
	return QString::null;
}
