// MatrixEditor.cpp: implementation of the CMatrixEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "MatrixEditor.h"
#include "MatrixZone.h"
#include "ListTableItem.h"
#include "ListZone.h"

#include <qtable.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <F100_Debug.h>

//-----------------------------------------------------------------------------------
CMatrixEditor::CMatrixEditor(QWidget* parent)
	: CEditor(parent)
	, m_pMatrixZone(NULL)
	, m_bLockRefresh(FALSE)
{
	QWidget* pCentralWidget = new QWidget(this);
	QGridLayout* pLayout    = new QGridLayout(pCentralWidget, 3, 4, 5);

	m_pTable           = new QTable(pCentralWidget);
	m_pAddRowButton	   = new QPushButton(pCentralWidget);
	m_pAddColButton	   = new QPushButton(pCentralWidget);
	m_pInsertRowButton = new QPushButton(pCentralWidget);
	m_pInsertColButton = new QPushButton(pCentralWidget);
	m_pRemoveRowButton = new QPushButton(pCentralWidget);
	m_pRemoveColButton = new QPushButton(pCentralWidget);

	pLayout->addMultiCellWidget(m_pTable,  0, 0, 0, 3);
	pLayout->addWidget(m_pAddRowButton,    1, 0);
	pLayout->addWidget(m_pInsertRowButton, 1, 1);
	pLayout->addWidget(m_pRemoveRowButton, 1, 2);
	pLayout->addWidget(m_pAddColButton,    2, 0);
	pLayout->addWidget(m_pInsertColButton, 2, 1);
	pLayout->addWidget(m_pRemoveColButton, 2, 2);

	setCentralWidget(pCentralWidget);

	m_pTable->setNumCols(1);
	m_pTable->setReadOnly(FALSE);
	m_pTable->setSorting(FALSE);
	m_pTable->setRowMovingEnabled(FALSE);
	m_pTable->setColumnMovingEnabled(FALSE);
	m_pTable->verticalHeader()->setResizeEnabled(FALSE);
	m_pTable->setSelectionMode(QTable::SingleRow);

	connect(m_pTable,           SIGNAL(valueChanged(int, int)), SLOT(OnValueModified(int, int)));
	connect(m_pAddRowButton,    SIGNAL(clicked()), SLOT(OnAddRow()));
	connect(m_pAddColButton,    SIGNAL(clicked()), SLOT(OnAddCol()));
	connect(m_pInsertRowButton, SIGNAL(clicked()), SLOT(OnInsertRow()));
	connect(m_pInsertColButton, SIGNAL(clicked()), SLOT(OnInsertCol()));
	connect(m_pRemoveRowButton, SIGNAL(clicked()), SLOT(OnRemoveRow()));
	connect(m_pRemoveColButton, SIGNAL(clicked()), SLOT(OnRemoveCol()));

	DoInitWords();
	RefreshData();
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::DoInitWords()
{
	CEditor::DoInitWords();
	m_pAddRowButton->setText(tr("Ajouter une ligne"));
	m_pAddColButton->setText(tr("Ajouter une colonne"));
	m_pInsertRowButton->setText(tr("Insrer une ligne"));
	m_pInsertColButton->setText(tr("Insrer une colonne"));
	m_pRemoveRowButton->setText(tr("Supprimer une ligne"));
	m_pRemoveColButton->setText(tr("Supprimer une colonne"));
}

//-----------------------------------------------------------------------------------
bool CMatrixEditor::SetFileZone(CFileZone* pFile)
{
	CMatrixZone* pMatrixZone = dynamic_cast<CMatrixZone*>(pFile);
	if(pMatrixZone == NULL) {
		return false;
	}
	if(m_pMatrixZone != NULL) {
		disconnect(m_pMatrixZone);
	}
	m_pMatrixZone = pMatrixZone;
	connect(m_pMatrixZone, SIGNAL(DataChanged()), SLOT(OnDataChanged()));
	connect(m_pMatrixZone, SIGNAL(Renamed()),     SLOT(OnFileRenamed()));
	RefreshData();
	return true;
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::RefreshData()
{
	if(m_bLockRefresh) {
		return;
	}
	if(m_pMatrixZone != NULL) {
		UpdateCaption();
		m_pTable->setNumRows(m_pMatrixZone->GetNbRows());
		m_pTable->setNumCols(m_pMatrixZone->GetNbCols());
		for(int r = 0; r < m_pMatrixZone->GetNbRows(); r++) {
			for(int c = 0; c < m_pMatrixZone->GetNbCols(); c++) {
				QTableItem* pItem = m_pTable->item(r, c);
				if(pItem == NULL) {
					pItem = new CListTableItem(m_pTable, QTableItem::OnTyping, QString::number(m_pMatrixZone->GetElement(r, c), 'g', 15));
					m_pTable->setItem(r, c, pItem);
				}
				else {
					pItem->setText(QString::number(m_pMatrixZone->GetElement(r, c), 'g', 15));
					m_pTable->updateCell(r, c);
				}
			}
		}
	}
	else {
		m_pTable->setNumRows(0);
		m_pTable->setNumCols(0);
	}
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnValueModified(int nRow, int nCol)
{
	assert(m_pMatrixZone != NULL);
	m_bLockRefresh = TRUE;
	m_pMatrixZone->SetElement(nRow, nCol, m_pTable->text(nRow, nCol).toDouble());
	m_bLockRefresh = FALSE;
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnDataChanged()
{
	RefreshData();
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnFileRenamed()
{
	UpdateCaption();
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::UpdateCaption()
{
	if(m_pMatrixZone != NULL) {
		setCaption(tr("Editeur de matrice") + " - " + m_pMatrixZone->GetName());
	}
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnAddRow()
{
	assert(m_pMatrixZone != NULL);
	m_pMatrixZone->AddRow();
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnInsertRow()
{
	assert(m_pMatrixZone != NULL);
	m_pMatrixZone->AddRow(m_pTable->currentRow());
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnAddCol()
{
	assert(m_pMatrixZone != NULL);
	m_pMatrixZone->AddCol();
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnInsertCol()
{
	assert(m_pMatrixZone != NULL);
	m_pMatrixZone->AddCol(m_pTable->currentColumn());
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnRemoveRow()
{
	assert(m_pMatrixZone != NULL);
	m_pMatrixZone->RemoveRow(m_pTable->currentRow());
}

//-----------------------------------------------------------------------------------
void CMatrixEditor::OnRemoveCol()
{
	assert(m_pMatrixZone != NULL);
	m_pMatrixZone->RemoveCol(m_pTable->currentColumn());
}
