// RamManagerOption.cpp: implementation of the CRamManagerOption class.
//
//////////////////////////////////////////////////////////////////////

#include "RamManagerOption.h"
#include "IOptionListener.h"
#include "SyntaxHighligterFactory.h"
#include "SyntaxSettingsWidget.h"
#include "SyntaxProperties.h"

#include <qhgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qfontdialog.h>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qcolordialog.h>

#include <Label.h>

#define REG_FONT      "Editor/Font"
#define REG_FORECOLOR "Editor/ForeColor"
#define REG_BACKCOLOR "Editor/BackColor"
#define REG_AUTOFORE  "Editor/AutoForeColor"
#define REG_AUTOBACK  "Editor/AutoBackColor"

CRamManagerOption* CRamManagerOption::m_pInstance = NULL;

//-----------------------------------------------------------------------------------
CRamManagerOption::CRamManagerOption(IMainWindow* pMainWindow)
	: CSettingsWidget(pMainWindow, PLUGIN_NAME)
	, m_qVBoxLayout(this, 5)
{
	m_pInstance = this;
	m_pEditorGroup = new QHGroupBox(this);
	m_pEditorGroup->setColumns(3);

	m_pFontName     = new QLabel(m_pEditorGroup);
	m_pForeCheckBox = new QCheckBox(m_pEditorGroup);
	m_pBackCheckBox = new QCheckBox(m_pEditorGroup);
	m_pFontButton   = new QPushButton(m_pEditorGroup);
	m_pForeButton   = new CLabel(m_pEditorGroup);
	m_pBackButton   = new CLabel(m_pEditorGroup);

	SyntaxPropertiesList::const_iterator it  = CSyntaxHighligterFactory::GetInstance()->GetSyntaxPropertiesList().begin();
	SyntaxPropertiesList::const_iterator end = CSyntaxHighligterFactory::GetInstance()->GetSyntaxPropertiesList().end();
	for(; it != end; ++it) {
		CSyntaxProperties* pProperties = *it;
		if(pProperties->GetProperties().size() > 0) {
			AddChild(new CSyntaxSettingsWidget(this, pProperties));
		}
	}

	m_qVBoxLayout.addWidget(m_pEditorGroup, 0, AlignVCenter);
	
	DoInitWords();
	DoRefresh();

	connect(m_pFontButton, SIGNAL(clicked()), SLOT(OnModifyFont()));
	connect(m_pForeCheckBox, SIGNAL(toggled(bool)), m_pForeButton, SLOT(setDisabled(bool)));
	connect(m_pBackCheckBox, SIGNAL(toggled(bool)), m_pBackButton, SLOT(setDisabled(bool)));
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::DoInitWords()
{
	setCaption(tr("Gestionnaire de RAM"));
	m_pEditorGroup->setTitle(tr("Editeur de texte"));
	m_pFontButton->setText(tr("Modifier la police"));
	m_pForeButton->setText(tr("Couleur du texte"));
	m_pBackButton->setText(tr("Couleur de fond"));
	m_pForeCheckBox->setText(tr("Couleur du texte par dfaut"));
	m_pBackCheckBox->setText(tr("Couleur du fond par dfaut"));
	RefreshFontEditor();
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::DoRefresh()
{
	QString s;
	ReadSettings(REG_FONT, s);
	if(!s.isEmpty()) {
		m_fEditor.fromString(s);
	}
	m_fTmp = m_fEditor;

	ReadSettings(REG_FORECOLOR, s, colorGroup().foreground().name());
	m_fore = QColor(s);
	ReadSettings(REG_BACKCOLOR, s, colorGroup().base().name());
	m_back = QColor(s);
	RefreshFontEditor();

	ReadSettings(REG_AUTOFORE, m_bAutoForeColor, 1);
	m_pForeCheckBox->setChecked(m_bAutoForeColor);
	m_pForeButton->setEnabled(!m_bAutoForeColor);

	ReadSettings(REG_AUTOBACK, m_bAutoBackColor, 1);
	m_pBackCheckBox->setChecked(m_bAutoBackColor);
	m_pBackButton->setEnabled(!m_bAutoBackColor);

	m_pForeButton->SetColorChooser(m_fore);
	m_pBackButton->SetColorChooser(m_back);
}

//-----------------------------------------------------------------------------------
int CRamManagerOption::DoSave()
{
	m_fEditor = m_fTmp;
	m_fore = m_pForeButton->GetColor();
	m_back = m_pBackButton->GetColor();

	m_bAutoForeColor = m_pForeCheckBox->isChecked();
	m_bAutoBackColor = m_pBackCheckBox->isChecked();

	WriteSettings(REG_FONT,      m_fEditor.toString());
	WriteSettings(REG_FORECOLOR, m_fore.name());
	WriteSettings(REG_BACKCOLOR, m_back.name());
	WriteSettings(REG_AUTOFORE,  m_bAutoForeColor);
	WriteSettings(REG_AUTOBACK,  m_bAutoBackColor);

	SendOptionChanged();
	return 1;
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::SendOptionChanged()
{
	ListenerList::iterator it = m_lListeners.begin();
	for(; it != m_lListeners.end(); it++) {
		(*it)->OnOptionChanged();
	}
}

//-----------------------------------------------------------------------------------
QString CRamManagerOption::GetDescription() const
{
	return tr("Gestionnaire de RAM");
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::AddListener(IOptionListener* pListener)
{
	if(!m_lListeners.contains(pListener)) {
		m_lListeners.push_back(pListener);
	}
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::RemoveListener(IOptionListener* pListener)
{
	m_lListeners.remove(pListener);
}

//-----------------------------------------------------------------------------------
const QFont& CRamManagerOption::GetFontEditor() const
{
	return m_fEditor;
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::OnModifyFont()
{
	bool ok;
	QFont font = QFontDialog::getFont(&ok, m_fTmp, this);
	if(ok) {
		m_fTmp = font;
	}
	RefreshFontEditor();
}

//-----------------------------------------------------------------------------------
void CRamManagerOption::RefreshFontEditor()
{
	m_pFontName->setText(GetInformations(m_fTmp));
}

//-----------------------------------------------------------------------------------
const QColor& CRamManagerOption::GetForeColor() const
{
	if(m_bAutoForeColor) {
		return QApplication::palette().color(QPalette::Active, QColorGroup::Text);
	}
	return m_fore;
}

//-----------------------------------------------------------------------------------
const QColor& CRamManagerOption::GetBackColor() const
{
	if(m_bAutoBackColor) {
		return QApplication::palette().color(QPalette::Active, QColorGroup::Base);
	}
	return m_back;
}

//-----------------------------------------------------------------------------------
CRamManagerOption* CRamManagerOption::GetInstance()
{
	return m_pInstance;
}

//-----------------------------------------------------------------------------------
QString CRamManagerOption::GetInformations(const QFont& font)
{
	return tr("Police : %1, Taille : %2").arg(font.rawName()).arg(font.pointSize());
}
