// SyntaxSettingsWidget.cpp: implementation of the CSyntaxSettingsWidget class.
//
//////////////////////////////////////////////////////////////////////

#include "SyntaxSettingsWidget.h"
#include "SyntaxProperties.h"
#include "SyntaxProperty.h"
#include "RamManagerOption.h"

#include <qlayout.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qheader.h>
#include <qpushbutton.h>
#include <qfontdialog.h>

#include <assert.h>

#include <Label.h>

#define REG_BOLD      "Bold"
#define REG_ITALIC    "Italic"
#define REG_UNDERLINE "Underline"
#define REG_COLOR     "Color"
#define REG_FONT      "Font"

#define INVALID_COLOR_TEXT "Invalid"

//-----------------------------------------------------------------------------------
CSyntaxSettingsWidget::CSyntaxSettingsWidget(CSettingTabWidget* parent, CSyntaxProperties* pProperties)
	: CSettingTabWidget(parent->GetMainWindow(), parent->GetKey() + "Syntax/" + pProperties->GetName() + "/", TRUE, FALSE)
	, m_pProperties(pProperties)
{
	m_pTmpProperties = m_pProperties->Clone();

	QGridLayout* pLayout = new QGridLayout(this, 7, 3, 5);

	m_pList        = new QListView(this);
	m_pBold        = new QCheckBox(this);
	m_pUnderline   = new QCheckBox(this);
	m_pItalic      = new QCheckBox(this);
	m_pCustomColor = new QCheckBox(this);
	m_pColor       = new CLabel(this);
	m_pCustomFont  = new QCheckBox(this);
	m_pFont        = new QPushButton(this);

	pLayout->addMultiCellWidget(m_pList,        0, 4, 0, 0);
	pLayout->addMultiCellWidget(m_pCustomColor, 2, 2, 1, 2);
	pLayout->addMultiCellWidget(m_pColor,       3, 3, 1, 2);
	pLayout->addWidget(m_pBold,       0, 1);
	pLayout->addWidget(m_pItalic,     0, 2);
	pLayout->addWidget(m_pUnderline,  1, 1);
	pLayout->addWidget(m_pFont,       6, 0, AlignLeft);
	pLayout->addMultiCellWidget(m_pCustomFont, 5, 5, 0, 1);

	pLayout->setRowStretch(4, 10);
	pLayout->setColStretch(0, 10);

	m_pList->header()->hide();
	m_pList->addColumn(QString::null);
	m_pList->setResizeMode(QListView::LastColumn);

	connect(m_pList, SIGNAL(selectionChanged(QListViewItem*)), SLOT(OnSelectionChanged(QListViewItem*)));
	connect(m_pCustomColor, SIGNAL(toggled(bool)), m_pColor, SLOT(setEnabled(bool)));

	connect(m_pBold,        SIGNAL(released()), this, SLOT(OnSettingsChanged()));
	connect(m_pUnderline,   SIGNAL(released()), this, SLOT(OnSettingsChanged()));
	connect(m_pItalic,      SIGNAL(released()), this, SLOT(OnSettingsChanged()));
	connect(m_pCustomColor, SIGNAL(released()), this, SLOT(OnSettingsChanged()));
	connect(m_pColor,       SIGNAL(colorChanged(const QColor&)), this, SLOT(OnSettingsChanged()));
	connect(m_pCustomFont,  SIGNAL(released()), this, SLOT(OnChangeFont()));
	connect(m_pFont,        SIGNAL(released()), this, SLOT(OnChangeFont()));

	DoInitWords();
	DoRefresh();
}

//-----------------------------------------------------------------------------------
CSyntaxSettingsWidget::~CSyntaxSettingsWidget()
{
	delete m_pTmpProperties;
}

//-----------------------------------------------------------------------------------
QString CSyntaxSettingsWidget::GetDescription() const
{
	return QString::null;
}

//-----------------------------------------------------------------------------------
void CSyntaxSettingsWidget::DoInitWords()
{
	setCaption(m_pProperties->GetName());
	m_pBold->setText(tr("Gras"));
	m_pItalic->setText(tr("Italique"));
	m_pUnderline->setText(tr("Soulign"));
	m_pCustomColor->setText(tr("Spficier une couleur"));
	m_pFont->setText(tr("Modifier la police"));
	m_pList->clear();
	PropertyList::iterator it  = GetProperties()->GetProperties().begin();
	PropertyList::iterator end = GetProperties()->GetProperties().end();
	for(; it != end; it++) {
		new QListViewItem(m_pList, (*it)->GetName(), (*it)->GetId());
	}
	RefreshCustomFont();
}

//-----------------------------------------------------------------------------------
int CSyntaxSettingsWidget::DoSave()
{
	assert(m_pTmpProperties != NULL);
	if(*m_pTmpProperties != *m_pProperties) {
		*m_pProperties = *m_pTmpProperties;
		
		PropertyList::iterator it  = m_pProperties->GetProperties().begin();
		PropertyList::iterator end = m_pProperties->GetProperties().end();
		if(m_pProperties->GetCustomFont() != NULL) {
			WriteSettings(REG_FONT, m_pProperties->GetCustomFont()->toString());
		}
		else {
			WriteSettings(REG_FONT, QString::null);
		}
		for(; it != end; it++) {
			const CSyntaxProperty* pProperty = *it;
			QString sPath = pProperty->GetId() + "/%1";
			WriteSettings(sPath.arg(REG_BOLD),      pProperty->IsBold());
			WriteSettings(sPath.arg(REG_ITALIC),    pProperty->IsItalic());
			WriteSettings(sPath.arg(REG_UNDERLINE), pProperty->IsUnderline());
			if(pProperty->GetColor().isValid()) {
				WriteSettings(sPath.arg(REG_COLOR), pProperty->GetColor().name());
			}
			else {
				WriteSettings(sPath.arg(REG_COLOR), INVALID_COLOR_TEXT);
			}
		}
		CRamManagerOption::GetInstance()->SendOptionChanged();
	}
	return 1;
}

//-----------------------------------------------------------------------------------
void CSyntaxSettingsWidget::DoRefresh()
{
	assert(m_pTmpProperties != NULL);
	m_pTmpProperties->Reset();
	QString s;
	PropertyList::iterator it  = m_pTmpProperties->GetProperties().begin();
	PropertyList::iterator end = m_pTmpProperties->GetProperties().end();
	if(ReadSettings(REG_FONT, s)) {
		if(!s.isEmpty()) {
			QFont f;
			if(f.fromString(s)) {
				m_pTmpProperties->SetCustomFont(&f);
			}
		}
		else {
			m_pTmpProperties->SetCustomFont(NULL);
		}
	}
	for(; it != end; it++) {
		CSyntaxProperty* pProperty = *it;
		QString sColor;
		int nBold, nItalic, nUnderLine;
		QString sPath = pProperty->GetId() + "/%1";
		ReadSettings(sPath.arg(REG_BOLD),      nBold,      pProperty->IsBold());
		ReadSettings(sPath.arg(REG_ITALIC),    nItalic,    pProperty->IsItalic());
		ReadSettings(sPath.arg(REG_UNDERLINE), nUnderLine, pProperty->IsUnderline());
		ReadSettings(sPath.arg(REG_COLOR),     sColor,     pProperty->GetColor().isValid()?pProperty->GetColor().name():INVALID_COLOR_TEXT);
		pProperty->SetBold(nBold);
		pProperty->SetItalic(nItalic);
		pProperty->SetUnderline(nUnderLine);
		if(sColor == INVALID_COLOR_TEXT) {
			pProperty->SetColor(QColor());
		}
		else {
			pProperty->SetColor(sColor);
		}
	}

	if(*m_pTmpProperties != *m_pProperties) {
		*m_pProperties = *m_pTmpProperties;
		CRamManagerOption::GetInstance()->SendOptionChanged();		
	}

	m_pList->setSelected(m_pList->firstChild(), true);
	RefreshCustomFont();
	OnSelectionChanged(m_pList->selectedItem());
}

//-----------------------------------------------------------------------------------
CSyntaxProperties* CSyntaxSettingsWidget::GetProperties() const
{
	return m_pProperties;
}

//-----------------------------------------------------------------------------------
void CSyntaxSettingsWidget::OnSelectionChanged(QListViewItem* pItem)
{
	assert(m_pTmpProperties != NULL);
	if(pItem != NULL) {
		CSyntaxProperty* pProperty = m_pTmpProperties->GetProperty(pItem->text(1));
		if(pProperty != NULL) {
			m_pBold->setChecked(pProperty->IsBold());
			m_pUnderline->setChecked(pProperty->IsUnderline());
			m_pItalic->setChecked(pProperty->IsItalic());
			m_pCustomColor->setChecked(pProperty->GetColor().isValid());
			m_pColor->setEnabled(m_pCustomColor->isChecked());
			m_pColor->SetColorChooser(pProperty->GetColor());
		}
	}
}

//-----------------------------------------------------------------------------------
void CSyntaxSettingsWidget::OnSettingsChanged()
{
	QListViewItem* pItem = m_pList->selectedItem();
	assert(m_pTmpProperties != NULL && pItem != NULL);

	CSyntaxProperty* pProperty = m_pTmpProperties->GetProperty(pItem->text(1));
	if(pProperty != NULL) {
		pProperty->SetBold(m_pBold->isChecked());
		pProperty->SetItalic(m_pItalic->isChecked());
		pProperty->SetUnderline(m_pUnderline->isChecked());
		if(m_pCustomColor->isChecked()) {
			pProperty->SetColor(m_pColor->GetColor());
		}
		else {
			pProperty->SetColor(QColor());
		}
	}
}

//-----------------------------------------------------------------------------------
void CSyntaxSettingsWidget::OnChangeFont()
{
	if(m_pCustomFont->isChecked()) {
		bool ok;
		QFont f;
		if(m_pTmpProperties->GetCustomFont() == NULL) {
			f = CRamManagerOption::GetInstance()->GetFontEditor();
		}
		else {
			f = *m_pTmpProperties->GetCustomFont();
		}
		
		f = QFontDialog::getFont(&ok, f, this);
		if(ok) {
			m_pTmpProperties->SetCustomFont(&f);
		}
	}
	else {
		m_pTmpProperties->SetCustomFont(NULL);
	}
	RefreshCustomFont();
}
//-----------------------------------------------------------------------------------
void CSyntaxSettingsWidget::RefreshCustomFont()
{
	if(m_pTmpProperties != NULL) {
		QString s(tr("Police personnalise"));
		const QFont* pFont = m_pTmpProperties->GetCustomFont();
		if(pFont != NULL) {
			m_pCustomFont->setChecked(TRUE);
			m_pFont->setEnabled(TRUE);
			m_pCustomFont->setText(s + " (" + CRamManagerOption::GetInformations(*pFont) + ")");
		}
		else {
			m_pFont->setEnabled(FALSE);
			m_pCustomFont->setText(s);
			m_pCustomFont->setChecked(FALSE);
		}
	}
}
