// TextEditor.cpp: implementation of the CTextEditor class.
//
//////////////////////////////////////////////////////////////////////

#include "TextEditor.h"
#include "SyntaxProperties.h"
#include "SyntaxHighlighter.h"

#include <qheader.h>
#include <qpopupmenu.h>
#include <qlistview.h>
#include <private/qrichtext_p.h>

//-----------------------------------------------------------------------------------
CTextEditor::CTextEditor(QWidget* parent)
	: QTextEdit(parent)
{
	m_pMenu     = new QPopupMenu(this);
	m_pKeywords = new QListView(m_pMenu);

	m_pMenu->insertItem(m_pKeywords);

	m_pKeywords->header()->hide();
	m_pKeywords->addColumn(QString::null);
	m_pKeywords->setColumnWidthMode(0, QListView::Maximum);

	connect(m_pKeywords, SIGNAL(doubleClicked(QListViewItem*, const QPoint&, int)), SLOT(OnInsertItem(QListViewItem*)));
	connect(m_pKeywords, SIGNAL(returnPressed(QListViewItem*)), SLOT(OnInsertItem(QListViewItem*)));
}

//-----------------------------------------------------------------------------------
void CTextEditor::keyPressEvent(QKeyEvent* e)
{
	if(e->state() == ControlButton && e->key() == Key_Space) {
		if(m_pKeywords->childCount() > 0) {
			QPoint p (textCursor()->x(), textCursor()->y() + textCursor()->paragraph()->rect().y() + textCursor()->paragraph()->rect().height());
			SelectKeywords(p);
		}
	}
	else {
		QTextEdit::keyPressEvent(e);
	}
}

//-----------------------------------------------------------------------------------
void CTextEditor::SelectItem(QListViewItem* pItem)
{
	m_pKeywords->setSelected(pItem, TRUE);
	m_pKeywords->ensureItemVisible(pItem);
}

//-----------------------------------------------------------------------------------
void CTextEditor::keyReleaseEvent(QKeyEvent* e)
{
	if(!(e->state() == ControlButton && e->key() == Key_Space)) {
		QTextEdit::keyReleaseEvent(e);
	}
}

//-----------------------------------------------------------------------------------
bool CTextEditor::IsStart(const QString& s, int index) const
{
	QChar c = s[index];
	return c == ' ' || c == '\t';
}

//-----------------------------------------------------------------------------------
int CTextEditor::Match(const QString& sItem, const QString& sLine) const
{
	bool bNotNeedSpace = sItem[0] == ' ';
	uint i = QMIN(sItem.length(), sLine.length());
	for(; i > 1; i--)  {
		if(bNotNeedSpace || i == sLine.length() || IsStart(sLine, sLine.length() - i - 1)) {
			QString s = sLine.mid(sLine.length() - i, i);
			if(sItem.find(s) == 0) {
				return i;
			}
		}
	}
	return 0;
}

//-----------------------------------------------------------------------------------
void CTextEditor::SelectKeywords(const QPoint& p)
{
	int para, index;
	m_nBeginIndex = -1;
	getCursorPosition(&para, &index);

	QString sLine = text(para);
	sLine.truncate(index);
	QString sLCLine = sLine.lower();

	QListViewItem* pItem  = m_pKeywords->firstChild();
	QListViewItem* pFound = NULL;
	bool bLower = FALSE;
	int nSize = 0;
	
	if(!sLine.isEmpty()) {
		while(pItem != NULL) {
			QString sItem = pItem->text(0);
			int nMatch = Match(sItem, sLine);
			if(nMatch > nSize || (bLower && nMatch == nSize)) {
				pFound = pItem;
				nSize  = nMatch;
				bLower = FALSE;
			}
			else {
				nMatch = Match(sItem.lower(), sLCLine);
				if(nMatch > nSize) {
					pFound = pItem;
					nSize  = nMatch;
					bLower = TRUE;
				}
			}
			pItem = pItem->nextSibling();
		}
	}

	if(pFound != NULL) {
		QListViewItem* pNext = pFound->nextSibling();
		m_nBeginIndex = sLine.length() - nSize;
		if(pNext == NULL || (!bLower && Match(pNext->text(0), sLine) < nSize)
			|| (bLower && Match(pNext->text(0).lower(), sLCLine) < nSize)) {
			OnInsertItem(pFound);
			return;
		}

	}
	if(pFound == NULL) {
		if(sLine.isEmpty()) {
			m_nBeginIndex = 0;
			pFound = m_pKeywords->firstChild();
		}
		else if(sLine.length() == 1 || IsStart(sLine, sLine.length() - 2)) {
			m_nBeginIndex = sLine.length() - 1;
			QChar c = sLine[m_nBeginIndex];
			pFound = m_pKeywords->firstChild();
			while(pFound != NULL) {
				if(pFound->text(0).startsWith(c)) {
					break;
				}
				pFound = pFound->nextSibling();
			}
		}
	}
	if(pFound != NULL) {
		SelectItem(pFound);
		m_pMenu->popup(mapToGlobal(p));
		m_pKeywords->setFocus();
	}
}

//-----------------------------------------------------------------------------------
void CTextEditor::SetSyntax(CSyntaxHighlighter* pSyntax)
{
	m_pKeywords->clear();
	if(pSyntax != NULL) {
		CSyntaxProperties* pProperties = pSyntax->GetProperties();
		if(pProperties != NULL) {
			QStringList lKeywords;
			pProperties->GetKeywords(lKeywords);
			if(!lKeywords.isEmpty()) {
				m_pKeywords->setFont(pSyntax->GetFont());
				QStringList::iterator it  = lKeywords.begin();
				QStringList::iterator end = lKeywords.end();
				for(; it != end; it++) {
					new QListViewItem(m_pKeywords, *it);
				}
			}
		}
	}
}

//-----------------------------------------------------------------------------------
void CTextEditor::OnInsertItem(QListViewItem* pItem)
{
	if(pItem != NULL) {
		if(m_nBeginIndex != -1) {
			int para, index;
			getCursorPosition(&para, &index);
			setSelection(para, m_nBeginIndex, para, index);
			removeSelectedText();
			insertAt(pItem->text(0), para, m_nBeginIndex);
			setCursorPosition(para, m_nBeginIndex + pItem->text(0).length());
		}
		else {
			insert(pItem->text(0));
		}
		m_pMenu->hide();
	}
}
