#include "converter.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qcolordialog.h>

#include "config.h"
#include "config_f100.h"

ConvImageLbl::ConvImageLbl(QWidget* parent, QString imageFile, NivCouleurs &niv, QRgb masque, int mode) 
	: QWidget(parent,"ConvImageLbl",WNoAutoErase)
{
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

	m_imageFile = imageFile;
	QImage img(imageFile);
	m_posImg.setSize( img.size() );
	m_bpl = ((img.width()*8+31)/32)*4;	// bytes per scanline
	m_imgData = new uchar[m_bpl*img.height()];
	m_imgPal = new QRgb[256];
	m_zoom=100;

	createFromImage(img, niv, masque, mode);
}

///////////////////////////////////////////////////////////////////////////////////////

ConvImageLbl::~ConvImageLbl() 
{
	delete[] m_imgData;
	delete[] m_imgPal;
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::createFromImage(QImage &img, NivCouleurs &niv, QRgb masque, int mode)
{
	QRgb noir = qRgb(0,0,0);
	int l=img.width(), h=img.height();

	m_img.reset();
	m_img = QImage(m_imgData, l, h, 8, m_imgPal, 256, QImage::IgnoreEndian);

	// creation des donnees de l'image
	uchar* ptrData = m_imgData;
	uint suite = m_bpl-l;	// espace libre entre 2 lignes de l'image
	for(int i=0; i<h; i++) 
	{
		for(int j=0; j<l; j++)  
		{
			QRgb coul = img.pixel(j,i);
			uchar index;
			if(coul==masque) index=0;
			else if(coul==noir) index=1;
			else index = (uchar) qGray(coul);
			*(ptrData++) = index;
		}
		ptrData+=suite;	// debut de la ligne suivante
	}

	createPalette(niv,masque,mode);

	m_mode = mode;
	m_masque = masque;
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::createPalette(NivCouleurs &niv, QRgb masque, int mode)
{
		// creation de la palette
	m_imgPal[0] = masque;		// masque
	short k=1;
	QRgb* ptrPal = m_imgPal+1;

	QRgb rgb = qRgb(0,0,0);
	for( ; k<niv.noir; k++) *(ptrPal++) = rgb;		// noir

	if(mode==MODE_5)
	{
		rgb = qRgb(64,64,64);
		for( ; k<niv.grisFonce; k++) *(ptrPal++) = rgb;		// gris fonce
	}

	rgb = qRgb(128,128,128);
	for( ; k<niv.grisMoyen; k++) *(ptrPal++) = rgb;	// gris normal

	if(mode==MODE_5)
	{
		rgb = qRgb(192,192,192);
		for( ; k<niv.grisClair; k++) *(ptrPal++) = rgb;	// gris clair
	}

	rgb = qRgb(255,255,255);
	for( ; k<256; k++) *(ptrPal++) = rgb;	// blanc

	m_niveaux = niv;
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::getConvertedTab(Tab &dest)
{
		// table de correspondance
	uchar corresp[256];
	corresp[0] = TRANS;
	int k=1;
						for( ; k<m_niveaux.noir;	  k++) corresp[k] = NOIR;
	if(m_mode==MODE_5)  for( ; k<m_niveaux.grisFonce; k++) corresp[k] = GRIS_FONCE;
						for( ; k<m_niveaux.grisMoyen; k++) corresp[k] = GRIS;
	if(m_mode==MODE_5)  for( ; k<m_niveaux.grisClair; k++) corresp[k] = GRIS_CLAIR;
						for( ; k<256;				  k++) corresp[k] = BLANC;


	dest.resize( m_img.size() );

	int h=m_img.height(), l=m_img.width();
	uchar* ptrData = m_imgData;
	uint suite = m_bpl-l;
	for(int i=0; i<h; i++) 
	{
		uchar* tabLine = dest[i];
		for(int j=0; j<l; j++) *(tabLine++) = corresp[ *(ptrData++) ];
		ptrData+=suite;
	}

}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::changeNiv(NivCouleurs niv)
{
	createPalette(niv, m_masque, m_mode);

	repaint(m_posImg,false);
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::changeZoom(int zoom)
{
	m_zoom = zoom;

	m_posImg.setSize( QSize(zoom*m_img.width()/100, zoom*m_img.height()/100 ));
	m_posImg.moveCenter(rect().center());

	repaint(false);
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::changeMask(QRgb masque)
{
	QImage img(m_imageFile);
	createFromImage(img,m_niveaux,masque,m_mode);
	repaint(m_posImg,false);
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::changeMode(int mode)
{
	m_mode = mode;

	createPalette(m_niveaux, m_masque, m_mode);
	repaint(m_posImg,false);
}

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::paintEvent(QPaintEvent* e)
{
	QRect r = e->rect();
	QPixmap pix(r.size());
	pix.fill(colorGroup().background());
	if(r.intersects(m_posImg))
	{
		QPainter p(&pix);
		p.translate(m_posImg.left()-r.left(), m_posImg.top()-r.top());
		p.scale(m_zoom/100.0, m_zoom/100.0);
		p.drawImage(QPoint(0,0),m_img);
	}
	bitBlt(this,r.topLeft(),&pix);
}	

///////////////////////////////////////////////////////////////////////////////////////

void ConvImageLbl::resizeEvent(QResizeEvent* e)
{
	m_posImg.moveCenter(QPoint(e->size().width()/2,e->size().height()/2));
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

OrigImageLbl::OrigImageLbl(QWidget* parent, QString imageFile)
	: QWidget(parent,"OrigImageLbl",WNoAutoErase)
	, m_img(imageFile)
	, m_zoom(100)
{
	setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

	m_posImg.setSize(m_img.size());
	m_posImg.moveCenter(rect().center());
}

///////////////////////////////////////////////////////////////////////////////////////

void OrigImageLbl::changeZoom(int zoom)
{
	m_zoom = zoom;

	m_posImg.setSize( QSize(zoom*m_img.width()/100, zoom*m_img.height()/100 ));
	m_posImg.moveCenter(rect().center());

	repaint(false);
}

///////////////////////////////////////////////////////////////////////////////////////

void OrigImageLbl::paintEvent(QPaintEvent* e)
{
	QRect r = e->rect();
	QPixmap pix(r.size());
	pix.fill(colorGroup().background());
	if(r.intersects(m_posImg))
	{
		QPainter p(&pix);
		p.translate(m_posImg.left()-r.left(), m_posImg.top()-r.top());
		p.scale(m_zoom/100.0, m_zoom/100.0);
		p.drawImage(QPoint(0,0),m_img);
	}
	bitBlt(this,r.topLeft(),&pix);
}	

///////////////////////////////////////////////////////////////////////////////////////

void OrigImageLbl::resizeEvent(QResizeEvent* e)
{
	m_posImg.moveCenter(QPoint(e->size().width()/2,e->size().height()/2));
}

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

void Converter::setMode(int mode)
{
	if(mode==MODE_3)
	{
		nivGrFonceLbl->setEnabled(false);
		nivGrClairLbl->setEnabled(false);
		nivGrFonceSpin->setEnabled(false);
		nivGrClairSpin->setEnabled(false);
		nivGrClairSlider->setEnabled(false);
		nivGrFonceSlider->setEnabled(false);
	}
	else
	{
		nivGrFonceLbl->setEnabled(true);
		nivGrClairLbl->setEnabled(true);
		nivGrFonceSpin->setEnabled(true);
		nivGrClairSpin->setEnabled(true);
		nivGrClairSlider->setEnabled(true);
		nivGrFonceSlider->setEnabled(true);
	}

	m_mode = mode;
	ColorMode = mode;
	emit modeChanged(mode);
}

///////////////////////////////////////////////////////////////////////////////////////

void Converter::setZoom(int zoom)
{
	m_zoom = zoom;
	emit zoomChanged(zoom);
}

///////////////////////////////////////////////////////////////////////////////////////

void Converter::setNiveau(uchar coul, int niv)
{
	switch(coul)
	{
	case NOIR:
		if(m_niveaux.noir==niv) return;
		m_niveaux.noir = niv;
		nivNoirSpin->setValue(niv);
		nivNoirSlider->setValue(niv);
		break;

	case GRIS_FONCE:
		if(m_niveaux.grisFonce==niv) return;
		m_niveaux.grisFonce = niv;
		nivGrFonceSpin->setValue(niv);
		nivGrFonceSlider->setValue(niv);
 		break;

	case GRIS:
		if(m_niveaux.grisMoyen==niv) return;
		m_niveaux.grisMoyen = niv;
		nivGrMoySpin->setValue(niv);
		nivGrMoySlider->setValue(niv);
 		break;

	case GRIS_CLAIR:
		if(m_niveaux.grisClair==niv) return;
		m_niveaux.grisClair = niv;
		nivGrClairSpin->setValue(niv);
		nivGrClairSlider->setValue(niv);
 		break;
	}

	emit nivChanged(m_niveaux);
}

///////////////////////////////////////////////////////////////////////////////////////

void Converter::chooseMask()
{
    QColor c = QColorDialog::getColor(QColor(m_masque),this,"colordialog");
	if( ! c.isValid() ) return;
	
	m_masque = c.rgb();

	QPixmap pix(20,12);
	QPainter p(&pix);
	p.setPen(black);
	p.setBrush(c);
	p.drawRect(0,0,20,12);
	p.end();
	maskBtn->setPixmap(pix);

	imageSortie->changeMask(m_masque);
}

///////////////////////////////////////////////////////////////////////////////////////

Converter::~Converter()
{
	m_pConfig->writeEntry("Prefs", "last_niv_b",m_niveaux.noir);
	m_pConfig->writeEntry("Prefs", "last_niv_dg",m_niveaux.grisFonce);
	m_pConfig->writeEntry("Prefs", "last_niv_mg",m_niveaux.grisMoyen);
	m_pConfig->writeEntry("Prefs", "last_niv_lg",m_niveaux.grisClair);
	m_pConfig->writeEntry("Geometry", "conv_width",width());
	m_pConfig->writeEntry("Geometry", "conv_height",height());
	m_pConfig->writeEntry("Prefs", "conv_mask",(int)m_masque), 
	m_pConfig->writeEntry("Prefs", "conv_mode",m_mode&1);
}

///////////////////////////////////////////////////////////////////////////////////////

Converter::Converter( QWidget* parent, Config* pConfig, QString &imageFile )
    : QDialog( parent, "Converter", true )
	, m_pConfig(pConfig)
{
    QHBoxLayout* ConverterLayout = new QHBoxLayout( this, 11, 6, "ConverterLayout"); 

    QVBoxLayout* layout18 = new QVBoxLayout( 0, 0, 6, "layout18"); 

		// IMAGE D'ENTREE
	EntreeGrpBox = new QGroupBox(this, "EntreeGrpBox" );
    EntreeGrpBox->setColumnLayout(0, Qt::Vertical );
    EntreeGrpBox->layout()->setSpacing( 6 );
    EntreeGrpBox->layout()->setMargin( 11 );
    QVBoxLayout* EntreeGrpBoxLayout = new QVBoxLayout( EntreeGrpBox->layout() );
    EntreeGrpBoxLayout->setAlignment( Qt::AlignTop );

    imageEntree = new OrigImageLbl( EntreeGrpBox, imageFile );
    EntreeGrpBoxLayout->addWidget( imageEntree );
    layout18->addWidget( EntreeGrpBox );

		// IMAGE DE SORTIE
    SortieGrpBox = new QGroupBox(this, "SortieGrpBox" );
    SortieGrpBox->setColumnLayout(0, Qt::Vertical );
    SortieGrpBox->layout()->setSpacing( 6 );
    SortieGrpBox->layout()->setMargin( 11 );
    QVBoxLayout* SortieGrpBoxLayout = new QVBoxLayout( SortieGrpBox->layout() );
    SortieGrpBoxLayout->setAlignment( Qt::AlignCenter );

		// ZOOM
    QSpacerItem* spacer = new QSpacerItem( 31, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );

    zoomLbl = new QLabel(this, "zoomLbl" );
    zoomSpin = new QSpinBox( this, "zoomSpin" );
	zoomSpin->setRange(50, 400);
	zoomSpin->setValue(100);
	zoomSpin->setSteps(50,100);

    QSpacerItem* spacer_2 = new QSpacerItem( 20, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );

	QHBoxLayout* layout17 = new QHBoxLayout( 0, 0, 6, "layout17"); 
    layout17->addItem( spacer );
    layout17->addWidget( zoomLbl );
    layout17->addWidget( zoomSpin );
    layout17->addItem( spacer_2 );
    layout18->addLayout( layout17 );
    ConverterLayout->addLayout( layout18 );

		// SPRITE A CREER
    QVBoxLayout* nomReglLayout = new QVBoxLayout( 0, 0, 6, "nomReglLayout"); 

    QHBoxLayout* creationLayout = new QHBoxLayout( 0, 0, 6, "creationLayout"); 

    QButtonGroup* CreationBtnBox = new QButtonGroup(this, "CreationBtnBox" );
	CreationBtnBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding);
    CreationBtnBox->setColumnLayout(0, Qt::Vertical );
    CreationBtnBox->layout()->setSpacing( 6 );
    CreationBtnBox->layout()->setMargin( 11 );
	CreationBtnBox->setExclusive(true);
    QVBoxLayout* CreationBtnBoxLayout = new QVBoxLayout( CreationBtnBox->layout() );
    CreationBtnBoxLayout->setAlignment( Qt::AlignTop );

    QHBoxLayout* nomSpriteLayout = new QHBoxLayout( 0, 0, 6, "nomSpriteLayout"); 

    nomSpriteLbl = new QLabel(CreationBtnBox, "nomSpriteLbl" );
    nomSpriteLayout->addWidget( nomSpriteLbl );

    nomSpriteEdit = new QLineEdit( CreationBtnBox, "nomSpriteEdit" );
    nomSpriteLayout->addWidget( nomSpriteEdit );
    QSpacerItem* spacer_3 = new QSpacerItem( 51, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
    nomSpriteLayout->addItem( spacer_3 );
    CreationBtnBoxLayout->addLayout( nomSpriteLayout );

    coul3Btn = new QRadioButton( CreationBtnBox, "coul3Btn" );
    CreationBtnBoxLayout->addWidget( coul3Btn );

    coul5Btn = new QRadioButton( CreationBtnBox, "coul5Btn" );
    CreationBtnBoxLayout->addWidget( coul5Btn );
    creationLayout->addWidget( CreationBtnBox );

	QHBoxLayout* maskLayout = new QHBoxLayout(0,0,6,"maskLayout");
	maskLbl = new QLabel(CreationBtnBox,"maskLbl");
    maskLayout->addWidget( maskLbl );
	maskBtn = new QPushButton(CreationBtnBox,"maskBtn");
	maskBtn->setMaximumSize(40,25);
    maskLayout->addWidget( maskBtn );
    QSpacerItem* spacer_4 = new QSpacerItem( 51, 21, QSizePolicy::Expanding, QSizePolicy::Minimum );
	maskLayout->addItem(spacer_4);
    CreationBtnBoxLayout->addLayout( maskLayout );

		// BOUTONS OK ET ANNULER
	QVBoxLayout* BtnsLayout = new QVBoxLayout( 0, 0, 6, "BtnsLayout"); 

    QPushButton* okBtn = new QPushButton( "OK", this, "okBtn" );
    okBtn->setMinimumHeight( 30 );
    BtnsLayout->addWidget( okBtn );

    annulerBtn = new QPushButton( this, "annulerBtn" );
    annulerBtn->setMinimumHeight( 30 );
    BtnsLayout->addWidget( annulerBtn );
    creationLayout->addLayout( BtnsLayout );
    nomReglLayout->addLayout( creationLayout );
    ConverterLayout->addLayout( nomReglLayout );

		// REGLAGES
		// groupbox et layout
    ReglagesGrpBox = new QGroupBox( "Rglages", this, "ReglagesGrpBox" );
	ReglagesGrpBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding);
    ReglagesGrpBox->setColumnLayout(0, Qt::Vertical );
    ReglagesGrpBox->layout()->setSpacing( 6 );
    ReglagesGrpBox->layout()->setMargin( 11 );
    QGridLayout* ReglagesGrpBoxLayout = new QGridLayout( ReglagesGrpBox->layout() );
    ReglagesGrpBoxLayout->setAlignment( Qt::AlignTop );

		// noir
	nivNoirLbl = new QLabel(ReglagesGrpBox, "nivNoirLbl" );
    nivNoirSpin = new QSpinBox( ReglagesGrpBox, "nivNoirSpin" );
	nivNoirSpin->setMaximumWidth(40);
    nivNoirSpin->setMaxValue( 255 );
    nivNoirSlider = new QSlider(QSlider::Horizontal, ReglagesGrpBox, "nivNoirSlider" );
    nivNoirSlider->setMaxValue( 255 );

    ReglagesGrpBoxLayout->addWidget( nivNoirLbl,    0, 0 );
    ReglagesGrpBoxLayout->addWidget( nivNoirSpin,   0, 1 );
    ReglagesGrpBoxLayout->addWidget( nivNoirSlider, 0, 2 );

		// gris fonce
	nivGrFonceLbl = new QLabel(ReglagesGrpBox, "nivGrFonceLbl" );
    nivGrFonceSpin = new QSpinBox( ReglagesGrpBox, "nivGrFoncSpin" );
    nivGrFonceSpin->setMaximumWidth( 40 );
    nivGrFonceSpin->setMaxValue( 255 );
    nivGrFonceSlider = new QSlider( QSlider::Horizontal, ReglagesGrpBox, "nivGrFonceSlider" );
    nivGrFonceSlider->setMaxValue( 255 );

	ReglagesGrpBoxLayout->addWidget( nivGrFonceLbl,    1, 0 );
    ReglagesGrpBoxLayout->addWidget( nivGrFonceSpin,   1, 1 );
    ReglagesGrpBoxLayout->addWidget( nivGrFonceSlider, 1, 2 );

		// gris moyen
	nivGrMoyLbl = new QLabel( ReglagesGrpBox, "nivGrMoyLbl" );
    nivGrMoySpin = new QSpinBox( ReglagesGrpBox, "nivGrMoySpin" );
    nivGrMoySpin->setMaximumWidth( 40 );
    nivGrMoySpin->setMaxValue( 255 );
    nivGrMoySlider = new QSlider( QSlider::Horizontal, ReglagesGrpBox, "nivGrMoySlider" );
    nivGrMoySlider->setMaxValue( 255 );

    ReglagesGrpBoxLayout->addWidget( nivGrMoyLbl,    2, 0 );
    ReglagesGrpBoxLayout->addWidget( nivGrMoySpin,   2, 1 );
    ReglagesGrpBoxLayout->addWidget( nivGrMoySlider, 2, 2 );

		// gris clair
    nivGrClairLbl = new QLabel(ReglagesGrpBox, "nivGrClaitLbl" );
    nivGrClairSpin = new QSpinBox( ReglagesGrpBox, "nivGrClairSpin" );
    nivGrClairSpin->setMaximumWidth( 40 );
    nivGrClairSpin->setMaxValue( 255 );
    nivGrClairSlider = new QSlider( QSlider::Horizontal, ReglagesGrpBox, "nivGrClairSlider" );
    nivGrClairSlider->setMaxValue( 255 );

    ReglagesGrpBoxLayout->addWidget( nivGrClairLbl,    3, 0 );
    ReglagesGrpBoxLayout->addWidget( nivGrClairSpin,   3, 1 );
    ReglagesGrpBoxLayout->addWidget( nivGrClairSlider, 3, 2 );

    nomReglLayout->addWidget( ReglagesGrpBox );


		// CONNEXIONS
	connect(CreationBtnBox,	  SIGNAL(clicked(int)),		 this, SLOT(setMode(int)));
	connect(zoomSpin,		  SIGNAL(valueChanged(int)), this, SLOT(setZoom(int)));
	connect(okBtn,			  SIGNAL(clicked()),		 this, SLOT(accept()));
	connect(annulerBtn,		  SIGNAL(clicked()),		 this, SLOT(reject()));
	connect(maskBtn,		  SIGNAL(clicked()),		 this, SLOT(chooseMask()));
	connect(nivGrClairSlider, SIGNAL(valueChanged(int)), this, SLOT(setLGNiv(int)));
	connect(nivGrMoySlider,	  SIGNAL(valueChanged(int)), this, SLOT(setMGNiv(int)));
	connect(nivGrFonceSlider, SIGNAL(valueChanged(int)), this, SLOT(setFGNiv(int)));
	connect(nivNoirSlider,	  SIGNAL(valueChanged(int)), this, SLOT(setBNiv(int)));
	connect(nivGrClairSpin,   SIGNAL(valueChanged(int)), this, SLOT(setLGNiv(int)));
	connect(nivGrMoySpin,	  SIGNAL(valueChanged(int)), this, SLOT(setMGNiv(int)));
	connect(nivGrFonceSpin,   SIGNAL(valueChanged(int)), this, SLOT(setFGNiv(int)));
	connect(nivNoirSpin,	  SIGNAL(valueChanged(int)), this, SLOT(setBNiv(int)));

	// INITIALISATIONS
    setNiveau( NOIR,		m_pConfig->readEntry("Prefs", "last_niv_b",51) );
    setNiveau( GRIS_FONCE,	m_pConfig->readEntry("Prefs", "last_niv_dg",102) );
    setNiveau( GRIS,		m_pConfig->readEntry("Prefs", "last_niv_mg",153) );
    setNiveau( GRIS_CLAIR,	m_pConfig->readEntry("Prefs", "last_niv_lg",204) );
    resize(	m_pConfig->readEntry("Geometry", "conv_width",450), 
			m_pConfig->readEntry("Geometry", "conv_height",270) );
	m_masque = m_pConfig->readEntry("Prefs", "conv_mask",qRgb(255,255,0)), 
	m_zoom = 100; 
	m_imgFile = imageFile;

	QPixmap pix(20,12);
	QPainter p(&pix);
	p.setPen(black);
	p.setBrush(QColor(m_masque));
	p.drawRect(0,0,20,12);
	p.end();
	maskBtn->setPixmap(pix);

	if(ColorMode!=NO_MODE)
	{
		CreationBtnBox->setButton(ColorMode);
		setMode( ColorMode );
		if(m_mode==MODE_5) coul3Btn->setDisabled(true);
	}
	else 
	{
		int m = m_pConfig->readEntry("Prefs", "conv_mode",MODE_3);
		setMode( m );
		CreationBtnBox->setButton(m);
	}

    clearWState( WState_Polished );
	nomSpriteEdit->setText( QFileInfo(imageFile).baseName().replace(' ','_') );
	setSizeGripEnabled(true);

		// IMAGE DE SORTIE
	imageSortie = new ConvImageLbl( SortieGrpBox, imageFile, m_niveaux, m_masque, m_mode );
	SortieGrpBoxLayout->addWidget( imageSortie );
    layout18->addWidget( SortieGrpBox );

	connect(this, SIGNAL(nivChanged(NivCouleurs)),	imageSortie, SLOT(changeNiv(NivCouleurs)));
	connect(this, SIGNAL(zoomChanged(int)),			imageSortie, SLOT(changeZoom(int)));
	connect(this, SIGNAL(zoomChanged(int)),			imageEntree, SLOT(changeZoom(int)));
	connect(this, SIGNAL(modeChanged(int)),			imageSortie, SLOT(changeMode(int)));

	languageChange();
	//emit nivChanged(m_niveaux);
}

///////////////////////////////////////////////////////////////////////////////////////

void Converter::languageChange()
{
    setCaption( QString(tr("Conversion - %1","Caption of the window")).arg(m_imgFile) );

	EntreeGrpBox->setTitle(tr("Input", "Label of the \'Input\' picture"));
    SortieGrpBox->setTitle(tr("Output", "Label of the \'Output\' picture"));

    zoomLbl->setText(tr("Zoom: (%)"));
	nomSpriteLbl->setText(tr("Name of the Sprite:"));

    coul3Btn->setText(tr("3 colors (Normal/DrawLib)", "1st option"));
    coul5Btn->setText(tr("5 colors (gxLib)", "2nd option"));

	maskLbl->setText(tr("Mask color:"));
	annulerBtn->setText(tr("Cancel", "Title of the \'Cancel\' button"));
    ReglagesGrpBox->setTitle(tr("Settings", "Title of the \'Settings\' box"));

	nivNoirLbl->setText(tr("Black"));
	nivGrFonceLbl->setText(tr("Dark Gray"));
	nivGrMoyLbl->setText(tr("Normal Gray"));
    nivGrClairLbl->setText(tr("Light Gray"));
}

///////////////////////////////////////////////////////////////////////////////////////


