
#ifndef ICOMDEVICE_H
#define ICOMDEVICE_H

/*!
	\class IComDevice IComDevice.h
	\brief Permet de transmettre des donnes  un priphrique de communication.
	\author Quaranta David
	\version 1.00
	
	\ingroup include

	Pour permettre le transfert de donnes sur un priphrique particulier (USB, port COM, LPT, ...),
	vous dever crer une classe qui drive de cette interface.
*/
class IComDevice
{
public:
	/** Liste d'erreur possible retourne par GetLastError() */
	enum ERRORS
	{
		ER_OPENPORT = 0, /*!< Impossible d'ouvrir le port. */  
		ER_GETPORT,      /*!< Impossible d'accder au port en lecture. */
		ER_SETPORT,      /*!< Impossible d'accder au port en criture. */
		ER_OK,           /*!< Pas d'erreur. */
	};

	/** Ouvre le port du priphrique
	 * @param[in] nSpeed Vitesse de communication en bit/s
	 * @return Retourne non-zro pour OK
	 * @pre A appeler aprs Init(int , int)
	 * @see Init(int , int)
	 * @see ClosePort()
	 */
	virtual int          OpenPort(int nSpeed) = 0;

	/** Ferme le port du priphrique
	 * @see OpenPort(int)
	 */
	virtual void         ClosePort() = 0;

	/** Initialise le priphrique
	 * @param[in] nPort Numro du port souhait pour le transfert
	 * @param[in] nCable Numro du cble souhait pour le transfert
	 */
	virtual void         Init(int nPort, int nCable) = 0;

	/** Envoi des donnes
	 * @param[in] buffer Donnes  envoyer
	 * @param[in] nNbBytes Nombre d'octets  envoyer
	 * @param[in] nTimeOut Temps en milli-seconde indiquant le temps maximum d'attente entre deux octets envoys. Si la valeur
	 * est infrieur  0, alors le temps n'est pas modifi (l'ancienne valeur est prise en compte)
	 * @return Retourne non-zro pour OK
	 * @see Receive(void*, unsigned int, int)
	 */
	virtual int          Send(const void* buffer, unsigned int nNbBytes, int nTimeOut = -1) = 0;

	/** Rception de donnes
	 * @param[out] buffer Zone mmoire pour recevoir les donnes
	 * @param[in] nNbBytes Nombre d'octets  recevoir
	 * @param[in] nTimeOut Temps en milli-seconde indiquant le temps maximum d'attente entre deux octets envoys. Si la valeur
	 * est infrieur  0, alors le temps n'est pas modifi (l'ancienne valeur est prise en compte)
	 * @return Retourne non-zro pour OK
	 * @see Send(const void*, unsigned int, int)
	 */
	virtual int          Receive(void* buffer, unsigned int nNbBytes, int nTimeOut = -1) = 0;

	/** Retourne la dernire erreur */
	virtual ERRORS       GetLastError() const = 0;

	/** Retourne la liste des cbles disponibles
	 * @param[out] nNbCable Reoit le nombre de cble disponible
	 */
	virtual const char** GetCables(int& nNbCable) const = 0;

	/** Retourne le nombre de port disponible
	 * @see GetPortName(int, char*, int) const
	 */
	virtual int          GetNbPort() const = 0;

	/** Permet de rcuprer le nom d'un port
	 * @param[in] nPort Numro du port
	 * @param[out] pBuffer Tampon mmoire qui va recevoir le nom du port
	 * @param[in] nBufferSize Taille du tampon mmoire
	 * @see GetNbPort()
	 */
	virtual void         GetPortName(int nPort, char* pBuffer, int nBufferSize) const = 0;

	/** Retourne la vitesse maximale en bit/s */
	virtual int          GetMaxSpeed() const = 0;
};

#endif
