
#ifndef IMAINWINDOW_H
#define IMAINWINDOW_H

#include <qstringlist.h>

class CToolWindow;
class IOptionWindow;
class CSendToMenu;

/*!
	\class IMainWindow IMainWindow.h
	\brief Interface de la fentre principale
	\author Quaranta David
	\version 1.01

	\ingroup include

	Permet de crer un outil (plug-in) et aussi d'accder aux fonctions d'criture/lecture dans le registre
*/
class IMainWindow
{
public:
	/** Permet de crer un outil depuis un type d'outil
	 * @param[in] sType Type de l'diteur dsir
	 * @param[in] sFileName Fichier  ouvrir (ou QString::null si aucun)
	 * @param[in] nStartBytes Dbut (en octet) du fichier  lire s'il y a un dcalage
	 * @param[in] bShowMsgError A vrai pour afficher une boite de dialogue signalant l'erreur produite, sinon  faux pour rien afficher
	 * @param[in] pParent Fentre parente. Si NULL cela implique que la fentre est gre par Flash100.
	 * @param[in] pExtraParam Paramtres optionnels, peut servir suivant les outils
	 * @return Retourne l'outil correspondant, sinon retourne NULL si pas trouv.
	 * @see IsSupported(const QString&)
	 */
	virtual CToolWindow* OpenToolByType(const QString& sType, const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL, void* pExtraParam = NULL) = 0;

	/** Permet de crer un outil depuis une extension de fichier
	 * @param[in] sSupportedFile Extension(s) que l'outil doit ouvrir
	 * @param[in] sFileName Fichier  ouvrir (ou QString::null si aucun)
	 * @param[in] nStartBytes Dbut (en octet) du fichier  lire s'il y a un dcalage
	 * @param[in] bShowMsgError A vrai pour afficher une boite de dialogue signalant l'erreur produite, sinon  faux pour rien afficher
	 * @param[in] pParent Fentre parente. Si NULL cela implique que la fentre est gre par Flash100.
	 * @return Retourne l'outil correspondant, sinon retourne NULL si pas trouv.
	 * @see IsLoaded(const QString&)
	 */
	virtual CToolWindow* OpenTool(const QString& sSupportedFile, const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL) = 0;

	/** Ouvrir un lecteur de la flash
	 * @param[in] sFileName Fichier  ouvrir (ou QString::null si aucun)
	 * @param[in] nStartBytes Dbut (en octet) du fichier  lire s'il y a un dcalage
	 * @param[in] bShowMsgError A vrai pour afficher une boite de dialogue signalant l'erreur produite, sinon  faux pour rien afficher
	 * @param[in] pParent Fentre parente. Si NULL cela implique que la fentre est gre par Flash100.
	 * @return Retourne l'outil correspondant, sinon retourne NULL si pas trouv.
	 */
	virtual CToolWindow* OpenFileDrive(const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL) = 0;

	/** Ouvrir un fichier systme
	 * @param[in] sFileName Fichier  ouvrir (ou QString::null si aucun)
	 * @param[in] nStartBytes Dbut (en octet) du fichier  lire s'il y a un dcalage
	 * @param[in] bShowMsgError A vrai pour afficher une boite de dialogue signalant l'erreur produite, sinon  faux pour rien afficher
	 * @param[in] pParent Fentre parente. Si NULL cela implique que la fentre est gre par Flash100.
	 * @return Retourne l'outil correspondant, sinon retourne NULL si pas trouv.
	 */
	virtual CToolWindow* OpenFileSystem(const QString& sFileName, int nStartBytes = 0, bool bShowMsgError = TRUE, QWidget* pParent = NULL) = 0;

	/** Permet de savoir si un outil dont le type est connu est charg
	 * @param[in] sToolType Type de l'diteur.
	 * @return Retourne Vrai si le type de l'outil spcifi est bien charg
	 */
	virtual bool         IsLoaded(const QString& sToolType) = 0;
	
	/** Permet de savoir si un outil charg peut ouvrir un type de fichier
	 * @param[in] sFileExt Extension(s)  ouvrir (Extensions spares par un point virgule si plusieurs)
	 * @return Retourne Vrai si cette extension de fichier est bien support par les outils chargs
	 * @warning Ceci n'implique pas que les outils chargs puissent ouvrir tous les fichiers d'une extension
	 */
	virtual bool         IsSupported(const QString& sFileExt) = 0;

	/** Permet de rcuprer la liste des extensions supportes
	 * @return Retourne la liste des extensions supportes sous forme d'une chane (ex : "lec;mems;prj")
	 */
	virtual QString      GetSupportedFiles() = 0;

	/** Ouvrir une liste de fichiers
	 * @param[in] lFiles Liste de fichiers  ouvrir
	 * @param[in] bDoLoadOnlyProject A vrai, si l'on veut charger que le projet (les autres fichiers sont ignors)
	 * @return Retourne le nombre de fichiers ouverts.
	 */
	virtual int          LoadFiles(QStringList& lFiles, bool bDoLoadOnlyProject = FALSE) = 0;

	/** @return La liste des outils chargs dans une liste de String contenant les types des diteurs */
	virtual QStringList  GetToolNames() const = 0;

	/** Permet de rcuprer des informations sur un type d'diteur
	 * @param[in] sToolType Type de l'diteur
	 * @param[out] sToolName Nom de l'outil
	 * @param[out] sDesc Description de l'outil
	 * @param[out] sFileExt Extensions ouvertes par cet outil
	 * @param[out] sAuthor Auteur(s) de l'outil
	 * @param[out] sVersion Version de l'outil
	 * @param[out] sBuildDate Date de cration de l'outil
	 */
	 virtual bool        GetToolInfos(const QString& sToolType, QString& sToolName, QString& sDesc, QString& sFileExt, QString& sAuthor, QString& sVersion, QString& sBuildDate) const = 0;

	/** Permet d'crire des donnes dans le registre
	 * @param[in] sPath Chemin relatif
	 * @param[in] nNumber Nombre  sauver
	 * @return Vrai si la sauvegarde a russi
	 * @remarks La sauvegarde des donnes se fera depuis le chemin de rfrence de Flash100
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour sauver des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual bool         WriteSettings(const QString& sPath, const int& nNumber)   = 0;

	/** Permet d'crire des donnes dans le registre
	 * @param[in] sPath Chemin relatif
	 * @param[in] sText Texte  sauver
	 * @return Vrai si la sauvegarde a russi
	 * @remarks La sauvegarde des donnes se fera depuis le chemin de rfrence de Flash100
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour sauver des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual bool         WriteSettings(const QString& sPath, const QString& sText) = 0;

	/** Permet de lire des donnes dans le registre
	 * @param[in] sPath Chemin relatif
	 * @param[out] nNumber Nombre lu
	 * @param[in] nDefault Valeur par dfaut retourne dans le nombre si la cl n'existe pas
	 * @return Vrai si le chargement a russi
	 * @remarks Le chargement des donnes se fera depuis le chemin de rfrence de Flash100
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour charger des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual bool         ReadSettings(const QString& sPath, int& nNumber, const int& nDefault = 0) const = 0;

	/** Permet de lire des donnes dans le registre
	 * @param[in] sPath Chemin relatif
	 * @param[out] sText Texte lu
	 * @param[in] sDefault Texte par dfaut retourn dans le texte lu si la cl n'existe pas
	 * @return Vrai si le chargement a russi
	 * @remarks Le chargement des donnes se fera depuis le chemin de rfrence de Flash100
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour charger des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual bool         ReadSettings(const QString& sPath, QString& sText, const QString& sDefault = QString::null) const = 0;

	/** Permet de lire les entres d'un rpertoire
	 * @param[in] sPath Chemin relatif
	 * @return La liste des sous-cls du dossier
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour charger des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual QStringList  SettingsList(const QString& sPath) const = 0;

	/** Permet de lire les rpertoires d'un rpertoire
	 * @param[in] sPath Chemin relatif
	 * @return La liste des sous-rpertoires du dossier
	 * @remarks Sous Windows les donnes sont stockes dans la base de registre, sous Linux dans un fichier
	 * @remarks Pour charger des donnes d'un outil il est cependant recommand d'utiliser
	 * les mthodes de la classe CSettingsWidget
	 */
	virtual QStringList  SubSettingsList(const QString& sPath) const = 0;
	
	/** Permet de supprimer un cl et ses sous-cls
	 * @param[in] sPath Chemin de la cl  supprimer
	 * @param[in] bRemoveSubKeys A vrai pour supprimer toutes les sous-cls,  faux pour ne supprimer que les lments de cette cl.
	 * @return Vrai si la cl a pu tre supprime, sinon faux.
	 * @remarks Si bRemoveSubKeys est  faux, le retour de la mthode sera  vrai uniquement si aucune sous-cl tait prsente
	 */
	virtual bool         RemoveSettings(const QString& sPath, bool bRemoveSubKeys = TRUE) = 0;

	/** Permet de redmarrer l'application en cours
	 * @return Retourne vrai si russi, sinon faux
	 */
	virtual bool         ReStartApplication() = 0;

	/** Permet de stopper l'application en cours
	 * @param[in] bForce A vrai pour forcer l'arrt, sinon la fermeture sera douce.
	 * @return Retourne vrai si russi, sinon faux
	 */
	virtual bool         CloseApplication(bool bForce = FALSE) = 0;

	/** @return La fentre des options associes  l'application */
	virtual IOptionWindow* GetOptionWindow() = 0;

	/** @return Le menu 'Envoyer vers' de l'application */
	virtual CSendToMenu*   GetSendToMenu() = 0;
};

#endif // IMAINWINDOW_H
