
#ifndef IPROTOCOL_H
#define IPROTOCOL_H

#include <IComDevice.h>
#include <casio.h>

using namespace Casio;

#ifndef NULL
#	define NULL 0
#endif

/*!
	\class IProtocol IProtocol.h
	\brief Interface pour grer les protocoles de communication
	\author Quaranta David
	\version 1.00
	
	\ingroup include

	Pour crer crer un protocole, vous devez absolument crer une classe qui drive de IProtocol
*/

/*!
	\page MemZone MemZone
	\section sec_memzone MemZone
	MemZone est une partie du programme Touche de 2072 permettant de rcuprer les donnes de la RAM de la Graph100. (fichiers BASIC, Listes, Matrices, etc...) \n
	Flash100 utilise ce systme pour le transfert de donnes vers une Graph100.
	\subsection sub_val_mem_zone Valeurs possibles
	 0 : Variables (A~Z) \n
	 1 : Fichiers BASIC \n
	 2 : Groupe (Fonctions 'Yxx') \n
	 3 : Systme \n
	 4 : Char \n
	 5 : Listes \n
	 6 : Matrices \n
	 7 : Images \n
	 8 : X-lect \n
	 9 : Local \n
	10 : DT 8 \n
	11 : DT 9 \n
	12 : Add-In
*/

class IProtocol
{
public:
	/** Mode de transfert disponible */
	enum TRANSFERT
	{
		TR_NONE = 0,   /*!< Aucun. */
		TR_FLASH,      /*!< Envoi ou rception de donnes de la Flash. */
		TR_RECEIVERAM, /*!< Rception de donnes de la RAM. */
		TR_SENDRAM,    /*!< Envoi de donnes en RAM.*/
	};
	/** Liste d'erreur possible retourne par GetLastError() */
	enum ERRORS
	{
		OPENPORT = IComDevice::ER_OPENPORT, /*!< Impossible d'ouvrir le port. */
		GETPORT  = IComDevice::ER_GETPORT,  /*!< Impossible d'accder au port en lecture. */
		SETPORT  = IComDevice::ER_SETPORT,  /*!< Impossible d'accder au port en criture. */
		OK       = IComDevice::ER_OK,       /*!< Pas d'erreur. */
		TIMEOUT,                            /*!< Temps d'attente dpass. */
		NOINIT,                             /*!< L'initialisation a chou. */
		NOSEND,                             /*!< Message pas entirement envoy. */
		NOSUPPORTED,                        /*!< Fonction non supporte. */
		INVALID_VALUE,                      /*!< La valeur reue est invalide. */
		BAD_CALL,                           /*!< Mauvais appel de fonction (ex:argument incorrect). */
		CANT_CHANGE_TRANSFERT_MODE,         /*!< Le changement de mode de transfert n'est pas autoris par ce protocole. */
	};

	/** Ouvre la communication
	 * @param[in] pComDevice Priphrique de transfert
	 * @param[in] nSpeed Index de la vitesse supporte par ce protocole
	 * @return Retourne non-zro si Ok
	 * @see GetAvailableSpeeds()
	 * @see CloseCom(int)
	 */
	virtual int InitCom(IComDevice* pComDevice, int nSpeed) = 0;
	
	/** Ferme la communication
	 * @param[in] nStatus Valeur d'arrt de la communication. Si suprieur  0, signale que la communication peut tre termine correctement,
	 * sinon la communication sera ferme sans prvenir l'autre interlocuteur.
	 * @see InitCom(IComDevice*, int)
	 */
	virtual void CloseCom(int nStatus) = 0;

	/** Defini le type de transfert
	 * @param[in] nTransfert Nouveau mode de transfert
	 * @return Retourne 1 si ok, ou -1 pour erreur (ie. si le protocole ne supporte pas les changements de mode par exemple)
	 * @remarks Peut tre sans effet sur certains protocoles
	 * @see GetTransfertMode() const
	 */
	virtual int SetTransfertMode(TRANSFERT nTransfert) = 0;

	/** Permet de rcuprer le type de transfert actuel
	 * @return Retourne le type de transfert en cours. Retourne IProtocol::TR_NONE si aucun mode de transfert n'a t dfini.
	 * @see SetTransfertMode(TRANSFERT)
	 */
	virtual TRANSFERT GetTransfertMode() const = 0;
	
	/** Demande l'importation d'un lecteur
	 * @param[in] nLect Numro du lecteur dont on veut les donnes (de 0  6)
	 * @return Retourne >= 0 pour Ok (taille de l'importation en octet), -1 erreur, -2 non support.
	 */
	virtual int ImportDrive(int nLect) = 0;
	
	/** Demande l'exportation d'un lecteur
	 * @param[in] nLect Numro du lecteur qui va recevoir les donnes (de 0  6)
	 * @param[in] pBuffer Donnes  envoyer
	 * @return Retourne 1 pour Ok, -1 erreur
	 */
	virtual int ExportDrive(int nLect, const void* pBuffer = NULL) = 0;

	/** Annonce l'envoi des donnes RAM (Fichier BASIC, listes, ...)
	 * @param[in] nZone Zone du fichier  envoyer (de 0  12). Voir \ref MemZone
	 * @param[in] name Nom du fichier
	 * @param[in] nSize Taille du fichier 
	 * @param[in] password Si non nul et \a nZone == 1 alors dfini le mode de passe
	 * @return Retourne > 0 pour OK, -1 erreur
	 * @remarks Suivant les protocoles l'envoi du mot de passe peut se faire soit
	 * dans cette mthode soit dans WaitRAMFileConfirmation(unsigned short, const char*)
	 * @see \ref MemZone
	 * @post WaitRAMFileConfirmation(unsigned short, const char*) doit tre appele pour savoir si le fichier est accpet
	 */
	virtual int SendRAMFile(unsigned short nZone, const char* name, unsigned int nSize, const char* password = NULL) = 0;

	/** Attent la confirmation de l'envoi d'un fichier
	 * @param[in] nZone Zone du fichier  envoyer (de 0  12). Voir \ref MemZone
	 * @param[in] password Si non nul et \a nZone == 1 alors dfini le mode de passe
	 * @return Retourne 0 en cas de non reception de la confirmation \n
	 *			-1 en cas d'annulation de l'utilisateur \n
	 *			-2 en cas d'erreur \n
	 *			 1 si tout va bien
	 * @pre Appele aprs SendRAMFile(unsigned short, const char*, unsigned int, const char*)
	 * @remarks Suivant les protocoles l'envoi du mot de passe peut se faire soit
	 * dans cette mthode soit dans SendRAMFile(unsigned short, const char*, unsigned int, const char*)
	 * @see \ref MemZone
	 */
	virtual int WaitRAMFileConfirmation(unsigned short nZone, const char* password = NULL) = 0;

	/** Permet de recevoir donnes de la RAM
	 * @param[in] nZone Zone du fichier  envoyer (de 0  12). Voir \ref MemZone
	 * @param[out] name Nom du fichier
	 * @param[out] password Mot de passe (si fichier basic).
	 * @return Retourne >= 0 pour OK (taille en octets des donnes  lire) \n
	 * -1 erreur \n
	 * -2 fini \n
	 * -3 rien de recu
	 * @remarks Suivant le protocole le mot de passe peut tre reu uniquement dans SendRAMFileConfirmation(bool, unsigned short, char*)
	 * @post SendRAMFileConfirmation(bool, unsigned short, char*) doit tre appele pour confirmer la rception ou non
	 * @see \ref MemZone
	 */
	virtual int ReceiveRAMFile(unsigned short& nZone, char* name, char* password = NULL) = 0;

	/** Valide ou non, la reception d'un fichier de la RAM
	 * @param[in] bOk A vrai si on valide la confirmation
	 * @param[in] nZone Zone du fichier  envoyer (de 0  12). Voir \ref MemZone
	 * @param[in] password Mot de passe (si fichier basic).
	 * @return Retourne > 0 pour OK, < 1 erreur
	 * @remarks Suivant le protocole le mot de passe peut tre reu uniquement dans ReceiveRAMFile(unsigned short, char*, char*)
	 * @pre Appele aprs ReceiveRAMFile(unsigned short, char*, char*)
	 * @see \ref MemZone
	 */
	virtual int SendRAMFileConfirmation(bool bOk, unsigned short nZone, char* password = NULL) = 0;

	/** Permet de rcuprer la dernire erreur
	 * @return Retourne la dernire erreur
	 */
	virtual ERRORS GetLastError() const = 0;
	
	/** Permet de savoir la taille utilise d'un buffer
	 * @param[in] pBuffer Zone mmoire  analyser
	 * @return Retourne la taille utile  envoyer pour \a pBuffer
	 */
	virtual unsigned int GetUsefulSize(const void* pBuffer) = 0;

	/** Permet de rcuprer les vitesses disponibles
	 * @param[out] nNumber Rcupre le nombre de vitesses supportes
	 * @return Retourne les vitesses supportes par ce protocole
	 */
	virtual int* GetAvailableSpeeds(int& nNumber) const = 0;
	
	/** Permet de connaitre la taille totale des lecteurs pour la rception
	 * @param[in] bSystem A vrai indique que le systme est pris en compte
	 * @param[in] bDriveL A vrai indique que le lecteur L est pris en compte
	 * @param[in] bDriveM A vrai indique que le lecteur M est pris en compte
	 * @param[in] bDriveN A vrai indique que le lecteur N est pris en compte
	 * @param[in] bDriveO A vrai indique que le lecteur O est pris en compte
	 * @param[in] bDriveP A vrai indique que le lecteur P est pris en compte
	 * @param[in] bDriveQ A vrai indique que le lecteur Q est pris en compte
	 * @return Retourne la taille totale en octets que prendrait la rception des lecteurs de la FLASH
	 * @remarks Sur certain protocole cela peut toujours tre constant, tandis que sur d'autre la demande
	 * est faite au receveur et donc la valeur peut varier selon les cas.
	 */
	virtual unsigned int GetSizeOfDrives(bool bSystem, bool bDriveL, bool bDriveM, bool bDriveN,
			bool bDriveO, bool bDriveP, bool bDriveQ) = 0;

	/** Rception de donnees aprs une demande de Flash ou RAM
	 * @param[out] buffer Tampon mmoire pour recevoir les donnes
	 * @param[in] nRemainBytes Octets restant
	 * @return Retourne une valeur ngative pour une erreur, sinon la taille reue
	 */
	virtual int ReceiveData(void* buffer, unsigned int nRemainBytes) = 0;

	/** Envoi de donnees aprs une demande de Flash ou RAM
	 * @param[in] buffer Donnes mmoire  envoyer
	 * @param[in] nRemainBytes Octets restant
	 * @return Retourne une valeur ngative pour une erreur, sinon la taille envoye
	 */
	virtual int SendData(const void* buffer, unsigned int nRemainBytes) = 0;

	/** @return Retourne le modle de calculatrice gr */
	virtual CasioType GetCasioType() const = 0;
};

#endif
